@@ -80,6 +80,9 @@
 /* Define as 1 if your zlib has gzseek() */
 #undef HAVE_GZSEEK
 
+/* Define as 1 if your va_list type is an array */
+#undef HAVE_VA_LIST_AS_ARRAY
+
 /* Define to the full path name of the bzip2 library (libbz2.a) */
 #undef BZIP2LIB
 
@@ -132,15 +135,6 @@
 /* define if the statfs() call takes 4 arguments */
 #undef STAT_STATFS4
 
-/* Absolute path to rpm top_sourcedir. */
-#undef TOP_SOURCEDIR
-
-/* define if support rpm-1.0 packages is desired */
-#undef ENABLE_V1_PACKAGES
-
-/* define if experimental support rpm-4.0 packages is desired */
-#undef ENABLE_V5_PACKAGES
-
 /* Use the included glob.c? */
 #undef USE_GNU_GLOB
 
@@ -153,8 +147,8 @@
 /* Use the Berkeley db1 retrofit to db2/db3 API? */
 #undef USE_DB1
 
-/* Use the Berkeley db1 API from glibc? */
-#undef USE_DB0
+/* Define if you have the <db3/db.h> header file.  */
+#undef HAVE_DB3_DB_H
 
 /* Build with dmalloc support? */
 #undef DMALLOC
