@@ -5,68 +5,10 @@
 #include "system.h"
 #include <rpmlib.h>
 #include <rpmmacro.h>	/* XXX for %_i18ndomains */
+#include "manifest.h"
 #include "misc.h"
 #include "debug.h"
 
-/**
- * Return ls-like formatted mode string.
- * @param mode		file mode
- * @return		formatted mode string (malloc'ed)
- */
-static /*@only@*/ char * permsString(int mode)	/*@*/
-{
-    char * perms = xmalloc(11);
-
-    strcpy(perms, "-----------");
-   
-    if (mode & S_ISVTX) perms[10] = 't';
-
-    /*@-unrecog@*/
-    if (mode & S_IRUSR) perms[1] = 'r';
-    if (mode & S_IWUSR) perms[2] = 'w';
-    if (mode & S_IXUSR) perms[3] = 'x';
- 
-    if (mode & S_IRGRP) perms[4] = 'r';
-    if (mode & S_IWGRP) perms[5] = 'w';
-    if (mode & S_IXGRP) perms[6] = 'x';
-
-    if (mode & S_IROTH) perms[7] = 'r';
-    if (mode & S_IWOTH) perms[8] = 'w';
-    if (mode & S_IXOTH) perms[9] = 'x';
-    /*@=unrecog@*/
-
-    if (mode & S_ISUID) {
-	if (mode & S_IXUSR) 
-	    perms[3] = 's'; 
-	else
-	    perms[3] = 'S'; 
-    }
-
-    if (mode & S_ISGID) {
-	if (mode & S_IXGRP) 
-	    perms[6] = 's'; 
-	else
-	    perms[6] = 'S'; 
-    }
-
-    if (S_ISDIR(mode)) 
-	perms[0] = 'd';
-    else if (S_ISLNK(mode)) {
-	perms[0] = 'l';
-    }
-    else if (S_ISFIFO(mode)) 
-	perms[0] = 'p';
-    else if (S_ISSOCK(mode)) 
-	perms[0] = 'l';
-    else if (S_ISCHR(mode)) {
-	perms[0] = 'c';
-    } else if (S_ISBLK(mode)) {
-	perms[0] = 'b';
-    }
-
-    return perms;
-}
-
 /**
  * @param type		tag type
  * @param data		tag value
@@ -113,9 +55,11 @@ static /*@only@*/ char * permsFormat(int_32 type, const void * data, char * form
     } else {
 	val = xmalloc(15 + padding);
 	strcat(formatPrefix, "s");
-	buf = permsString(*((int_32 *) data));
+	buf = rpmPermsString(*((int_32 *) data));
+	/*@-formatconst@*/
 	sprintf(val, formatPrefix, buf);
-	free(buf);
+	/*@=formatconst@*/
+	buf = _free(buf);
     }
 
     return val;
@@ -156,7 +100,9 @@ static /*@only@*/ char * fflagsFormat(int_32 type, const void * data,
 
 	val = xmalloc(5 + padding);
 	strcat(formatPrefix, "s");
+	/*@-formatconst@*/
 	sprintf(val, formatPrefix, buf);
+	/*@=formatconst@*/
     }
 
     return val;
@@ -192,7 +138,9 @@ static /*@only@*/ char * depflagsFormat(int_32 type, const void * data,
 
 	val = xmalloc(5 + padding);
 	strcat(formatPrefix, "s");
+	/*@-formatconst@*/
 	sprintf(val, formatPrefix, buf);
+	/*@=formatconst@*/
     }
 
     return val;
@@ -233,22 +181,25 @@ static int fsnamesTag( /*@unused@*/ Header h, /*@out@*/ int_32 * type,
  * @retval freedata	address of data-was-malloc'ed indicator
  * @return		0 on success
  */
-static int instprefixTag(Header h, /*@out@*/ int_32 * type,
-	/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
-	/*@out@*/ int * freeData)
-		/*@modifies h, *type, *data, *count, *freeData @*/
+static int instprefixTag(Header h, /*@null@*/ /*@out@*/ rpmTagType * type,
+	/*@null@*/ /*@out@*/ const void ** data,
+	/*@null@*/ /*@out@*/ int_32 * count,
+	/*@null@*/ /*@out@*/ int * freeData)
+		/*@modifies *type, *data, *freeData @*/
 {
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HFD_t hfd = headerFreeData;
+    rpmTagType ipt;
     char ** array;
 
-    if (headerGetEntry(h, RPMTAG_INSTALLPREFIX, type, (void **)data, count)) {
-	*freeData = 0;
+    if (hge(h, RPMTAG_INSTALLPREFIX, type, (void **)data, count)) {
+	if (freeData) *freeData = 0;
 	return 0;
-    } else if (headerGetEntry(h, RPMTAG_INSTPREFIXES, NULL, (void **) &array, 
-			      count)) {
-	*data = xstrdup(array[0]);
-	*freeData = 1;
-	*type = RPM_STRING_TYPE;
-	free(array);
+    } else if (hge(h, RPMTAG_INSTPREFIXES, &ipt, (void **) &array, count)) {
+	if (data) *data = xstrdup(array[0]);
+	if (freeData) *freeData = 1;
+	if (type) *type = RPM_STRING_TYPE;
+	array = hfd(array, ipt);
 	return 0;
     } 
 
@@ -263,18 +214,18 @@ static int instprefixTag(Header h, /*@out@*/ int_32 * type,
  * @retval freedata	address of data-was-malloc'ed indicator
  * @return		0 on success
  */
-static int fssizesTag(Header h, /*@out@*/ int_32 * type,
+static int fssizesTag(Header h, /*@out@*/ rpmTagType * type,
 	/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
 	/*@out@*/ int * freeData)
-		/*@modifies h, *type, *data, *count, *freeData @*/
+		/*@modifies *type, *data, *count, *freeData @*/
 {
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
     const char ** filenames;
     int_32 * filesizes;
     uint_32 * usages;
     int numFiles;
 
-    if (!headerGetEntry(h, RPMTAG_FILESIZES, NULL, (void **) &filesizes, 
-		       &numFiles)) {
+    if (!hge(h, RPMTAG_FILESIZES, NULL, (void **) &filesizes, &numFiles)) {
 	filesizes = NULL;
 	numFiles = 0;
 	filenames = NULL;
@@ -301,7 +252,7 @@ static int fssizesTag(Header h, /*@out@*/ int_32 * type,
 
     *data = usages;
 
-    if (filenames) free(filenames);
+    filenames = _free(filenames);
 
     return 0;
 }
@@ -314,11 +265,14 @@ static int fssizesTag(Header h, /*@out@*/ int_32 * type,
  * @retval freedata	address of data-was-malloc'ed indicator
  * @return		0 on success
  */
-static int triggercondsTag(Header h, /*@out@*/ int_32 * type,
+static int triggercondsTag(Header h, /*@out@*/ rpmTagType * type,
 	/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
 	/*@out@*/ int * freeData)
-		/*@modifies h, *type, *data, *count, *freeData @*/
+		/*@modifies *type, *data, *count, *freeData @*/
 {
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HFD_t hfd = headerFreeData;
+    rpmTagType tnt, tvt, tst;
     int_32 * indices, * flags;
     char ** names, ** versions;
     int numNames, numScripts;
@@ -328,17 +282,16 @@ static int triggercondsTag(Header h, /*@out@*/ int_32 * type,
     int i, j;
     char buf[5];
 
-    if (!headerGetEntry(h, RPMTAG_TRIGGERNAME, NULL, (void **) &names, 
-			&numNames)) {
+    if (!hge(h, RPMTAG_TRIGGERNAME, &tnt, (void **) &names, &numNames)) {
 	*freeData = 0;
 	return 0;
     }
 
-    headerGetEntry(h, RPMTAG_TRIGGERINDEX, NULL, (void **) &indices, NULL);
-    headerGetEntry(h, RPMTAG_TRIGGERFLAGS, NULL, (void **) &flags, NULL);
-    headerGetEntry(h, RPMTAG_TRIGGERVERSION, NULL, (void **) &versions, NULL);
-    headerGetEntry(h, RPMTAG_TRIGGERSCRIPTS, NULL, (void **) &s, &numScripts);
-    free(s);
+    (void) hge(h, RPMTAG_TRIGGERINDEX, NULL, (void **) &indices, NULL);
+    (void) hge(h, RPMTAG_TRIGGERFLAGS, NULL, (void **) &flags, NULL);
+    (void) hge(h, RPMTAG_TRIGGERVERSION, &tvt, (void **) &versions, NULL);
+    (void) hge(h, RPMTAG_TRIGGERSCRIPTS, &tst, (void **) &s, &numScripts);
+    s = hfd(s, tst);
 
     *freeData = 1;
     *data = conds = xmalloc(sizeof(char * ) * numScripts);
@@ -353,25 +306,24 @@ static int triggercondsTag(Header h, /*@out@*/ int_32 * type,
 	    item = xmalloc(strlen(names[j]) + strlen(versions[j]) + 20);
 	    if (flags[j] & RPMSENSE_SENSEMASK) {
 		buf[0] = '%', buf[1] = '\0';
-		flagsStr = depflagsFormat(RPM_INT32_TYPE, flags, buf,
-					  0, j);
+		flagsStr = depflagsFormat(RPM_INT32_TYPE, flags, buf, 0, j);
 		sprintf(item, "%s %s %s", names[j], flagsStr, versions[j]);
-		free(flagsStr);
+		flagsStr = _free(flagsStr);
 	    } else {
 		strcpy(item, names[j]);
 	    }
 
 	    chptr = xrealloc(chptr, strlen(chptr) + strlen(item) + 5);
-	    if (*chptr) strcat(chptr, ", ");
+	    if (*chptr != '\0') strcat(chptr, ", ");
 	    strcat(chptr, item);
-	    free(item);
+	    item = _free(item);
 	}
 
 	conds[i] = chptr;
     }
 
-    free(names);
-    free(versions);
+    names = hfd(names, tnt);
+    versions = hfd(versions, tvt);
 
     return 0;
 }
@@ -384,26 +336,28 @@ static int triggercondsTag(Header h, /*@out@*/ int_32 * type,
  * @retval freedata	address of data-was-malloc'ed indicator
  * @return		0 on success
  */
-static int triggertypeTag(Header h, /*@out@*/ int_32 * type,
+static int triggertypeTag(Header h, /*@out@*/ rpmTagType * type,
 	/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
 	/*@out@*/ int * freeData)
-		/*@modifies h, *type, *data, *count, *freeData @*/
+		/*@modifies *type, *data, *count, *freeData @*/
 {
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HFD_t hfd = headerFreeData;
+    rpmTagType tst;
     int_32 * indices, * flags;
-    char ** conds, ** s;
+    const char ** conds;
+    const char ** s;
     int i, j;
     int numScripts, numNames;
 
-    if (!headerGetEntry(h, RPMTAG_TRIGGERINDEX, NULL, (void **) &indices, 
-			&numNames)) {
+    if (!hge(h, RPMTAG_TRIGGERINDEX, NULL, (void **) &indices, &numNames)) {
 	*freeData = 0;
 	return 1;
     }
 
-    headerGetEntry(h, RPMTAG_TRIGGERFLAGS, NULL, (void **) &flags, NULL);
-
-    headerGetEntry(h, RPMTAG_TRIGGERSCRIPTS, NULL, (void **) &s, &numScripts);
-    free(s);
+    (void) hge(h, RPMTAG_TRIGGERFLAGS, NULL, (void **) &flags, NULL);
+    (void) hge(h, RPMTAG_TRIGGERSCRIPTS, &tst, (void **) &s, &numScripts);
+    s = hfd(s, tst);
 
     *freeData = 1;
     *data = conds = xmalloc(sizeof(char * ) * numScripts);
@@ -419,7 +373,7 @@ static int triggertypeTag(Header h, /*@out@*/ int_32 * type,
 		conds[i] = xstrdup("un");
 	    else
 		conds[i] = xstrdup("postun");
-	    break;
+	    /*@innerbreak@*/ break;
 	}
     }
 
@@ -434,7 +388,7 @@ static int triggertypeTag(Header h, /*@out@*/ int_32 * type,
  * @retval freedata	address of data-was-malloc'ed indicator
  * @return		0 on success
  */
-static int filenamesTag(Header h, /*@out@*/ int_32 * type,
+static int filenamesTag(Header h, /*@out@*/ rpmTagType * type,
 	/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
 	/*@out@*/ int * freeData)
 		/*@modifies *type, *data, *count, *freeData @*/
@@ -451,10 +405,13 @@ static int filenamesTag(Header h, /*@out@*/ int_32 * type,
 
 /* I18N look aside diversions */
 
+/*@-exportlocal -exportheadervar@*/
 int _nl_msg_cat_cntr;	/* XXX GNU gettext voodoo */
-static const char * language = "LANGUAGE";
+/*@=exportlocal =exportheadervar@*/
+/*@observer@*/ static const char * language = "LANGUAGE";
 
-static char * _macro_i18ndomains = "%{?_i18ndomains:%{_i18ndomains}}";
+/*@observer@*/ static const char * _macro_i18ndomains =
+		"%{?_i18ndomains:%{_i18ndomains}}";
 
 /**
  * @param h		header
@@ -465,11 +422,12 @@ static char * _macro_i18ndomains = "%{?_i18ndomains:%{_i18ndomains}}";
  * @retval freedata	address of data-was-malloc'ed indicator
  * @return		0 on success
  */
-static int i18nTag(Header h, int_32 tag, /*@out@*/ int_32 * type,
+static int i18nTag(Header h, int_32 tag, /*@out@*/ rpmTagType * type,
 	/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
 	/*@out@*/ int * freeData)
-		/*@modifies h, *type, *data, *count, *freeData @*/
+		/*@modifies *type, *data, *count, *freeData @*/
 {
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
     char * dstring = rpmExpand(_macro_i18ndomains, NULL);
     int rc;
 
@@ -487,7 +445,7 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ int_32 * type,
 	{   const char * tn = tagName(tag);
 	    const char * n;
 	    char * mk;
-	    headerNVR(h, &n, NULL, NULL);
+	    (void) headerNVR(h, &n, NULL, NULL);
 	    mk = alloca(strlen(n) + strlen(tn) + sizeof("()"));
 	    sprintf(mk, "%s(%s)", n, tn);
 	    msgkey = mk;
@@ -495,7 +453,7 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ int_32 * type,
 
 	/* change to en_US for msgkey -> msgid resolution */
 	langval = getenv(language);
-	setenv(language, "en_US", 1);
+	(void) setenv(language, "en_US", 1);
 	++_nl_msg_cat_cntr;
 
 	msgid = NULL;
@@ -508,7 +466,7 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ int_32 * type,
 
 	/* restore previous environment for msgid -> msgstr resolution */
 	if (langval)
-	    setenv(language, langval, 1);
+	    (void) setenv(language, langval, 1);
 	else
 	    unsetenv(language);
 	++_nl_msg_cat_cntr;
@@ -519,17 +477,16 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ int_32 * type,
 	    *count = 1;
 	    *freeData = 1;
 	}
-	free(dstring); dstring = NULL;
-	if (*data) {
+	dstring = _free(dstring);
+	if (*data)
 	    return 0;
-	}
     }
 
-    if (dstring) free(dstring);
+    dstring = _free(dstring);
 
-    rc = headerGetEntry(h, tag, type, (void **)data, count);
+    rc = hge(h, tag, type, (void **)data, count);
 
-    if (rc) {
+    if (rc && (*data) != NULL) {
 	*data = xstrdup(*data);
 	*freeData = 1;
 	return 0;
@@ -549,10 +506,10 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ int_32 * type,
  * @retval freedata	address of data-was-malloc'ed indicator
  * @return		0 on success
  */
-static int summaryTag(Header h, /*@out@*/ int_32 * type,
+static int summaryTag(Header h, /*@out@*/ rpmTagType * type,
 	/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
 	/*@out@*/ int * freeData)
-		/*@modifies h, *type, *data, *count, *freeData @*/
+		/*@modifies *type, *data, *count, *freeData @*/
 {
     return i18nTag(h, RPMTAG_SUMMARY, type, data, count, freeData);
 }
@@ -565,10 +522,10 @@ static int summaryTag(Header h, /*@out@*/ int_32 * type,
  * @retval freedata	address of data-was-malloc'ed indicator
  * @return		0 on success
  */
-static int descriptionTag(Header h, /*@out@*/ int_32 * type,
+static int descriptionTag(Header h, /*@out@*/ rpmTagType * type,
 	/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
 	/*@out@*/ int * freeData)
-		/*@modifies h, *type, *data, *count, *freeData @*/
+		/*@modifies *type, *data, *count, *freeData @*/
 {
     return i18nTag(h, RPMTAG_DESCRIPTION, type, data, count, freeData);
 }
@@ -581,15 +538,15 @@ static int descriptionTag(Header h, /*@out@*/ int_32 * type,
  * @retval freedata	address of data-was-malloc'ed indicator
  * @return		0 on success
  */
-static int groupTag(Header h, /*@out@*/ int_32 * type,
+static int groupTag(Header h, /*@out@*/ rpmTagType * type,
 	/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
 	/*@out@*/ int * freeData)
-		/*@modifies h, *type, *data, *count, *freeData @*/
+		/*@modifies *type, *data, *count, *freeData @*/
 {
     return i18nTag(h, RPMTAG_GROUP, type, data, count, freeData);
 }
 
-const struct headerSprintfExtension rpmHeaderFormats[] = {
+const struct headerSprintfExtension_s rpmHeaderFormats[] = {
     { HEADER_EXT_TAG, "RPMTAG_GROUP", { groupTag } },
     { HEADER_EXT_TAG, "RPMTAG_DESCRIPTION", { descriptionTag } },
     { HEADER_EXT_TAG, "RPMTAG_SUMMARY", { summaryTag } },
