@@ -7,6 +7,8 @@
 #include <rpmmacro.h>	/* XXX for rpmGetPath */
 #include "debug.h"
 
+/*@-usereleased -onlytrans@*/
+
 struct fsinfo {
 /*@only@*/ const char * mntPoint;	/*!< path to mount point. */
     dev_t dev;				/*!< devno for mount point. */
@@ -22,9 +24,8 @@ void freeFilesystems(void)
     if (filesystems) {
 	int i;
 	for (i = 0; i < numFilesystems; i++)
-	    free((void *)filesystems[i].mntPoint);
-	free(filesystems);
-	filesystems = NULL;
+	    filesystems[i].mntPoint = _free(filesystems[i].mntPoint);
+	filesystems = _free(filesystems);
     }
     if (fsnames) {
 #if 0	/* XXX leak/segfault on exit of "rpm -qp --qf '%{#fsnames}' pkg" */
@@ -163,14 +164,18 @@ static int getFilesystemList(void)
     while (1) {
 #	if GETMNTENT_ONE
 	    /* this is Linux */
+	    /*@-modunconnomods@*/
 	    our_mntent * itemptr = getmntent(mtab);
 	    if (!itemptr) break;
 	    item = *itemptr;	/* structure assignment */
 	    mntdir = item.our_mntdir;
 #if defined(MNTOPT_RO)
+	    /*@-compdef@*/
 	    if (hasmntopt(itemptr, MNTOPT_RO) != NULL)
 		rdonly = 1;
+	    /*@=compdef@*/
 #endif
+	    /*@=modunconnomods@*/
 #	elif GETMNTENT_TWO
 	    /* Solaris, maybe others */
 	    if (getmntent(mtab, &item)) break;
@@ -201,9 +206,9 @@ static int getFilesystemList(void)
     }
 
 #   if GETMNTENT_ONE || GETMNTENT_TWO
-	fclose(mtab);
+	(void) fclose(mtab);
 #   elif HAVE_GETMNTINFO_R
-	free(mounts);
+	mounts = _free(mounts);
 #   endif
 
     filesystems[numFilesystems].dev = 0;
@@ -286,8 +291,8 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 		if (errno != ENOENT) {
 		    rpmError(RPMERR_STAT, _("failed to stat %s: %s\n"), buf,
 				strerror(errno));
-		    free((void *)sourceDir);
-		    free(usages);
+		    sourceDir = _free(sourceDir);
+		    usages = _free(usages);
 		    return 1;
 		}
 
@@ -302,13 +307,14 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 
 	    if (lastDev != sb.st_dev) {
 		for (j = 0; j < numFilesystems; j++)
-		    if (filesystems[j].dev == sb.st_dev) break;
+		    if (filesystems && filesystems[j].dev == sb.st_dev)
+			/*@innerbreak@*/ break;
 
 		if (j == numFilesystems) {
 		    rpmError(RPMERR_BADDEV, 
 				_("file %s is on an unknown device\n"), buf);
-		    free((void *)sourceDir);
-		    free(usages);
+		    sourceDir = _free(sourceDir);
+		    usages = _free(usages);
 		    return 1;
 		}
 
@@ -321,9 +327,13 @@ int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes, int numFiles
 	usages[lastfs] += fssizes[i];
     }
 
-    if (sourceDir) free((void *)sourceDir);
+    sourceDir = _free(sourceDir);
 
-    *usagesPtr = usages;
+    if (usagesPtr)
+	*usagesPtr = usages;
+    else
+	usages = _free(usages);
 
     return 0;
 }
+/*@=usereleased =onlytrans@*/
