@@ -23,16 +23,18 @@
 #include "debug.h"
 
 static int _ie = 0x44332211;
-static union _mendian { int i; char b[4]; } *_endian = (union _mendian *)&_ie;
+static union _mendian {
+/*@unused@*/ int i;
+    char b[4];
+} *_endian = (union _mendian *)&_ie;
 #define	IS_BIG_ENDIAN()		(_endian->b[0] == '\x44')
 #define	IS_LITTLE_ENDIAN()	(_endian->b[0] == '\x11')
 
-void byteReverse(unsigned char *buf, unsigned longs);
-
 /*
  * Note: this code is harmless on little-endian machines.
  */
-void byteReverse(unsigned char *buf, unsigned longs)
+static void byteReverse(unsigned char *buf, unsigned longs)
+	/*@modifies *buf @*/
 {
     uint32 t;
     do {
@@ -122,7 +124,9 @@ void rpmMD5Update(struct MD5Context *ctx, unsigned char const *buf, unsigned len
  * Final wrapup - pad to 64-byte boundary with the bit pattern 
  * 1 0* (64-bit count of bits processed, MSB-first)
  */
+/*@-fixedformalarray@*/
 void rpmMD5Final(unsigned char digest[16], struct MD5Context *ctx)
+/*@=fixedformalarray@*/
 {
     unsigned count;
     unsigned char *p;
@@ -183,7 +187,9 @@ void rpmMD5Final(unsigned char digest[16], struct MD5Context *ctx)
  * reflect the addition of 16 longwords of new data.  rpmMD5Update blocks
  * the data and converts bytes into longwords for this routine.
  */
+/*@-fixedformalarray@*/
 void rpmMD5Transform(uint32 buf[4], uint32 const in[16])
+/*@=fixedformalarray@*/
 {
     register uint32 a, b, c, d;
 
