@@ -16,7 +16,10 @@ static int _debug = 0;
 /*@access Header@*/		/* XXX compared with NULL */
 /*@access FD_t@*/		/* XXX compared with NULL */
 
-char * RPMVERSION = VERSION;	/* just to put a marker in librpm.a */
+/*@-exportheadervar@*/
+/* just to put a marker in librpm.a */
+/*@unused@*/ /*@observer@*/ char * RPMVERSION = VERSION;
+/*@=exportheadervar@*/
 
 char ** splitString(const char * str, int length, char sep)
 {
@@ -56,8 +59,10 @@ char ** splitString(const char * str, int length, char sep)
 
 void freeSplitString(char ** list)
 {
-    free(list[0]);
-    free(list);
+    /*@-unqualifiedtrans@*/
+    list[0] = _free(list[0]);
+    /*@=unqualifiedtrans@*/
+    list = _free(list);
 }
 
 int rpmfileexists(const char * urlfn)
@@ -89,129 +94,25 @@ int rpmfileexists(const char * urlfn)
     return 1;
 }
 
-/* compare alpha and numeric segments of two versions */
-/* return 1: a is newer than b */
-/*        0: a and b are the same version */
-/*       -1: b is newer than a */
-int rpmvercmp(const char * a, const char * b)
-{
-    char oldch1, oldch2;
-    char * str1, * str2;
-    char * one, * two;
-    int rc;
-    int isnum;
-
-    /* easy comparison to see if versions are identical */
-    if (!strcmp(a, b)) return 0;
-
-    str1 = alloca(strlen(a) + 1);
-    str2 = alloca(strlen(b) + 1);
-
-    strcpy(str1, a);
-    strcpy(str2, b);
-
-    one = str1;
-    two = str2;
-
-    /* loop through each version segment of str1 and str2 and compare them */
-    while (*one && *two) {
-	while (*one && !isalnum(*one)) one++;
-	while (*two && !isalnum(*two)) two++;
-
-	str1 = one;
-	str2 = two;
-
-	/* grab first completely alpha or completely numeric segment */
-	/* leave one and two pointing to the start of the alpha or numeric */
-	/* segment and walk str1 and str2 to end of segment */
-	if (isdigit(*str1)) {
-	    while (*str1 && isdigit(*str1)) str1++;
-	    while (*str2 && isdigit(*str2)) str2++;
-	    isnum = 1;
-	} else {
-	    while (*str1 && isalpha(*str1)) str1++;
-	    while (*str2 && isalpha(*str2)) str2++;
-	    isnum = 0;
-	}
-
-	/* save character at the end of the alpha or numeric segment */
-	/* so that they can be restored after the comparison */
-	oldch1 = *str1;
-	*str1 = '\0';
-	oldch2 = *str2;
-	*str2 = '\0';
-
-	/* take care of the case where the two version segments are */
-	/* different types: one numeric and one alpha */
-	if (one == str1) return -1;	/* arbitrary */
-	if (two == str2) return -1;
-
-	if (isnum) {
-	    /* this used to be done by converting the digit segments */
-	    /* to ints using atoi() - it's changed because long  */
-	    /* digit segments can overflow an int - this should fix that. */
-
-	    /* throw away any leading zeros - it's a number, right? */
-	    while (*one == '0') one++;
-	    while (*two == '0') two++;
-
-	    /* whichever number has more digits wins */
-	    if (strlen(one) > strlen(two)) return 1;
-	    if (strlen(two) > strlen(one)) return -1;
-	}
-
-	/* strcmp will return which one is greater - even if the two */
-	/* segments are alpha or if they are numeric.  don't return  */
-	/* if they are equal because there might be more segments to */
-	/* compare */
-	rc = strcmp(one, two);
-	if (rc) return rc;
-
-	/* restore character that was replaced by null above */
-	*str1 = oldch1;
-	one = str1;
-	*str2 = oldch2;
-	two = str2;
-    }
-
-    /* this catches the case where all numeric and alpha segments have */
-    /* compared identically but the segment sepparating characters were */
-    /* different */
-    if ((!*one) && (!*two)) return 0;
-
-    /* whichever version still has characters left over wins */
-    if (!*one) return -1; else return 1;
-}
-
 int doputenv(const char *str)
 {
     char * a;
 
     /* FIXME: this leaks memory! */
-
     a = xmalloc(strlen(str) + 1);
     strcpy(a, str);
-
     return putenv(a);
 }
 
-int dosetenv(const char *name, const char *value, int overwrite)
+int dosetenv(const char * name, const char * value, int overwrite)
 {
-    int i;
     char * a;
 
-    /* FIXME: this leaks memory! */
-
     if (!overwrite && getenv(name)) return 0;
 
-    i = strlen(name) + strlen(value) + 2;
-    a = xmalloc(i);
-    if (!a) return 1;
-
-    strcpy(a, name);
-    strcat(a, "=");
-    strcat(a, value);
-
+    /* FIXME: this leaks memory! */
+    a = xmalloc(strlen(name) + strlen(value) + sizeof("="));
+    (void) stpcpy( stpcpy( stpcpy( a, name), "="), value);
     return putenv(a);
 }
 
@@ -226,7 +127,7 @@ static int rpmMkpath(const char * path, mode_t mode, uid_t uid, gid_t gid)
     d = alloca(strlen(path)+2);
     de = stpcpy(d, path);
     de[1] = '\0';
-    for (de = d; *de; de++) {
+    for (de = d; *de != '\0'; de++) {
 	struct stat st;
 	char savec;
 
@@ -293,11 +194,11 @@ int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
 	char tfnbuf[64];
 #ifndef	NOTYET
 	sprintf(tfnbuf, "rpm-tmp.%d", ran++);
-	if (tempfn)	free((void *)tempfn);
+	tempfn = _free(tempfn);
 	tempfn = rpmGenPath(prefix, tpmacro, tfnbuf);
 #else
 	strcpy(tfnbuf, "rpm-tmp.XXXXXX");
-	if (tempfn)	free((void *)tempfn);
+	tempfn = _free(tempfn);
 	tempfn = rpmGenPath(prefix, tpmacro, mktemp(tfnbuf));
 #endif
 
@@ -321,9 +222,9 @@ int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
 	goto errxit;
 
     switch(temput) {
-	struct stat sb, sb2;
     case URL_IS_PATH:
     case URL_IS_UNKNOWN:
+      {	struct stat sb, sb2;
 	if (!stat(tfn, &sb) && S_ISLNK(sb.st_mode)) {
 	    rpmError(RPMERR_SCRIPT, _("error creating temporary file %s\n"), tfn);
 	    goto errxit;
@@ -340,24 +241,24 @@ int makeTempFile(const char * prefix, const char ** fnptr, FD_t * fdptr)
 		goto errxit;
 	    }
 	}
-	break;
+      }	break;
     default:
 	break;
     }
 
     if (fnptr)
 	*fnptr = tempfn;
-    else if (tempfn) {
-	free((void *)tempfn);
-	tempfn = NULL;
-    }
+    else 
+	tempfn = _free(tempfn);
     *fdptr = fd;
 
     return 0;
 
 errxit:
-    if (tempfn) free((void *)tempfn);
-    if (fd) Fclose(fd);
+    tempfn = _free(tempfn);
+    /*@-usereleased@*/
+    if (fd) (void) Fclose(fd);
+    /*@=usereleased@*/
     return 1;
 }
 
@@ -387,10 +288,15 @@ static int dncmp(const void * a, const void * b)
 
 void compressFilelist(Header h)
 {
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HAE_t hae = (HAE_t)headerAddEntry;
+    HRE_t hre = (HRE_t)headerRemoveEntry;
+    HFD_t hfd = headerFreeData;
     char ** fileNames;
     const char ** dirNames;
     const char ** baseNames;
     int_32 * dirIndexes;
+    rpmTagType fnt;
     int count;
     int i;
     int dirIndex = -1;
@@ -402,13 +308,14 @@ void compressFilelist(Header h)
      */
 
     if (headerIsEntry(h, RPMTAG_DIRNAMES)) {
-	headerRemoveEntry(h, RPMTAG_OLDFILENAMES);
+	(void) hre(h, RPMTAG_OLDFILENAMES);
 	return;		/* Already converted. */
     }
 
-    if (!headerGetEntry(h, RPMTAG_OLDFILENAMES, NULL,
-			(void **) &fileNames, &count))
+    if (!hge(h, RPMTAG_OLDFILENAMES, &fnt, (void **) &fileNames, &count))
 	return;		/* no file list */
+    if (fileNames == NULL || count <= 0)
+	return;
 
     dirNames = alloca(sizeof(*dirNames) * count);	/* worst case */
     baseNames = alloca(sizeof(*dirNames) * count);
@@ -427,10 +334,15 @@ void compressFilelist(Header h)
 
     for (i = 0; i < count; i++) {
 	const char ** needle;
-	char *baseName = strrchr(fileNames[i], '/') + 1;
 	char savechar;
-	int len = baseName - fileNames[i];
+	char * baseName;
+	int len;
 
+	if (fileNames[i] == NULL)	/* XXX can't happen */
+	    continue;
+	baseName = strrchr(fileNames[i], '/') + 1;
+	len = baseName - fileNames[i];
+	needle = dirNames;
 	savechar = *baseName;
 	*baseName = '\0';
 	if (dirIndex < 0 ||
@@ -448,16 +360,17 @@ void compressFilelist(Header h)
     }
 
 exit:
-    headerAddEntry(h, RPMTAG_DIRINDEXES, RPM_INT32_TYPE,
-			dirIndexes, count);
-    headerAddEntry(h, RPMTAG_BASENAMES, RPM_STRING_ARRAY_TYPE,
+    if (count > 0) {
+	(void) hae(h, RPMTAG_DIRINDEXES, RPM_INT32_TYPE, dirIndexes, count);
+	(void) hae(h, RPMTAG_BASENAMES, RPM_STRING_ARRAY_TYPE,
 			baseNames, count);
-    headerAddEntry(h, RPMTAG_DIRNAMES, RPM_STRING_ARRAY_TYPE,
+	(void) hae(h, RPMTAG_DIRNAMES, RPM_STRING_ARRAY_TYPE,
 			dirNames, dirIndex + 1);
+    }
 
-    free((void *)fileNames);
+    fileNames = hfd(fileNames, fnt);
 
-    headerRemoveEntry(h, RPMTAG_OLDFILENAMES);
+    (void) hre(h, RPMTAG_OLDFILENAMES);
 }
 
 /*
@@ -465,26 +378,29 @@ exit:
  * is getting all of this into a single xmalloc'd block.
  */
 static void doBuildFileList(Header h, /*@out@*/ const char *** fileListPtr,
-			    /*@out@*/ int * fileCountPtr, int baseNameTag,
-			    int dirNameTag, int dirIndexesTag)
+			    /*@out@*/ int * fileCountPtr, rpmTag baseNameTag,
+			    rpmTag dirNameTag, rpmTag dirIndexesTag)
 {
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HFD_t hfd = headerFreeData;
     const char ** baseNames;
     const char ** dirNames;
     int * dirIndexes;
     int count;
     const char ** fileNames;
     int size;
+    rpmTagType bnt, dnt;
     char * data;
     int i;
 
-    if (!headerGetEntry(h, baseNameTag, NULL, (void **) &baseNames, &count)) {
-	*fileListPtr = NULL;
-	*fileCountPtr = 0;
+    if (!hge(h, baseNameTag, &bnt, (void **) &baseNames, &count)) {
+	if (fileListPtr) *fileListPtr = NULL;
+	if (fileCountPtr) *fileCountPtr = 0;
 	return;		/* no file list */
     }
 
-    headerGetEntry(h, dirNameTag, NULL, (void **) &dirNames, NULL);
-    headerGetEntry(h, dirIndexesTag, NULL, (void **) &dirIndexes, &count);
+    (void) hge(h, dirNameTag, &dnt, (void **) &dirNames, NULL);
+    (void) hge(h, dirIndexesTag, NULL, (void **) &dirIndexes, &count);
 
     size = sizeof(*fileNames) * count;
     for (i = 0; i < count; i++)
@@ -497,15 +413,20 @@ static void doBuildFileList(Header h, /*@out@*/ const char *** fileListPtr,
 	data = stpcpy( stpcpy(data, dirNames[dirIndexes[i]]), baseNames[i]);
 	*data++ = '\0';
     }
-    free((void *)baseNames);
-    free((void *)dirNames);
+    baseNames = hfd(baseNames, bnt);
+    dirNames = hfd(dirNames, dnt);
 
-    *fileListPtr = fileNames;
-    *fileCountPtr = count;
+    if (fileListPtr)
+	*fileListPtr = fileNames;
+    else
+	fileNames = _free(fileNames);
+    if (fileCountPtr) *fileCountPtr = count;
 }
 
 void expandFilelist(Header h)
 {
+    HAE_t hae = (HAE_t)headerAddEntry;
+    HRE_t hre = (HRE_t)headerRemoveEntry;
     const char ** fileNames = NULL;
     int count = 0;
 
@@ -514,14 +435,14 @@ void expandFilelist(Header h)
 			RPMTAG_DIRNAMES, RPMTAG_DIRINDEXES);
 	if (fileNames == NULL || count <= 0)
 	    return;
-	headerAddEntry(h, RPMTAG_OLDFILENAMES, RPM_STRING_ARRAY_TYPE,
+	(void) hae(h, RPMTAG_OLDFILENAMES, RPM_STRING_ARRAY_TYPE,
 			fileNames, count);
-	free((void *)fileNames);
+	fileNames = _free(fileNames);
     }
 
-    headerRemoveEntry(h, RPMTAG_DIRNAMES);
-    headerRemoveEntry(h, RPMTAG_BASENAMES);
-    headerRemoveEntry(h, RPMTAG_DIRINDEXES);
+    (void) hre(h, RPMTAG_DIRNAMES);
+    (void) hre(h, RPMTAG_BASENAMES);
+    (void) hre(h, RPMTAG_DIRINDEXES);
 }
 
 
@@ -598,10 +519,11 @@ int rpmGlob(const char * patterns, int * argcPtr, const char *** argvPtr)
 	    if (argc == 0)
 		argv = xmalloc((argc+2) * sizeof(*argv));
 	    else
-		argv = xrealloc(argv, (argc+1) * sizeof(*argv));
+		argv = xrealloc(argv, (argc+2) * sizeof(*argv));
+	    argv[argc] = xstrdup(av[j]);
 if (_debug)
-fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, av[j]);
-	    argv[argc++] = xstrdup(av[j]);
+fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, argv[argc]);
+	    argc++;
 	    continue;
 	}
 	
@@ -652,8 +574,10 @@ if (_debug)
 fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, globURL);
 	    argv[argc++] = xstrdup(globURL);
 	}
+	/*@-immediatetrans@*/
 	Globfree(&gl);
-	free((void *)globURL);
+	/*@=immediatetrans@*/
+	globURL = _free(globURL);
     }
     if (argv != NULL && argc > 0) {
 	argv[argc] = NULL;
@@ -667,13 +591,12 @@ fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, globURL);
 
 
 exit:
-    if (av)
-	free((void *)av);
-    if ((rc || argvPtr == NULL) && argv) {
+    av = _free(av);
+    if (rc || argvPtr == NULL) {
+	if (argv != NULL)
 	for (i = 0; i < argc; i++)
-	    free((void *)argv[i]);
-	free((void *)argv);
-	argv = NULL;
+	    argv[i] = _free(argv[i]);
+	argv = _free(argv);
     }
     return rc;
 }
@@ -770,6 +693,8 @@ int rpmPackageGetEntry( /*@unused@*/ void *leadp, Header sigs, Header h,
  */
 void providePackageNVR(Header h)
 {
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HFD_t hfd = headerFreeData;
     const char *name, *version, *release;
     int_32 * epoch;
     const char *pEVR;
@@ -777,18 +702,21 @@ void providePackageNVR(Header h)
     int_32 pFlags = RPMSENSE_EQUAL;
     const char ** provides = NULL;
     const char ** providesEVR = NULL;
+    rpmTagType pnt, pvt;
     int_32 * provideFlags = NULL;
     int providesCount;
     int i;
     int bingo = 1;
 
     /* Generate provides for this package name-version-release. */
-    headerNVR(h, &name, &version, &release);
+    (void) headerNVR(h, &name, &version, &release);
+    if (!(name && version && release))
+	return;
     pEVR = p = alloca(21 + strlen(version) + 1 + strlen(release) + 1);
     *p = '\0';
-    if (headerGetEntry(h, RPMTAG_EPOCH, NULL, (void **) &epoch, NULL)) {
+    if (hge(h, RPMTAG_EPOCH, NULL, (void **) &epoch, NULL)) {
 	sprintf(p, "%d:", *epoch);
-	while (*p)
+	while (*p != '\0')
 	    p++;
     }
     (void) stpcpy( stpcpy( stpcpy(p, version) , "-") , release);
@@ -797,31 +725,30 @@ void providePackageNVR(Header h)
      * Rpm prior to 3.0.3 does not have versioned provides.
      * If no provides at all are available, we can just add.
      */
-    if (!headerGetEntry(h, RPMTAG_PROVIDENAME, NULL,
-		(void **) &provides, &providesCount)) {
+    if (!hge(h, RPMTAG_PROVIDENAME, &pnt, (void **) &provides, &providesCount))
 	goto exit;
-    }
 
     /*
      * Otherwise, fill in entries on legacy packages.
      */
-    if (!headerGetEntry(h, RPMTAG_PROVIDEVERSION, NULL,
-		(void **) &providesEVR, NULL)) {
+    if (!hge(h, RPMTAG_PROVIDEVERSION, &pvt, (void **) &providesEVR, NULL)) {
 	for (i = 0; i < providesCount; i++) {
 	    char * vdummy = "";
 	    int_32 fdummy = RPMSENSE_ANY;
-	    headerAddOrAppendEntry(h, RPMTAG_PROVIDEVERSION, RPM_STRING_ARRAY_TYPE,
+	    (void) headerAddOrAppendEntry(h, RPMTAG_PROVIDEVERSION, RPM_STRING_ARRAY_TYPE,
 			&vdummy, 1);
-	    headerAddOrAppendEntry(h, RPMTAG_PROVIDEFLAGS, RPM_INT32_TYPE,
+	    (void) headerAddOrAppendEntry(h, RPMTAG_PROVIDEFLAGS, RPM_INT32_TYPE,
 			&fdummy, 1);
 	}
 	goto exit;
     }
 
-    headerGetEntry(h, RPMTAG_PROVIDEFLAGS, NULL,
-	(void **) &provideFlags, NULL);
+    (void) hge(h, RPMTAG_PROVIDEFLAGS, NULL, (void **) &provideFlags, NULL);
 
+    if (provides && providesEVR && provideFlags)
     for (i = 0; i < providesCount; i++) {
+        if (!(provides[i] && providesEVR[i]))
+            continue;
 	if (!(provideFlags[i] == RPMSENSE_EQUAL &&
 	    !strcmp(name, provides[i]) && !strcmp(pEVR, providesEVR[i])))
 	    continue;
@@ -830,15 +757,15 @@ void providePackageNVR(Header h)
     }
 
 exit:
-    if (provides) free((void *)provides);
-    if (providesEVR) free((void *)providesEVR);
+    provides = hfd(provides, pnt);
+    providesEVR = hfd(providesEVR, pvt);
 
     if (bingo) {
-	headerAddOrAppendEntry(h, RPMTAG_PROVIDENAME, RPM_STRING_ARRAY_TYPE,
+	(void) headerAddOrAppendEntry(h, RPMTAG_PROVIDENAME, RPM_STRING_ARRAY_TYPE,
 		&name, 1);
-	headerAddOrAppendEntry(h, RPMTAG_PROVIDEFLAGS, RPM_INT32_TYPE,
+	(void) headerAddOrAppendEntry(h, RPMTAG_PROVIDEFLAGS, RPM_INT32_TYPE,
 		&pFlags, 1);
-	headerAddOrAppendEntry(h, RPMTAG_PROVIDEVERSION, RPM_STRING_ARRAY_TYPE,
+	(void) headerAddOrAppendEntry(h, RPMTAG_PROVIDEVERSION, RPM_STRING_ARRAY_TYPE,
 		&pEVR, 1);
     }
 }
