@@ -1,64 +1,28 @@
 /** \ingroup rpmcli
  * \file lib/query.c
+ * Display tag values from package metadata.
  */
 
 #include "system.h"
 
 #ifndef PATH_MAX
+/*@-incondefs@*/	/* FIX: long int? */
 # define PATH_MAX 255
+/*@=incondefs@*/
 #endif
 
+#include <rpmcli.h>
 #include <rpmbuild.h>
-#include <rpmurl.h>
+#include "manifest.h"
 #include "debug.h"
 
-/*@access Header@*/			/* XXX compared with NULL */
 /*@access rpmdbMatchIterator@*/		/* XXX compared with NULL */
+/*@access Header@*/			/* XXX compared with NULL */
+/*@access rpmdb@*/			/* XXX compared with NULL */
+/*@access FD_t@*/			/* XXX compared with NULL */
 
-/* ======================================================================== */
-static char * permsString(int mode)
-{
-    char *perms = xstrdup("----------");
-   
-    if (S_ISDIR(mode)) 
-	perms[0] = 'd';
-    else if (S_ISLNK(mode))
-	perms[0] = 'l';
-    else if (S_ISFIFO(mode)) 
-	perms[0] = 'p';
-    else if (S_ISSOCK(mode)) 
-	perms[0] = 's';
-    else if (S_ISCHR(mode))
-	perms[0] = 'c';
-    else if (S_ISBLK(mode))
-	perms[0] = 'b';
-
-    /*@-unrecog@*/
-    if (mode & S_IRUSR) perms[1] = 'r';
-    if (mode & S_IWUSR) perms[2] = 'w';
-    if (mode & S_IXUSR) perms[3] = 'x';
- 
-    if (mode & S_IRGRP) perms[4] = 'r';
-    if (mode & S_IWGRP) perms[5] = 'w';
-    if (mode & S_IXGRP) perms[6] = 'x';
-
-    if (mode & S_IROTH) perms[7] = 'r';
-    if (mode & S_IWOTH) perms[8] = 'w';
-    if (mode & S_IXOTH) perms[9] = 'x';
-
-    if (mode & S_ISUID)
-	perms[3] = ((mode & S_IXUSR) ? 's' : 'S'); 
-
-    if (mode & S_ISGID)
-	perms[6] = ((mode & S_IXGRP) ? 's' : 'S'); 
-
-    if (mode & S_ISVTX)
-	perms[9] = ((mode & S_IXOTH) ? 't' : 'T');
-    /*@=unrecog@*/
-
-    return perms;
-}
-
+/**
+ */
 static void printFileInfo(char * te, const char * name,
 			  unsigned int size, unsigned short mode,
 			  unsigned int mtime,
@@ -68,23 +32,21 @@ static void printFileInfo(char * te, const char * name,
 {
     char sizefield[15];
     char ownerfield[9], groupfield[9];
-    char timefield[100] = "";
+    char timefield[100];
     time_t when = mtime;  /* important if sizeof(int_32) ! sizeof(time_t) */
     struct tm * tm;
     static time_t now;
     static struct tm nowtm;
     const char * namefield = name;
-    char * perms;
+    char * perms = rpmPermsString(mode);
 
     /* On first call, grab snapshot of now */
     if (now == 0) {
 	now = time(NULL);
 	tm = localtime(&now);
-	nowtm = *tm;	/* structure assignment */
+	if (tm) nowtm = *tm;	/* structure assignment */
     }
 
-    perms = permsString(mode);
-
     if (owner) 
 	strncpy(ownerfield, owner, 8);
     else
@@ -118,6 +80,8 @@ static void printFileInfo(char * te, const char * name,
 
     /* Convert file mtime to display format */
     tm = localtime(&when);
+    timefield[0] = '\0';
+    if (tm != NULL)
     {	const char *fmt;
 	if (now > when + 6L * 30L * 24L * 60L * 60L ||	/* Old. */
 	    now < when - 60L * 60L)			/* In the future.  */
@@ -138,10 +102,12 @@ static void printFileInfo(char * te, const char * name,
 
     sprintf(te, "%s %4d %-8s%-8s %10s %s %s", perms,
 	(int)nlink, ownerfield, groupfield, sizefield, timefield, namefield);
-    if (perms) free(perms);
+    perms = _free(perms);
 }
 
-static inline const char * queryHeader(Header h, const char * qfmt)
+/**
+ */
+static inline /*@null@*/ const char * queryHeader(Header h, const char * qfmt)
 {
     const char * errstr;
     const char * str;
@@ -152,39 +118,50 @@ static inline const char * queryHeader(Header h, const char * qfmt)
     return str;
 }
 
+/**
+ */
 static int countLinks(int_16 * fileRdevList, int_32 * fileInodeList, int nfiles,
 		int xfile)
 {
     int nlink = 0;
 
     /* XXX rpm-3.3.12 has not RPMTAG_FILEINODES */
-    if (!(fileRdevList && fileInodeList && nfiles > 0))
+    if (!(fileRdevList[xfile] != 0 && fileRdevList &&
+		fileInodeList[xfile] != 0 && fileInodeList && nfiles > 0))
 	return 1;
     while (nfiles-- > 0) {
+	if (fileRdevList[nfiles] == 0)
+	    continue;
 	if (fileRdevList[nfiles] != fileRdevList[xfile])
 	    continue;
+	if (fileInodeList[nfiles] == 0)
+	    continue;
 	if (fileInodeList[nfiles] != fileInodeList[xfile])
 	    continue;
 	nlink++;
     }
+    if (nlink == 0) nlink = 1;
     return nlink;
 }
 
-int showQueryPackage(QVA_t *qva, /*@unused@*/rpmdb rpmdb, Header h)
+int showQueryPackage(QVA_t qva, /*@unused@*/rpmdb rpmdb, Header h)
 {
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HFD_t hfd = headerFreeData;
     char * t, * te;
-    
-    int queryFlags = qva->qva_flags;
-    const char *queryFormat = qva->qva_queryFormat;
-
-    int_32 count, type;
+    rpmQueryFlags queryFlags = qva->qva_flags;
+    const char * queryFormat = qva->qva_queryFormat;
+    rpmTagType type;
+    int_32 count;
     char * prefix = NULL;
     const char ** dirNames = NULL;
     const char ** baseNames = NULL;
+    rpmTagType bnt, dnt;
     const char ** fileMD5List = NULL;
     const char ** fileOwnerList = NULL;
     const char ** fileGroupList = NULL;
     const char ** fileLinktoList = NULL;
+    rpmTagType m5t, fot, fgt, ltt;
     const char * fileStatesList;
     int_32 * fileFlagsList, * fileMTimeList, * fileSizeList;
     int_32 * fileUIDList = NULL;
@@ -200,9 +177,9 @@ int showQueryPackage(QVA_t *qva, /*@unused@*/rpmdb rpmdb, Header h)
     te = t = xmalloc(BUFSIZ);
     *te = '\0';
 
-    if (!queryFormat && !queryFlags) {
+    if (queryFormat == NULL && queryFlags == QUERY_FOR_DEFAULT) {
 	const char * name, * version, * release;
-	headerNVR(h, &name, &version, &release);
+	(void) headerNVR(h, &name, &version, &release);
 	te = stpcpy(te, name);
 	te = stpcpy( stpcpy(te, "-"), version);
 	te = stpcpy( stpcpy(te, "-"), release);
@@ -220,66 +197,75 @@ int showQueryPackage(QVA_t *qva, /*@unused@*/rpmdb rpmdb, Header h)
 		t = xrealloc(t, BUFSIZ+sb);
 		te = t + tb;
 	    }
+	    /*@-usereleased@*/
 	    te = stpcpy(te, str);
-	    free((void *)str);
+	    /*@=usereleased@*/
+	    str = _free(str);
 	}
     }
 
     if (!(queryFlags & QUERY_FOR_LIST))
 	goto exit;
 
-    if (!headerGetEntry(h, RPMTAG_BASENAMES, &type, 
-				(void **) &baseNames, &count)) {
+    if (!hge(h, RPMTAG_BASENAMES, &bnt, (void **) &baseNames, &count)) {
 	te = stpcpy(te, _("(contains no files)"));
 	goto exit;
     }
-    if (!headerGetEntry(h, RPMTAG_FILESTATES, &type, 
-			 (void **) &fileStatesList, &count)) {
+    if (!hge(h, RPMTAG_FILESTATES, &type, (void **) &fileStatesList, NULL))
 	fileStatesList = NULL;
-    }
-    headerGetEntry(h, RPMTAG_DIRNAMES, NULL, (void **) &dirNames, NULL);
-    headerGetEntry(h, RPMTAG_DIRINDEXES, NULL, (void **) &dirIndexes, NULL);
-    headerGetEntry(h, RPMTAG_FILEFLAGS, &type, (void **)&fileFlagsList, &count);
-    headerGetEntry(h, RPMTAG_FILESIZES, &type, (void **) &fileSizeList, &count);
-    headerGetEntry(h, RPMTAG_FILEMODES, &type, (void **) &fileModeList, &count);
-    headerGetEntry(h, RPMTAG_FILEMTIMES, &type, (void **)&fileMTimeList,&count);
-    headerGetEntry(h, RPMTAG_FILERDEVS, &type, (void **) &fileRdevList, &count);
-    headerGetEntry(h, RPMTAG_FILEINODES, &type, (void **)&fileInodeList,&count);
-    headerGetEntry(h, RPMTAG_FILELINKTOS,&type,(void **)&fileLinktoList,&count);
-    headerGetEntry(h, RPMTAG_FILEMD5S, &type, (void **) &fileMD5List, &count);
-
-    if (!headerGetEntry(h, RPMTAG_FILEUIDS, &type, 
-			 (void **) &fileUIDList, &count)) {
+    if (!hge(h, RPMTAG_DIRNAMES, &dnt, (void **) &dirNames, NULL))
+	dirNames = NULL;
+    if (!hge(h, RPMTAG_DIRINDEXES, NULL, (void **) &dirIndexes, NULL))
+	dirIndexes = NULL;
+    if (!hge(h, RPMTAG_FILEFLAGS, &type, (void **) &fileFlagsList, NULL))
+	fileFlagsList = NULL;
+    if (!hge(h, RPMTAG_FILESIZES, &type, (void **) &fileSizeList, NULL))
+	fileSizeList = NULL;
+    if (!hge(h, RPMTAG_FILEMODES, &type, (void **) &fileModeList, NULL))
+	fileModeList = NULL;
+    if (!hge(h, RPMTAG_FILEMTIMES, &type, (void **) &fileMTimeList, NULL))
+	fileMTimeList = NULL;
+    if (!hge(h, RPMTAG_FILERDEVS, &type, (void **) &fileRdevList, NULL))
+	fileRdevList = NULL;
+    if (!hge(h, RPMTAG_FILEINODES, &type, (void **) &fileInodeList, NULL))
+	fileInodeList = NULL;
+    if (!hge(h, RPMTAG_FILELINKTOS, &ltt, (void **) &fileLinktoList, NULL))
+	fileLinktoList = NULL;
+    if (!hge(h, RPMTAG_FILEMD5S, &m5t, (void **) &fileMD5List, NULL))
+	fileMD5List = NULL;
+    if (!hge(h, RPMTAG_FILEUIDS, &type, (void **) &fileUIDList, NULL))
 	fileUIDList = NULL;
-    } else if (!headerGetEntry(h, RPMTAG_FILEGIDS, &type, 
-			     (void **) &fileGIDList, &count)) {
+    if (!hge(h, RPMTAG_FILEGIDS, &type, (void **) &fileGIDList, NULL))
 	fileGIDList = NULL;
-    }
-
-    if (!headerGetEntry(h, RPMTAG_FILEUSERNAME, &type, 
-			 (void **) &fileOwnerList, &count)) {
+    if (!hge(h, RPMTAG_FILEUSERNAME, &fot, (void **) &fileOwnerList, NULL))
 	fileOwnerList = NULL;
-    } else if (!headerGetEntry(h, RPMTAG_FILEGROUPNAME, &type, 
-			     (void **) &fileGroupList, &count)) {
+    if (!hge(h, RPMTAG_FILEGROUPNAME, &fgt, (void **) &fileGroupList, NULL))
 	fileGroupList = NULL;
-    }
 
     for (i = 0; i < count; i++) {
+
 	/* If querying only docs, skip non-doc files. */
 	if ((queryFlags & QUERY_FOR_DOCS)
 	  && !(fileFlagsList[i] & RPMFILE_DOC))
 	    continue;
+
 	/* If querying only configs, skip non-config files. */
 	if ((queryFlags & QUERY_FOR_CONFIG)
 	  && !(fileFlagsList[i] & RPMFILE_CONFIG))
 	    continue;
 
+	/* If not querying %ghost, skip ghost files. */
+	if (!(qva->qva_fflags & RPMFILE_GHOST)
+	  && (fileFlagsList[i] & RPMFILE_GHOST))
+	    continue;
+
 	if (!rpmIsVerbose() && prefix)
 	    te = stpcpy(te, prefix);
 
 	if (queryFlags & QUERY_FOR_STATE) {
 	    if (fileStatesList) {
-		switch (fileStatesList[i]) {
+		rpmfileState fstate = fileStatesList[i];
+		switch (fstate) {
 		case RPMFILE_STATE_NORMAL:
 		    te = stpcpy(te, _("normal        ")); break;
 		case RPMFILE_STATE_REPLACED:
@@ -302,7 +288,7 @@ int showQueryPackage(QVA_t *qva, /*@unused@*/rpmdb rpmdb, Header h)
 	    sprintf(te, "%s%s %d %d %s 0%o ", 
 				   dirNames[dirIndexes[i]], baseNames[i],
 				   fileSizeList[i], fileMTimeList[i],
-				   fileMD5List[i], fileModeList[i]);
+				   fileMD5List[i], (unsigned) fileModeList[i]);
 	    te += strlen(te);
 
 	    if (fileOwnerList && fileGroupList) {
@@ -319,7 +305,7 @@ int showQueryPackage(QVA_t *qva, /*@unused@*/rpmdb rpmdb, Header h)
 	    sprintf(te, " %s %s %u ", 
 				 fileFlagsList[i] & RPMFILE_CONFIG ? "1" : "0",
 				 fileFlagsList[i] & RPMFILE_DOC ? "1" : "0",
-				 (unsigned)fileRdevList[i]);
+				 (unsigned) fileRdevList[i]);
 	    te += strlen(te);
 
 	    if (strlen(fileLinktoList[i]))
@@ -333,15 +319,22 @@ int showQueryPackage(QVA_t *qva, /*@unused@*/rpmdb rpmdb, Header h)
 	} else {
 	    char * filespec;
 	    int nlink;
+	    size_t fileSize;
 
 	    filespec = xmalloc(strlen(dirNames[dirIndexes[i]])
 					      + strlen(baseNames[i]) + 1);
 	    strcpy(filespec, dirNames[dirIndexes[i]]);
 	    strcat(filespec, baseNames[i]);
 					
+	    fileSize = fileSizeList[i];
 	    nlink = countLinks(fileRdevList, fileInodeList, count, i);
+
+if (S_ISDIR(fileModeList[i])) {
+    nlink++;
+    fileSize = 0;
+}
 	    if (fileOwnerList && fileGroupList) {
-		printFileInfo(te, filespec, fileSizeList[i],
+		printFileInfo(te, filespec, fileSize,
 					      fileModeList[i], fileMTimeList[i],
 					      fileRdevList[i], nlink,
 					      fileOwnerList[i], 
@@ -349,7 +342,7 @@ int showQueryPackage(QVA_t *qva, /*@unused@*/rpmdb rpmdb, Header h)
 					      -1, fileLinktoList[i]);
 		te += strlen(te);
 	    } else if (fileUIDList && fileGIDList) {
-		printFileInfo(te, filespec, fileSizeList[i],
+		printFileInfo(te, filespec, fileSize,
 					      fileModeList[i], fileMTimeList[i],
 					      fileRdevList[i], nlink,
 					      NULL, NULL, fileUIDList[i], 
@@ -361,7 +354,7 @@ int showQueryPackage(QVA_t *qva, /*@unused@*/rpmdb rpmdb, Header h)
 			_("package has neither file owner or id lists\n"));
 	    }
 
-	    free(filespec);
+	    filespec = _free(filespec);
 	}
 	if (te > t) {
 	    *te++ = '\n';
@@ -382,22 +375,24 @@ exit:
 	}
 	rpmMessage(RPMMESS_NORMAL, "%s", t);
     }
-    if (t) free(t);
-    if (dirNames) free(dirNames);
-    if (baseNames) free(baseNames);
-    if (fileLinktoList) free(fileLinktoList);
-    if (fileMD5List) free(fileMD5List);
-    if (fileOwnerList) free(fileOwnerList);
-    if (fileGroupList) free(fileGroupList);
+    t = _free(t);
+    dirNames = hfd(dirNames, dnt);
+    baseNames = hfd(baseNames, bnt);
+    fileLinktoList = hfd(fileLinktoList, ltt);
+    fileMD5List = hfd(fileMD5List, m5t);
+    fileOwnerList = hfd(fileOwnerList, fot);
+    fileGroupList = hfd(fileGroupList, fgt);
     return rc;
 }
 
+/**
+ */
 static void
 printNewSpecfile(Spec spec)
 {
-    Header h = spec->packages->header;
-    struct speclines *sl = spec->sl;
-    struct spectags *st = spec->st;
+    Header h;
+    speclines sl = spec->sl;
+    spectags st = spec->st;
     const char * msgstr = NULL;
     int i, j;
 
@@ -405,14 +400,41 @@ printNewSpecfile(Spec spec)
 	return;
 
     for (i = 0; i < st->st_ntags; i++) {
-	struct spectag * t = st->st_t + i;
+	spectag t = st->st_t + i;
 	const char * tn = tagName(t->t_tag);
 	const char * errstr;
-	char fmt[128];
+	char fmt[1024];
 
 	fmt[0] = '\0';
-	(void) stpcpy( stpcpy( stpcpy( fmt, "%{"), tn), "}\n");
-	if (msgstr) free((void *)msgstr);
+	if (t->t_msgid == NULL)
+	    h = spec->packages->header;
+	else {
+	    Package pkg;
+	    char *fe;
+
+	    strcpy(fmt, t->t_msgid);
+	    for (fe = fmt; *fe && *fe != '('; fe++)
+		{} ;
+	    if (*fe == '(') *fe = '\0';
+	    h = NULL;
+	    for (pkg = spec->packages; pkg != NULL; pkg = pkg->next) {
+		const char *pkgname;
+		h = pkg->header;
+		(void) headerNVR(h, &pkgname, NULL, NULL);
+		if (!strcmp(pkgname, fmt))
+		    /*@innerbreak@*/ break;
+	    }
+	    if (pkg == NULL || h == NULL)
+		h = spec->packages->header;
+	}
+
+	if (h == NULL)
+	    continue;
+
+	fmt[0] = '\0';
+	(void) stpcpy( stpcpy( stpcpy( fmt, "%{"), tn), "}");
+	msgstr = _free(msgstr);
+
 	msgstr = headerSprintf(h, fmt, rpmTagTable, rpmHeaderFormats, &errstr);
 	if (msgstr == NULL) {
 	    rpmError(RPMERR_QFMT, _("can't query %s: %s\n"), tn, errstr);
@@ -422,8 +444,9 @@ printNewSpecfile(Spec spec)
 	switch(t->t_tag) {
 	case RPMTAG_SUMMARY:
 	case RPMTAG_GROUP:
-	    free(sl->sl_lines[t->t_startx]);
-	    sl->sl_lines[t->t_startx] = NULL;
+	    /*@-unqualifiedtrans@*/
+	    sl->sl_lines[t->t_startx] = _free(sl->sl_lines[t->t_startx]);
+	    /*@=unqualifiedtrans@*/
 	    if (t->t_lang && strcmp(t->t_lang, RPMBUILD_DEFAULT_LANG))
 		continue;
 	    {   char *buf = xmalloc(strlen(tn) + sizeof(": ") + strlen(msgstr));
@@ -433,12 +456,15 @@ printNewSpecfile(Spec spec)
 	    break;
 	case RPMTAG_DESCRIPTION:
 	    for (j = 1; j < t->t_nlines; j++) {
-		free(sl->sl_lines[t->t_startx + j]);
-		sl->sl_lines[t->t_startx + j] = NULL;
+		if (*sl->sl_lines[t->t_startx + j] == '%')
+		    continue;
+		/*@-unqualifiedtrans@*/
+		sl->sl_lines[t->t_startx + j] =
+			_free(sl->sl_lines[t->t_startx + j]);
+		/*@=unqualifiedtrans@*/
 	    }
 	    if (t->t_lang && strcmp(t->t_lang, RPMBUILD_DEFAULT_LANG)) {
-		free(sl->sl_lines[t->t_startx]);
-		sl->sl_lines[t->t_startx] = NULL;
+		sl->sl_lines[t->t_startx] = _free(sl->sl_lines[t->t_startx]);
 		continue;
 	    }
 	    sl->sl_lines[t->t_startx + 1] = xstrdup(msgstr);
@@ -447,67 +473,75 @@ printNewSpecfile(Spec spec)
 	    break;
 	}
     }
-    if (msgstr) free((void *)msgstr);
+    msgstr = _free(msgstr);
 
     for (i = 0; i < sl->sl_nlines; i++) {
-	if (sl->sl_lines[i] == NULL)
+	const char * s = sl->sl_lines[i];
+	if (s == NULL)
 	    continue;
-	printf("%s", sl->sl_lines[i]);
+	printf("%s", s);
+	if (strchr(s, '\n') == NULL && s[strlen(s)-1] != '\n')
+	    printf("\n");
     }
 }
 
-void rpmDisplayQueryTags(FILE * f)
+void rpmDisplayQueryTags(FILE * fp)
 {
-    const struct headerTagTableEntry * t;
+    const struct headerTagTableEntry_s * t;
     int i;
-    const struct headerSprintfExtension * ext = rpmHeaderFormats;
+    const struct headerSprintfExtension_s * ext = rpmHeaderFormats;
 
-    for (i = 0, t = rpmTagTable; i < rpmTagTableSize; i++, t++) {
-	fprintf(f, "%s\n", t->name + 7);
-    }
+    for (i = 0, t = rpmTagTable; i < rpmTagTableSize; i++, t++)
+	if (t->name) fprintf(fp, "%s\n", t->name + 7);
 
-    while (ext->name) {
+    while (ext->name != NULL) {
 	if (ext->type == HEADER_EXT_MORE) {
 	    ext = ext->u.more;
 	    continue;
 	}
 	/* XXX don't print query tags twice. */
 	for (i = 0, t = rpmTagTable; i < rpmTagTableSize; i++, t++) {
+	    if (t->name == NULL)	/* XXX programmer error. */
+		continue;
 	    if (!strcmp(t->name, ext->name))
-	    	break;
+	    	/*@innerbreak@*/ break;
 	}
 	if (i >= rpmTagTableSize && ext->type == HEADER_EXT_TAG)
-	    fprintf(f, "%s\n", ext->name + 7);
+	    fprintf(fp, "%s\n", ext->name + 7);
 	ext++;
     }
 }
 
-int showMatches(QVA_t *qva, rpmdbMatchIterator mi, QVF_t showPackage)
+int showMatches(QVA_t qva, rpmdbMatchIterator mi, QVF_t showPackage)
 {
     Header h;
     int ec = 0;
 
     while ((h = rpmdbNextIterator(mi)) != NULL) {
 	int rc;
+	/*@-nullpass@*/
 	if ((rc = showPackage(qva, rpmdbGetIteratorRpmDB(mi), h)) != 0)
 	    ec = rc;
+	/*@=nullpass@*/
     }
-    rpmdbFreeIterator(mi);
+    mi = rpmdbFreeIterator(mi);
     return ec;
 }
 
+/*@-redecl@*/
 /**
  * @todo Eliminate linkage loop into librpmbuild.a
  */
 int	(*parseSpecVec) (Spec *specp, const char *specFile, const char *rootdir,
-		const char *buildRoot, int inBuildArch, const char *passPhrase,
+		const char *buildRoot, int recursing, const char *passPhrase,
 		char *cookie, int anyarch, int force) = NULL;
 /**
  * @todo Eliminate linkage loop into librpmbuild.a
  */
-void	(*freeSpecVec) (Spec spec) = NULL;
+/*@null@*/ Spec	(*freeSpecVec) (Spec spec) = NULL;
+/*@=redecl@*/
 
-int rpmQueryVerify(QVA_t *qva, rpmQVSources source, const char * arg,
+int rpmQueryVerify(QVA_t qva, rpmQVSources source, const char * arg,
 	rpmdb rpmdb, QVF_t showPackage)
 {
     rpmdbMatchIterator mi = NULL;
@@ -515,64 +549,92 @@ int rpmQueryVerify(QVA_t *qva, rpmQVSources source, const char * arg,
     int rc;
     int isSource;
     int retcode = 0;
-    char *end = NULL;
+    const char ** av = NULL;
+    char * end = NULL;
 
     switch (source) {
     case RPMQV_RPM:
-    {	int argc = 0;
-	const char ** argv = NULL;
+    {	int ac = 0;
+	const char * fileURL = NULL;
+	rpmRC rpmrc;
 	int i;
 
-	rc = rpmGlob(arg, &argc, &argv);
-	if (rc)
-	    return 1;
-	for (i = 0; i < argc; i++) {
+	rc = rpmGlob(arg, &ac, &av);
+	if (rc) return 1;
+
+restart:
+	for (i = 0; i < ac; i++) {
 	    FD_t fd;
-	    fd = Fopen(argv[i], "r.ufdio");
-	    if (Ferror(fd)) {
-		/* XXX Fstrerror */
-		rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), argv[i],
-#ifndef	NOTYET
-			urlStrerror(argv[i])
-#else
-			Fstrerror(fd)
-#endif
-		);
-		if (fd) Fclose(fd);
+
+	    fileURL = _free(fileURL);
+	    fileURL = av[i];
+	    av[i] = NULL;
+
+	    /* Try to read the header from a package file. */
+	    fd = Fopen(fileURL, "r.ufdio");
+	    if (fd == NULL || Ferror(fd)) {
+		rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), fileURL,
+			Fstrerror(fd));
+		if (fd) (void) Fclose(fd);
 		retcode = 1;
-		break;
+		/*@loopbreak@*/ break;
 	    }
 
-	    retcode = rpmReadPackageHeader(fd, &h, &isSource, NULL, NULL);
+	    /*@-mustmod@*/	/* LCL: segfault. */
+	    rpmrc = rpmReadPackageHeader(fd, &h, &isSource, NULL, NULL);
+	    /*@=mustmod@*/
+	    (void) Fclose(fd);
 
-	    Fclose(fd);
-
-	    switch (retcode) {
-	    case 0:
-		if (h == NULL) {
-		    rpmError(RPMERR_QUERY,
+	    if (!(rpmrc == RPMRC_OK || rpmrc == RPMRC_BADMAGIC)) {
+		rpmError(RPMERR_QUERY, _("query of %s failed\n"), fileURL);
+		retcode = 1;
+		/*@loopbreak@*/ break;
+	    }
+	    if (rpmrc == RPMRC_OK && h == NULL) {
+		rpmError(RPMERR_QUERY,
 			_("old format source packages cannot be queried\n"));
-		    retcode = 1;
-		    break;
-		}
-		retcode = showPackage(qva, rpmdb, h);
-		headerFree(h);
-		break;
-	    case 1:
-		rpmError(RPMERR_QUERY,
-			_("%s does not appear to be a RPM package\n"), argv[i]);
-		/*@fallthrough@*/
-	    case 2:
-		rpmError(RPMERR_QUERY,
-			_("query of %s failed\n"), argv[i]);
 		retcode = 1;
-		break;
+		/*@loopbreak@*/ break;
 	    }
+
+	    /* Query a package file. */
+	    if (rpmrc == RPMRC_OK) {
+		retcode = showPackage(qva, rpmdb, h);
+		h = headerFree(h);
+		continue;
+	    }
+
+	    /* Try to read a package manifest. */
+	    fd = Fopen(fileURL, "r.fpio");
+	    if (fd == NULL || Ferror(fd)) {
+		rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), fileURL,
+			Fstrerror(fd));
+		if (fd) (void) Fclose(fd);
+		retcode = 1;
+		/*@loopbreak@*/ break;
+	    }
+	    
+	    /* Read list of packages from manifest. */
+	    retcode = rpmReadPackageManifest(fd, &ac, &av);
+	    if (retcode) {
+		rpmError(RPMERR_MANIFEST, _("%s: read manifest failed: %s\n"),
+			fileURL, Fstrerror(fd));
+		retcode = 1;
+	    }
+	    (void) Fclose(fd);
+
+	    /* If successful, restart the query loop. */
+	    if (retcode == 0)
+		goto restart;
+
+	    /*@loopbreak@*/ break;
 	}
-	if (argv) {
-	    for (i = 0; i < argc; i++)
-		free((void *)argv[i]);
-	    free((void *)argv);
+
+	fileURL = _free(fileURL);
+	if (av) {
+	    for (i = 0; i < ac; i++)
+		av[i] = _free(av[i]);
+	    av = _free(av);
 	}
     }	break;
 
@@ -587,34 +649,32 @@ int rpmQueryVerify(QVA_t *qva, rpmQVSources source, const char * arg,
       { Spec spec = NULL;
 	Package pkg;
 	char * buildRoot = NULL;
-	int inBuildArch = 0;
+	int recursing = 0;
 	char * passPhrase = "";
 	char *cookie = NULL;
 	int anyarch = 1;
 	int force = 1;
 
-	rc = parseSpecVec(&spec, arg, "/", buildRoot, inBuildArch, passPhrase,
+	rc = parseSpecVec(&spec, arg, "/", buildRoot, recursing, passPhrase,
 		cookie, anyarch, force);
 	if (rc || spec == NULL) {
-	    
 	    rpmError(RPMERR_QUERY,
 	    		_("query of specfile %s failed, can't parse\n"), arg);
-	    if (spec != NULL) freeSpecVec(spec);
+	    spec = freeSpecVec(spec);
 	    retcode = 1;
 	    break;
 	}
 
 	if (specedit) {
 	    printNewSpecfile(spec);
-	    freeSpecVec(spec);
+	    spec = freeSpecVec(spec);
 	    retcode = 0;
 	    break;
 	}
 
-	for (pkg = spec->packages; pkg != NULL; pkg = pkg->next) {
-	    showPackage(qva, NULL, pkg->header);
-	}
-	freeSpecVec(spec);
+	for (pkg = spec->packages; pkg != NULL; pkg = pkg->next)
+	    (void) showPackage(qva, NULL, pkg->header);
+	spec = freeSpecVec(spec);
       }	break;
 
     case RPMQV_ALL:
@@ -624,7 +684,15 @@ int rpmQueryVerify(QVA_t *qva, rpmQVSources source, const char * arg,
 	    rpmError(RPMERR_QUERYINFO, _("no packages\n"));
 	    retcode = 1;
 	} else {
-	    retcode = showMatches(qva, mi, showPackage);
+	    for (av = (const char **) arg; av && *av; av++) {
+		if (!rpmdbSetIteratorRE(mi, RPMTAG_NAME, RPMMIRE_DEFAULT, *av))
+		    continue;
+		mi = rpmdbFreeIterator(mi);
+		retcode = 1;
+		/*@loopbreak@*/ break;
+	    }
+	    if (!retcode)
+		retcode = showMatches(qva, mi, showPackage);
 	}
 	break;
 
@@ -675,13 +743,17 @@ int rpmQueryVerify(QVA_t *qva, rpmQVSources source, const char * arg,
     {   const char * s;
 	char * fn;
 
-	for (s = arg; *s; s++)
-	    if (!(*s == '.' || *s == '/')) break;
+	for (s = arg; *s != '\0'; s++)
+	    if (!(*s == '.' || *s == '/'))
+		/*@loopbreak@*/ break;
 
 	if (*s == '\0') {
 	    char fnbuf[PATH_MAX];
 	    fn = /*@-unrecog@*/ realpath(arg, fnbuf) /*@=unrecog@*/;
-	    fn = xstrdup( (fn ? fn : arg) );
+	    if (fn)
+		fn = xstrdup(fn);
+	    else
+		fn = xstrdup(arg);
 	} else
 	    fn = xstrdup(arg);
 	(void) rpmCleanPath(fn);
@@ -705,13 +777,13 @@ int rpmQueryVerify(QVA_t *qva, rpmQVSources source, const char * arg,
 	} else {
 	    retcode = showMatches(qva, mi, showPackage);
 	}
-	free((void *)fn);
+	fn = _free(fn);
     }	break;
 
     case RPMQV_DBOFFSET:
     {	int mybase = 10;
 	const char * myarg = arg;
-	int recOffset;
+	unsigned recOffset;
 
 	/* XXX should be in strtoul */
 	if (*myarg == '0') {
@@ -732,7 +804,7 @@ int rpmQueryVerify(QVA_t *qva, rpmQVSources source, const char * arg,
 	mi = rpmdbInitIterator(rpmdb, RPMDBI_PACKAGES, &recOffset, sizeof(recOffset));
 	if (mi == NULL) {
 	    rpmError(RPMERR_QUERY,
-		_("record %d could not be read\n"), recOffset);
+		_("record %u could not be read\n"), recOffset);
 	    retcode = 1;
 	} else {
 	    retcode = showMatches(qva, mi, showPackage);
@@ -754,7 +826,7 @@ int rpmQueryVerify(QVA_t *qva, rpmQVSources source, const char * arg,
     return retcode;
 }
 
-int rpmQuery(QVA_t *qva, rpmQVSources source, const char * arg)
+int rpmQuery(QVA_t qva, rpmQVSources source, const char * arg)
 {
     rpmdb rpmdb = NULL;
     int rc;
@@ -771,8 +843,8 @@ int rpmQuery(QVA_t *qva, rpmQVSources source, const char * arg)
 
     rc = rpmQueryVerify(qva, source, arg, rpmdb, showQueryPackage);
 
-    if (rpmdb)
-	rpmdbClose(rpmdb);
+    if (rpmdb != NULL)
+	(void) rpmdbClose(rpmdb);
 
     return rc;
 }
