@@ -35,7 +35,7 @@ int writeLead(FD_t fd, struct rpmlead *lead)
     l.osnum = htons(l.osnum);
     l.signature_type = htons(l.signature_type);
 	
-    if (Fwrite(&l, sizeof(char), sizeof(l), fd) < 0) {
+    if (Fwrite(&l, sizeof(char), sizeof(l), fd) != sizeof(l)) {
 	return 1;
     }
 
@@ -44,6 +44,7 @@ int writeLead(FD_t fd, struct rpmlead *lead)
 
 int readLead(FD_t fd, struct rpmlead *lead)
 {
+    memset(lead, 0, sizeof(*lead));
     if (timedRead(fd, (char *)lead, sizeof(*lead)) != sizeof(*lead)) {
 	rpmError(RPMERR_READ, _("read failed: %s (%d)\n"), Fstrerror(fd), 
 	      errno);
