@@ -8,29 +8,33 @@
 #include "debug.h"
 
 static struct rpmlibProvides {
-    const char * featureName;
-    const char * featureEVR;
+/*@observer@*/ /*@null@*/ const char * featureName;
+/*@observer@*/ /*@null@*/ const char * featureEVR;
     int featureFlags;
-    const char * featureDescription;
+/*@observer@*/ /*@null@*/ const char * featureDescription;
 } rpmlibProvides[] = {
     { "rpmlib(VersionedDependencies)",	"3.0.3-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
-	"PreReq:, Provides:, and Obsoletes: dependencies support versions." },
+    N_("PreReq:, Provides:, and Obsoletes: dependencies support versions.") },
     { "rpmlib(CompressedFileNames)",	"3.0.4-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
-	"file names stored as (dirName,BaseName,dirIndex) tuple, not as path."},
+    N_("file names stored as (dirName,baseName,dirIndex) tuple, not as path.")},
     { "rpmlib(PayloadIsBzip2)",		"3.0.5-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
-	"package payload compressed using bzip2." },
+    N_("package payload compressed using bzip2.") },
     { "rpmlib(PayloadFilesHavePrefix)",	"4.0-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
-	"package payload files have \"./\" prefix." },
+    N_("package payload files have \"./\" prefix.") },
     { "rpmlib(ExplicitPackageProvide)",	"4.0-1",
 	(RPMSENSE_RPMLIB|RPMSENSE_EQUAL),
-	"package name-version-release not implicitly provided." },
-    { "rpmlib(HeaderLoadSortsTags)",    "4.0.1-1",      RPMSENSE_EQUAL,
-        "header tags are always sorted after being loaded." },
-    { NULL,				NULL,	0 }
+    N_("package name-version-release not implicitly provided.") },
+    { "rpmlib(HeaderLoadSortsTags)",    "4.0.1-1",
+	(                RPMSENSE_EQUAL),
+    N_("header tags are always sorted after being loaded.") },
+    { "rpmlib(ScriptletInterpreterArgs)",    "4.0.3-1",
+	(                RPMSENSE_EQUAL),
+    N_("the scriptlet interpreter can use arguments from header.") },
+    { NULL,				NULL, 0,	NULL }
 };
 
 void rpmShowRpmlibProvides(FILE * fp)
@@ -39,10 +43,11 @@ void rpmShowRpmlibProvides(FILE * fp)
 
     for (rlp = rpmlibProvides; rlp->featureName != NULL; rlp++) {
 	fprintf(fp, "    %s", rlp->featureName);
-	if (rlp->featureFlags)
+	if (rlp->featureEVR && rlp->featureFlags)
 	    printDepFlags(fp, rlp->featureEVR, rlp->featureFlags);
 	fprintf(fp, "\n");
-	fprintf(fp, "\t%s\n", rlp->featureDescription);
+	if (rlp->featureDescription)
+	    fprintf(fp, "\t%s\n", rlp->featureDescription);
     }
 }
 
@@ -53,7 +58,8 @@ int rpmCheckRpmlibProvides(const char * keyName, const char * keyEVR,
     int rc = 0;
 
     for (rlp = rpmlibProvides; rlp->featureName != NULL; rlp++) {
-	rc = rpmRangesOverlap(keyName, keyEVR, keyFlags,
+	if (rlp->featureEVR && rlp->featureFlags)
+	    rc = rpmRangesOverlap(keyName, keyEVR, keyFlags,
 		rlp->featureName, rlp->featureEVR, rlp->featureFlags);
 	if (rc)
 	    break;
@@ -71,9 +77,9 @@ int rpmGetRpmlibProvides(const char *** provNames, int ** provFlags,
     while (rpmlibProvides[n].featureName != NULL)
         n++;
 
-    names = xmalloc(sizeof(*names) * (n+1));
-    versions = xmalloc(sizeof(*versions) * (n+1));
-    flags = xmalloc(sizeof(*flags) * (n+1));
+    names = xcalloc((n+1), sizeof(*names));
+    versions = xcalloc((n+1), sizeof(*versions));
+    flags = xcalloc((n+1), sizeof(*flags));
     
     for (n = 0; rpmlibProvides[n].featureName != NULL; n++) {
         names[n] = rpmlibProvides[n].featureName;
@@ -81,12 +87,20 @@ int rpmGetRpmlibProvides(const char *** provNames, int ** provFlags,
         versions[n] = rpmlibProvides[n].featureEVR;
     }
     
-    names[n] = NULL;
-    versions[n] = NULL;
-    flags[n] = -1;
-    
-    *provNames = names;
-    *provFlags = flags;
-    *provVersions = versions;
+    if (provNames)
+	*provNames = names;
+    else
+	names = _free(names);
+
+    if (provFlags)
+	*provFlags = flags;
+    else
+	flags = _free(flags);
+
+    if (provVersions)
+	*provVersions = versions;
+    else
+	versions = _free(versions);
+
     return n;
 }
