@@ -14,77 +14,83 @@
 #include "misc.h"
 #include "debug.h"
 
-static const char *defrcfiles = LIBRPMRC_FILENAME ":/etc/rpmrc:~/.rpmrc";
+/*@access FD_t@*/		/* compared with NULL */
 
-const char * macrofiles = MACROFILES;
+/*@observer@*/ static const char *defrcfiles =
+	LIBRPMRC_FILENAME ":/etc/rpmrc:~/.rpmrc";
+
+/*@observer@*/ const char * macrofiles = MACROFILES;
 
 typedef /*@owned@*/ const char * cptr_t;
 
-struct machCacheEntry {
+typedef struct machCacheEntry_s {
     const char * name;
     int count;
     cptr_t * equivs;
     int visited;
-};
+} * machCacheEntry;
 
-struct machCache {
-    struct machCacheEntry * cache;
+typedef struct machCache_s {
+    machCacheEntry cache;
     int size;
-};
+} * machCache;
 
-struct machEquivInfo {
+typedef struct machEquivInfo_s {
     const char * name;
     int score;
-};
+} * machEquivInfo;
 
-struct machEquivTable {
+typedef struct machEquivTable_s {
     int count;
-    struct machEquivInfo * list;
-};
+    machEquivInfo list;
+} * machEquivTable;
 
 struct rpmvarValue {
     const char * value;
     /* eventually, this arch will be replaced with a generic condition */
     const char * arch;
-    struct rpmvarValue * next;
+/*@only@*/ /*@null@*/ struct rpmvarValue * next;
 };
 
 struct rpmOption {
     const char * name;
     int var;
-    int archSpecific, required, macroize, localize;
-    struct rpmOptionValue * value;
+    int archSpecific;
+/*@unused@*/ int required;
+    int macroize;
+    int localize;
+/*@unused@*/ struct rpmOptionValue * value;
 };
 
-struct defaultEntry {
-/*@owned@*/ const char * name;
-/*@owned@*/ const char * defName;
-};
+typedef struct defaultEntry_s {
+/*@owned@*/ /*@null@*/ const char * name;
+/*@owned@*/ /*@null@*/ const char * defName;
+} * defaultEntry;
 
-struct canonEntry {
+typedef struct canonEntry_s {
 /*@owned@*/ const char * name;
 /*@owned@*/ const char * short_name;
     short num;
-};
+} * canonEntry;
 
 /* tags are 'key'canon, 'key'translate, 'key'compat
  *
  * for giggles, 'key'_canon, 'key'_compat, and 'key'_canon will also work
  */
-struct tableType {
-    const char * const key;
+typedef struct tableType_s {
+/*@observer@*/ const char * const key;
     const int hasCanon;
     const int hasTranslate;
-    struct machEquivTable equiv;
-    struct machCache cache;
-    struct defaultEntry * defaults;
-    struct canonEntry * canons;
+    struct machEquivTable_s equiv;
+    struct machCache_s cache;
+    defaultEntry defaults;
+    canonEntry canons;
     int defaultsLength;
     int canonsLength;
-};
+} * tableType;
 
 /*@-fullinitblock@*/
-static struct tableType tables[RPM_MACHTABLE_COUNT] = {
+static struct tableType_s tables[RPM_MACHTABLE_COUNT] = {
     { "arch", 1, 0 },
     { "os", 1, 0 },
     { "buildarch", 0, 1 },
@@ -112,19 +118,26 @@ static struct rpmvarValue values[RPMVAR_NUM];
 static int defaultsInitialized = 0;
 
 /* prototypes */
-static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn);
-static void rpmSetVarArch(int var, const char * val, const char * arch);
-static void rebuildCompatTables(int type, const char *name);
+static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
+	/*@modifies fd, fileSystem @*/;
+static void rpmSetVarArch(int var, const char * val,
+		/*@null@*/ const char * arch)
+	/*@modifies internalState @*/;
+static void rebuildCompatTables(int type, const char * name)
+	/*@modifies internalState @*/;
 
-static int optionCompare(const void * a, const void * b) {
+static int optionCompare(const void * a, const void * b)
+	/*@*/
+{
     return xstrcasecmp(((struct rpmOption *) a)->name,
 		      ((struct rpmOption *) b)->name);
 }
 
-static void rpmRebuildTargetVars(const char **target, const char ** canontarget);
+static void rpmRebuildTargetVars(/*@null@*/ const char **target, /*@null@*/ const char ** canontarget);
 
-static /*@observer@*/ struct machCacheEntry *
-machCacheFindEntry(struct machCache * cache, const char * key)
+static /*@observer@*/ /*@null@*/ machCacheEntry
+machCacheFindEntry(const machCache cache, const char * key)
+	/*@*/
 {
     int i;
 
@@ -135,14 +148,16 @@ machCacheFindEntry(struct machCache * cache, const char * key)
 }
 
 static int machCompatCacheAdd(char * name, const char * fn, int linenum,
-				struct machCache * cache)
+				machCache cache)
+	/*@modifies *name, cache->cache, cache->size @*/
 {
-    char * chptr, * equivs;
+    machCacheEntry entry = NULL;
+    char * chptr;
+    char * equivs;
     int delEntry = 0;
     int i;
-    struct machCacheEntry * entry = NULL;
 
-    while (*name && isspace(*name)) name++;
+    while (*name && xisspace(*name)) name++;
 
     chptr = name;
     while (*chptr && *chptr != ':') chptr++;
@@ -155,10 +170,10 @@ static int machCompatCacheAdd(char * name, const char * fn, int linenum,
 	return 1;
     }
 
-    while (*chptr == ':' || isspace(*chptr)) chptr--;
+    while (*chptr == ':' || xisspace(*chptr)) chptr--;
     *(++chptr) = '\0';
     equivs = chptr + 1;
-    while (*equivs && isspace(*equivs)) equivs++;
+    while (*equivs && xisspace(*equivs)) equivs++;
     if (!*equivs) {
 	delEntry = 1;
     }
@@ -167,9 +182,8 @@ static int machCompatCacheAdd(char * name, const char * fn, int linenum,
 	entry = machCacheFindEntry(cache, name);
 	if (entry) {
 	    for (i = 0; i < entry->count; i++)
-		free((void *)entry->equivs[i]);
-	    free((void *)entry->equivs);
-	    entry->equivs = NULL;
+		entry->equivs[i] = _free(entry->equivs[i]);
+	    entry->equivs = _free(entry->equivs);
 	    entry->count = 0;
 	}
     }
@@ -202,8 +216,9 @@ static int machCompatCacheAdd(char * name, const char * fn, int linenum,
     return 0;
 }
 
-static /*@observer@*/ struct machEquivInfo *
-	machEquivSearch(const struct machEquivTable * table, const char * name)
+static /*@observer@*/ /*@null@*/ machEquivInfo
+machEquivSearch(const machEquivTable table, const char * name)
+	/*@*/
 {
     int i;
 
@@ -214,10 +229,11 @@ static /*@observer@*/ struct machEquivInfo *
     return NULL;
 }
 
-static void machAddEquiv(struct machEquivTable * table, const char * name,
+static void machAddEquiv(machEquivTable table, const char * name,
 			   int distance)
+	/*@modifies table->list, table->count @*/
 {
-    struct machEquivInfo * equiv;
+    machEquivInfo equiv;
 
     equiv = machEquivSearch(table, name);
     if (!equiv) {
@@ -232,12 +248,11 @@ static void machAddEquiv(struct machEquivTable * table, const char * name,
     }
 }
 
-static void machCacheEntryVisit(struct machCache * cache,
-				  struct machEquivTable * table,
-				  const char * name,
-	  			  int distance)
+static void machCacheEntryVisit(machCache cache,
+		machEquivTable table, const char * name, int distance)
+	/*@modifies table->list, table->count @*/
 {
-    struct machCacheEntry * entry;
+    machCacheEntry entry;
     int i;
 
     entry = machCacheFindEntry(cache, name);
@@ -254,9 +269,9 @@ static void machCacheEntryVisit(struct machCache * cache,
     }
 }
 
-static void machFindEquivs(struct machCache * cache,
-			     struct machEquivTable * table,
-			     const char * key)
+static void machFindEquivs(machCache cache, machEquivTable table,
+		const char * key)
+	/*@modifies cache->cache, table->list, table->count @*/
 {
     int i;
 
@@ -264,26 +279,29 @@ static void machFindEquivs(struct machCache * cache,
 	cache->cache[i].visited = 0;
 
     while (table->count > 0) {
-	free((void *)table->list[--table->count].name);
-	table->list[table->count].name = NULL;
+	--table->count;
+	table->list[table->count].name = _free(table->list[table->count].name);
     }
     table->count = 0;
-    if (table->list) free((void *)table->list);
-    table->list = NULL;
+    table->list = _free(table->list);
 
     /*
      *	We have a general graph built using strings instead of pointers.
      *	Yuck. We have to start at a point at traverse it, remembering how
      *	far away everything is.
      */
+    /*@-nullstate@*/	/* FIX: table->list may be NULL. */
     machAddEquiv(table, key, 1);
     machCacheEntryVisit(cache, table, key, 2);
+    return;
+    /*@=nullstate@*/
 }
 
-static int addCanon(struct canonEntry ** table, int * tableLen, char * line,
+static int addCanon(canonEntry * table, int * tableLen, char * line,
 		    const char * fn, int lineNum)
+	/*@modifies *table, *tableLen, *line @*/
 {
-    struct canonEntry *t;
+    canonEntry t;
     char *s, *s1;
     const char * tname;
     const char * tshort_name;
@@ -291,10 +309,12 @@ static int addCanon(struct canonEntry ** table, int * tableLen, char * line,
 
     if (! *tableLen) {
 	*tableLen = 2;
-	*table = xmalloc(2 * sizeof(struct canonEntry));
+	*table = xmalloc(2 * sizeof(struct canonEntry_s));
     } else {
 	(*tableLen) += 2;
-	*table = xrealloc(*table, sizeof(struct canonEntry) * (*tableLen));
+	/*@-unqualifiedtrans@*/
+	*table = xrealloc(*table, sizeof(struct canonEntry_s) * (*tableLen));
+	/*@=unqualifiedtrans@*/
     }
     t = & ((*table)[*tableLen - 2]);
 
@@ -312,40 +332,46 @@ static int addCanon(struct canonEntry ** table, int * tableLen, char * line,
 	return RPMERR_RPMRC;
     }
 
+    /*@-nullpass@*/	/* LCL: s != NULL here. */
     tnum = strtoul(s, &s1, 10);
     if ((*s1) || (s1 == s) || (tnum == ULONG_MAX)) {
 	rpmError(RPMERR_RPMRC, _("Bad arch/os number: %s (%s:%d)\n"), s,
 	      fn, lineNum);
 	return(RPMERR_RPMRC);
     }
+    /*@=nullpass@*/
 
     t[0].name = xstrdup(tname);
-    t[0].short_name = xstrdup(tshort_name);
+    t[0].short_name = (tshort_name ? xstrdup(tshort_name) : xstrdup(""));
     t[0].num = tnum;
 
     /* From A B C entry */
     /* Add  B B C entry */
-    t[1].name = xstrdup(tshort_name);
-    t[1].short_name = xstrdup(tshort_name);
+    t[1].name = (tshort_name ? xstrdup(tshort_name) : xstrdup(""));
+    t[1].short_name = (tshort_name ? xstrdup(tshort_name) : xstrdup(""));
     t[1].num = tnum;
 
     return 0;
 }
 
-static int addDefault(struct defaultEntry **table, int *tableLen, char *line,
-			const char *fn, int lineNum)
+static int addDefault(defaultEntry * table, int * tableLen, char * line,
+			const char * fn, int lineNum)
+	/*@modifies *table, *tableLen, *line @*/
 {
-    struct defaultEntry *t;
+    defaultEntry t;
 
     if (! *tableLen) {
 	*tableLen = 1;
-	*table = xmalloc(sizeof(struct defaultEntry));
+	*table = xmalloc(sizeof(struct defaultEntry_s));
     } else {
 	(*tableLen)++;
-	*table = xrealloc(*table, sizeof(struct defaultEntry) * (*tableLen));
+	/*@-unqualifiedtrans@*/
+	*table = xrealloc(*table, sizeof(struct defaultEntry_s) * (*tableLen));
+	/*@=unqualifiedtrans@*/
     }
     t = & ((*table)[*tableLen - 1]);
 
+    /*@-temptrans@*/
     t->name = strtok(line, ": \t");
     t->defName = strtok(NULL, " \t");
     if (! (t->name && t->defName)) {
@@ -360,32 +386,37 @@ static int addDefault(struct defaultEntry **table, int *tableLen, char *line,
     }
 
     t->name = xstrdup(t->name);
-    t->defName = xstrdup(t->defName);
+    t->defName = (t->defName ? xstrdup(t->defName) : NULL);
+    /*@=temptrans@*/
 
     return 0;
 }
 
-static /*@null@*/ const struct canonEntry *lookupInCanonTable(const char *name,
-	const struct canonEntry *table, int tableLen)
+static /*@null@*/ const canonEntry lookupInCanonTable(const char * name,
+		const canonEntry table, int tableLen)
+	/*@*/
 {
     while (tableLen) {
 	tableLen--;
-	if (!strcmp(name, table[tableLen].name)) {
-	    return &(table[tableLen]);
-	}
+	if (strcmp(name, table[tableLen].name))
+	    continue;
+	/*@-immediatetrans -retalias@*/
+	return &(table[tableLen]);
+	/*@=immediatetrans =retalias@*/
     }
 
     return NULL;
 }
 
-static /*@observer@*/ const char * lookupInDefaultTable(const char *name,
-		const struct defaultEntry *table, int tableLen)
+static /*@observer@*/ /*@null@*/
+const char * lookupInDefaultTable(const char * name,
+		const defaultEntry table, int tableLen)
+	/*@*/
 {
     while (tableLen) {
 	tableLen--;
-	if (!strcmp(name, table[tableLen].name)) {
+	if (table[tableLen].name && !strcmp(name, table[tableLen].name))
 	    return table[tableLen].defName;
-	}
     }
 
     return name;
@@ -395,6 +426,7 @@ int rpmReadConfigFiles(const char * file, const char * target)
 {
 
     /* Preset target macros */
+    /*@-nullstate@*/	/* FIX: target can be NULL */
     rpmRebuildTargetVars(&target, NULL);
 
     /* Read the files */
@@ -402,19 +434,22 @@ int rpmReadConfigFiles(const char * file, const char * target)
 
     /* Reset target macros */
     rpmRebuildTargetVars(&target, NULL);
+    /*@=nullstate@*/
 
     /* Finally set target platform */
     {	const char *cpu = rpmExpand("%{_target_cpu}", NULL);
 	const char *os = rpmExpand("%{_target_os}", NULL);
 	rpmSetMachine(cpu, os);
-	free((void *)cpu);
-	free((void *)os);
+	cpu = _free(cpu);
+	os = _free(os);
     }
 
     return 0;
 }
 
-static void setVarDefault(int var, const char *macroname, const char *val, const char *body)
+static void setVarDefault(int var, const char * macroname, const char * val,
+		/*@null@*/ const char * body)
+	/*@modifies internalState @*/
 {
     if (var >= 0) {	/* XXX Dying ... */
 	if (rpmGetVar(var)) return;
@@ -425,7 +460,8 @@ static void setVarDefault(int var, const char *macroname, const char *val, const
     addMacro(NULL, macroname, NULL, body, RMIL_DEFAULT);
 }
 
-static void setPathDefault(int var, const char *macroname, const char *subdir)
+static void setPathDefault(int var, const char * macroname, const char * subdir)
+	/*@modifies internalState @*/
 {
 
     if (var >= 0) {	/* XXX Dying ... */
@@ -443,7 +479,7 @@ static void setPathDefault(int var, const char *macroname, const char *subdir)
 	strcat(fn, subdir);
 
 	rpmSetVar(var, fn);
-	if (topdir)	free((void *)topdir);
+	topdir = _free(topdir);
     }
 
     if (macroname != NULL) {
@@ -456,7 +492,7 @@ static void setPathDefault(int var, const char *macroname, const char *subdir)
     }
 }
 
-static const char *prescriptenviron = "\n\
+/*@observer@*/ static const char * prescriptenviron = "\n\
 RPM_SOURCE_DIR=\"%{_sourcedir}\"\n\
 RPM_BUILD_DIR=\"%{_builddir}\"\n\
 RPM_OPT_FLAGS=\"%{optflags}\"\n\
@@ -473,7 +509,9 @@ export RPM_PACKAGE_NAME RPM_PACKAGE_VERSION RPM_PACKAGE_RELEASE\n\
 export RPM_BUILD_ROOT\n}\
 ";
 
-static void setDefaults(void) {
+static void setDefaults(void)
+	/*@modifies internalState @*/
+{
 
     addMacro(NULL, "_usr", NULL, "/usr", RMIL_DEFAULT);
     addMacro(NULL, "_var", NULL, "/var", RMIL_DEFAULT);
@@ -522,14 +560,14 @@ int rpmReadRC(const char * rcfiles)
 
     /* Read each file in rcfiles. */
     rc = 0;
-    for (r = myrcfiles = xstrdup(rcfiles); *r != '\0'; r = re) {
+    for (r = myrcfiles = xstrdup(rcfiles); r && *r != '\0'; r = re) {
 	char fn[4096];
 	FD_t fd;
 
 	/* Get pointer to rest of files */
 	for (re = r; (re = strchr(re, ':')) != NULL; re++) {
 	    if (!(re[1] == '/' && re[2] == '/'))
-		break;
+		/*@innerbreak@*/ break;
 	}
 	if (re && *re == ':')
 	    *re++ = '\0';
@@ -575,24 +613,26 @@ int rpmReadRC(const char * rcfiles)
 	}
  	if (rc) break;
     }
-    if (myrcfiles)	free(myrcfiles);
+    myrcfiles = _free(myrcfiles);
     if (rc)
 	return rc;
 
     rpmSetMachine(NULL, NULL);	/* XXX WTFO? Why bother? */
 
-    {	const char *macrofiles;
-	if ((macrofiles = rpmGetVar(RPMVAR_MACROFILES)) != NULL) {
-	    macrofiles = strdup(macrofiles);
-	    rpmInitMacros(NULL, macrofiles);
-	    free((void *)macrofiles);
+    {	const char *mfpath;
+	if ((mfpath = rpmGetVar(RPMVAR_MACROFILES)) != NULL) {
+	    mfpath = xstrdup(mfpath);
+	    rpmInitMacros(NULL, mfpath);
+	    mfpath = _free(mfpath);
 	}
     }
 
     return rc;
 }
 
+/*@-usedef@*/	/*@ FIX: se usage inconsistent, W2DO? */
 static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
+	/*@modifies fd, fileSystem @*/
 {
     const char *s;
     char *se, *next;
@@ -604,7 +644,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
   { off_t size = fdSize(fd);
     size_t nb = (size >= 0 ? size : (8*BUFSIZ - 2));
     if (nb == 0) {
-	Fclose(fd);
+	(void) Fclose(fd);
 	return 0;
     }
     next = alloca(nb + 2);
@@ -616,44 +656,44 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 	rc = 1;
     } else
 	rc = 0;
-    Fclose(fd);
+    (void) Fclose(fd);
     if (rc) return rc;
     next[nb] = '\n';
     next[nb + 1] = '\0';
   }
 
-    while (*next) {
+    while (*next != '\0') {
 	linenum++;
 
 	s = se = next;
 
 	/* Find end-of-line. */
 	while (*se && *se != '\n') se++;
-	if (*se) *se++ = '\0';
+	if (*se != '\0') *se++ = '\0';
 	next = se;
 
 	/* Trim leading spaces */
-	while (*s && isspace(*s)) s++;
+	while (*s && xisspace(*s)) s++;
 
 	/* We used to allow comments to begin anywhere, but not anymore. */
 	if (*s == '#' || *s == '\0') continue;
 
 	/* Find end-of-keyword. */
 	se = (char *)s;
-	while (*se && !isspace(*se) && *se != ':') se++;
+	while (*se && !xisspace(*se) && *se != ':') se++;
 
-	if (isspace(*se)) {
+	if (xisspace(*se)) {
 	    *se++ = '\0';
-	    while (*se && isspace(*se) && *se != ':') se++;
+	    while (*se && xisspace(*se) && *se != ':') se++;
 	}
 
 	if (*se != ':') {
 	    rpmError(RPMERR_RPMRC, _("missing ':' (found 0x%02x) at %s:%d\n"),
-		     (0xff & *se), urlfn, linenum);
+		     (unsigned)(0xff & *se), urlfn, linenum);
 	    return 1;
 	}
 	*se++ = '\0';	/* terminate keyword or option, point to value */
-	while (*se && isspace(*se)) se++;
+	while (*se && xisspace(*se)) se++;
 
 	/* Find keyword in table */
 	searchOption.name = s;
@@ -675,8 +715,8 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 	      {	FD_t fdinc;
 
 		s = se;
-		while (*se && !isspace(*se)) se++;
-		if (*se) *se++ = '\0';
+		while (*se && !xisspace(*se)) se++;
+		if (*se != '\0') *se++ = '\0';
 
 		rpmRebuildTargetVars(NULL, NULL);
 
@@ -684,7 +724,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		if (fn == NULL || *fn == '\0') {
 		    rpmError(RPMERR_RPMRC, _("%s expansion failed at %s:%d \"%s\"\n"),
 			option->name, urlfn, linenum, s);
-		    if (fn) free((void *)fn);
+		    fn = _free(fn);
 		    return 1;
 		    /*@notreached@*/
 		}
@@ -697,7 +737,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		} else {
 		    rc = doReadRC(fdinc, fn);
 		}
-		if (fn) free((void *)fn);
+		fn = _free(fn);
 		if (rc) return rc;
 		continue;	/* XXX don't save include value as var/macro */
 	      }	/*@notreached@*/ break;
@@ -706,7 +746,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		if (fn == NULL || *fn == '\0') {
 		    rpmError(RPMERR_RPMRC, _("%s expansion failed at %s:%d \"%s\"\n"),
 			option->name, urlfn, linenum, fn);
-		    if (fn) free((void *)fn);
+		    fn = _free(fn);
 		    return 1;
 		}
 		se = (char *)fn;
@@ -716,9 +756,9 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		s = rpmGetVar(RPMVAR_PROVIDES);
 		if (s == NULL) s = "";
 		fn = t = xmalloc(strlen(s) + strlen(se) + 2);
-		while (*s) *t++ = *s++;
+		while (*s != '\0') *t++ = *s++;
 		*t++ = ' ';
-		while (*se) *t++ = *se++;
+		while (*se != '\0') *t++ = *se++;
 		*t++ = '\0';
 		se = (char *)fn;
 	      }	break;
@@ -728,7 +768,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 
 	    if (option->archSpecific) {
 		arch = se;
-		while (*se && !isspace(*se)) se++;
+		while (*se && !xisspace(*se)) se++;
 		if (*se == '\0') {
 		    rpmError(RPMERR_RPMRC,
 				_("missing architecture for %s at %s:%d\n"),
@@ -736,7 +776,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		    return 1;
 		}
 		*se++ = '\0';
-		while (*se && isspace(*se)) se++;
+		while (*se && xisspace(*se)) se++;
 		if (*se == '\0') {
 		    rpmError(RPMERR_RPMRC,
 				_("missing argument for %s at %s:%d\n"),
@@ -759,7 +799,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		free(name);
 	    }
 	    rpmSetVarArch(option->var, val, arch);
-	    if (fn) free((void *)fn);
+	    fn = _free(fn);
 
 	} else {	/* For arch/os compatibilty tables ... */
 	    int gotit;
@@ -769,7 +809,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 
 	    for (i = 0; i < RPM_MACHTABLE_COUNT; i++) {
 		if (!strncmp(tables[i].key, s, strlen(tables[i].key)))
-		    break;
+		    /*@innerbreak@*/ break;
 	    }
 
 	    if (i < RPM_MACHTABLE_COUNT) {
@@ -806,6 +846,7 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 
     return 0;
 }
+/*@=usedef@*/
 
 #	if defined(__linux__) && defined(__i386__)
 #include <setjmp.h>
@@ -815,7 +856,11 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
  * Generic CPUID function
  */
 static inline void cpuid(int op, int *eax, int *ebx, int *ecx, int *edx)
+	/*@modifies *eax, *ebx, *ecx, *edx @*/
 {
+#ifdef	__LCLINT__
+    *eax = *ebx = *ecx = *edx = 0;
+#endif
 #ifdef PIC
 	__asm__("pushl %%ebx; cpuid; movl %%ebx,%1; popl %%ebx"
 		: "=a"(*eax), "=g"(*ebx), "=&c"(*ecx), "=&d"(*edx)
@@ -832,6 +877,7 @@ static inline void cpuid(int op, int *eax, int *ebx, int *ecx, int *edx)
  * CPUID functions returning a single datum
  */
 static inline unsigned int cpuid_eax(unsigned int op)
+	/*@*/
 {
 	unsigned int val;
 
@@ -846,6 +892,7 @@ static inline unsigned int cpuid_eax(unsigned int op)
 }
 
 static inline unsigned int cpuid_ebx(unsigned int op)
+	/*@*/
 {
 	unsigned int tmp, val;
 
@@ -860,6 +907,7 @@ static inline unsigned int cpuid_ebx(unsigned int op)
 }
 
 static inline unsigned int cpuid_ecx(unsigned int op)
+	/*@*/
 {
 	unsigned int tmp, val;
 #ifdef PIC
@@ -874,6 +922,7 @@ static inline unsigned int cpuid_ecx(unsigned int op)
 }
 
 static inline unsigned int cpuid_edx(unsigned int op)
+	/*@*/
 {
 	unsigned int tmp, val;
 #ifdef PIC
@@ -890,11 +939,13 @@ static inline unsigned int cpuid_edx(unsigned int op)
 static sigjmp_buf jenv;
 
 static inline void model3(int _unused)
+	/*@modifies internalState @*/
 {
 	siglongjmp(jenv, 1);
 }
 
 static inline int RPMClass(void)
+	/*@modifies internalState @*/
 {
 	int cpu;
 	unsigned int tfms, junk, cap;
@@ -919,28 +970,60 @@ static inline int RPMClass(void)
 	return 5;
 }
 
+/* should only be called for model 6 CPU's */
+static int is_athlon(void)
+	/*@*/
+{
+	unsigned int eax, ebx, ecx, edx;
+	char vendor[16];
+	int i;
+	
+	cpuid (0, &eax, &ebx, &ecx, &edx);
+
+ 	/* If you care about space, you can just check ebx, ecx and edx directly
+ 	   instead of forming a string first and then doing a strcmp */
+ 	memset(vendor, 0, sizeof(vendor));
+ 	
+ 	for (i=0; i<4; i++)
+ 		vendor[i] = (unsigned char) (ebx >>(8*i));
+ 	for (i=0; i<4; i++)
+ 		vendor[4+i] = (unsigned char) (edx >>(8*i));
+ 	for (i=0; i<4; i++)
+ 		vendor[8+i] = (unsigned char) (ecx >>(8*i));
+ 		
+ 	if (strcmp(vendor, "AuthenticAMD") != 0)  
+ 		return 0;
+
+	return 1;
+}
+
 #endif
 
-static void defaultMachine(/*@out@*/ const char ** arch, /*@out@*/ const char ** os)
+static void defaultMachine(/*@out@*/ const char ** arch,
+		/*@out@*/ const char ** os)
+	/*@modifies *arch, *os @*/
 {
     static struct utsname un;
     static int gotDefaults = 0;
     char * chptr;
-    const struct canonEntry * canon;
+    canonEntry canon;
+    int rc;
 
     if (!gotDefaults) {
-	uname(&un);
+	rc = uname(&un);
+	if (rc < 0) return;
 
 #if !defined(__linux__)
 #ifdef SNI
 	/* USUALLY un.sysname on sinix does start with the word "SINIX"
 	 * let's be absolutely sure
 	 */
-	sprintf(un.sysname,"SINIX");
+	strncpy(un.sysname, "SINIX", sizeof(un.sysname));
 #endif
+	/*@-nullpass@*/
 	if (!strcmp(un.sysname, "AIX")) {
 	    strcpy(un.machine, __power_pc() ? "ppc" : "rs6000");
-	    sprintf(un.sysname,"aix%s.%s",un.version,un.release);
+	    sprintf(un.sysname,"aix%s.%s", un.version, un.release);
 	}
 	else if (!strcmp(un.sysname, "SunOS")) {
 	    if (!strncmp(un.release,"4", 1)) /* SunOS 4.x */ {
@@ -948,7 +1031,7 @@ static void defaultMachine(/*@out@*/ const char ** arch, /*@out@*/ const char **
 		for (fd = 0;
 		    (un.release[fd] != 0 && (fd < sizeof(un.release)));
 		    fd++) {
-		      if (!isdigit(un.release[fd]) && (un.release[fd] != '.')) {
+		      if (!xisdigit(un.release[fd]) && (un.release[fd] != '.')) {
 			un.release[fd] = 0;
 			break;
 		      }
@@ -962,10 +1045,10 @@ static void defaultMachine(/*@out@*/ const char ** arch, /*@out@*/ const char **
 	}
 	else if (!strcmp(un.sysname, "HP-UX"))
 	    /*make un.sysname look like hpux9.05 for example*/
-	    sprintf(un.sysname, "hpux%s", strpbrk(un.release,"123456789"));
+	    sprintf(un.sysname, "hpux%s", strpbrk(un.release, "123456789"));
 	else if (!strcmp(un.sysname, "OSF1"))
 	    /*make un.sysname look like osf3.2 for example*/
-	    sprintf(un.sysname,"osf%s",strpbrk(un.release,"123456789"));
+	    sprintf(un.sysname, "osf%s", strpbrk(un.release, "123456789"));
 	else if (!strncmp(un.sysname, "IP", 2))
 	    un.sysname[2] = '\0';
 	else if (!strncmp(un.sysname, "SINIX", 5)) {
@@ -978,33 +1061,34 @@ static void defaultMachine(/*@out@*/ const char ** arch, /*@out@*/ const char **
 		!strncmp(un.release, "4.0", 3))
 	{
 	    /* we are on ncr-sysv4 */
-	    char *prelid = NULL;
-	    FD_t fd;
-	    fd = Fopen("/etc/.relid", "r.fdio");
-	    if (!Ferror(fd)) {
-		chptr = (char *) xcalloc(1, 256);
-		if (chptr != NULL) {
-		    int irelid = Fread(chptr, sizeof(*chptr), 256, fd);
-		    Fclose(fd);
+	    char * prelid = NULL;
+	    FD_t fd = Fopen("/etc/.relid", "r.fdio");
+	    int gotit = 0;
+	    if (fd != NULL && !Ferror(fd)) {
+		chptr = xcalloc(1, 256);
+		{   int irelid = Fread(chptr, sizeof(*chptr), 256, fd);
+		    (void) Fclose(fd);
 		    /* example: "112393 RELEASE 020200 Version 01 OS" */
 		    if (irelid > 0) {
-			if ((prelid=strstr(chptr, "RELEASE "))){
+			if ((prelid = strstr(chptr, "RELEASE "))){
 			    prelid += strlen("RELEASE ")+1;
 			    sprintf(un.sysname,"ncr-sysv4.%.*s",1,prelid);
+			    gotit = 1;
 			}
 		    }
-		    free (chptr);
 		}
+		chptr = _free (chptr);
 	    }
-	    if (prelid == NULL)	/* parsing /etc/.relid file failed? */
+	    if (!gotit)	/* parsing /etc/.relid file failed? */
 		strcpy(un.sysname,"ncr-sysv4");
 	    /* wrong, just for now, find out how to look for i586 later*/
 	    strcpy(un.machine,"i486");
 	}
+	/*@=nullpass@*/
 #endif	/* __linux__ */
 
 	/* get rid of the hyphens in the sysname */
-	for (chptr = un.machine; *chptr; chptr++)
+	for (chptr = un.machine; *chptr != '\0'; chptr++)
 	    if (*chptr == '/') *chptr = '-';
 
 #	if defined(__MIPSEL__) || defined(__MIPSEL) || defined(_MIPSEL)
@@ -1110,11 +1194,28 @@ static void defaultMachine(/*@out@*/ const char ** arch, /*@out@*/ const char **
 	{
 	    char class = (char) (RPMClass() | '0');
 
-	    if (strchr("3456", un.machine[1]) && un.machine[1] != class)
+	    if (class == '6' && is_athlon())
+	    	strcpy(un.machine, "athlon");
+	    else if (strchr("3456", un.machine[1]) && un.machine[1] != class)
 		un.machine[1] = class;
 	}
 #	endif
 
+#	if defined(__linux__) && defined(__powerpc__)
+	{
+	    unsigned pvr;
+	    __asm__ __volatile__ ("mfspr %0, 287" : "=r" (pvr));
+
+	    pvr >>= 16;
+	    if ( pvr >= 0x40)
+		strcpy(un.machine, "ppcpseries");
+	    else if ( (pvr == 0x36) || (pvr == 0x37) )
+		strcpy(un.machine, "ppciseries");
+	    else
+		strcpy(un.machine, "ppc");
+	}
+#	endif
+
 	/* the uname() result goes through the arch_canon table */
 	canon = lookupInCanonTable(un.machine,
 				   tables[RPM_MACHTABLE_INSTARCH].canons,
@@ -1134,10 +1235,13 @@ static void defaultMachine(/*@out@*/ const char ** arch, /*@out@*/ const char **
     if (os) *os = un.sysname;
 }
 
-static const char * rpmGetVarArch(int var, const char * arch) {
-    struct rpmvarValue * next;
+static /*@observer@*/ /*@null@*/
+const char * rpmGetVarArch(int var, /*@null@*/ const char * arch)
+	/*@*/
+{
+    const struct rpmvarValue * next;
 
-    if (!arch) arch = current[ARCH];
+    if (arch == NULL) arch = current[ARCH];
 
     if (arch) {
 	next = &values[var];
@@ -1159,31 +1263,32 @@ const char *rpmGetVar(int var)
 }
 
 /* this doesn't free the passed pointer! */
-static void freeRpmVar(struct rpmvarValue * orig) {
+static void freeRpmVar(/*@only@*/ struct rpmvarValue * orig)
+	/*@modifies *orig @*/
+{
     struct rpmvarValue * next, * var = orig;
 
     while (var) {
 	next = var->next;
-	if (var->arch) {
-	    free((void *)var->arch);
-	    var->arch = NULL;
-	}
-	if (var->value) {
-	    free((void *)var->value);
-	    var->value = NULL;
-	}
+	var->arch = _free(var->arch);
+	var->value = _free(var->value);
 
-	if (var != orig) free(var);
+	if (var != orig) var = _free(var);
 	var = next;
     }
 }
 
-void rpmSetVar(int var, const char *val) {
+void rpmSetVar(int var, const char * val)
+{
+    /*@-immediatetrans@*/
     freeRpmVar(&values[var]);
+    /*@=immediatetrans@*/
     values[var].value = (val ? xstrdup(val) : NULL);
 }
 
-static void rpmSetVarArch(int var, const char * val, const char * arch) {
+static void rpmSetVarArch(int var, const char * val, const char * arch)
+	/*@*/
+{
     struct rpmvarValue * next = values + var;
 
     if (next->value) {
@@ -1199,9 +1304,11 @@ static void rpmSetVarArch(int var, const char * val, const char * arch) {
 	    }
 	}
 
+	/*@-nullpass@*/	/* LCL: arch != NULL here. */
 	if (next->arch && arch && !strcmp(next->arch, arch)) {
-	    if (next->value) free((void *)next->value);
-	    if (next->arch) free((void *)next->arch);
+	/*@=nullpass@*/
+	    next->value = _free(next->value);
+	    next->arch = _free(next->arch);
 	} else if (next->arch || arch) {
 	    next->next = xmalloc(sizeof(*next->next));
 	    next = next->next;
@@ -1215,7 +1322,8 @@ static void rpmSetVarArch(int var, const char * val, const char * arch) {
     next->arch = (arch ? xstrdup(arch) : NULL);
 }
 
-void rpmSetTables(int archTable, int osTable) {
+void rpmSetTables(int archTable, int osTable)
+{
     const char * arch, * os;
 
     defaultMachine(&arch, &os);
@@ -1231,12 +1339,13 @@ void rpmSetTables(int archTable, int osTable) {
     }
 }
 
-int rpmMachineScore(int type, const char * name) {
-    struct machEquivInfo * info = machEquivSearch(&tables[type].equiv, name);
+int rpmMachineScore(int type, const char * name)
+{
+    machEquivInfo info = machEquivSearch(&tables[type].equiv, name);
     return (info != NULL ? info->score : 0);
 }
 
-void rpmGetMachine(const char **arch, const char **os)
+void rpmGetMachine(const char ** arch, const char ** os)
 {
     if (arch)
 	*arch = current[ARCH];
@@ -1245,7 +1354,8 @@ void rpmGetMachine(const char **arch, const char **os)
 	*os = current[OS];
 }
 
-void rpmSetMachine(const char * arch, const char * os) {
+void rpmSetMachine(const char * arch, const char * os)
+{
     const char * host_cpu, * host_os;
 
     defaultMachine(&host_cpu, &host_os);
@@ -1257,6 +1367,7 @@ void rpmSetMachine(const char * arch, const char * os) {
 			    tables[currTables[ARCH]].defaults,
 			    tables[currTables[ARCH]].defaultsLength);
     }
+    if (arch == NULL) return;	/* XXX can't happen */
 
     if (os == NULL) {
 	os = host_os;
@@ -1265,16 +1376,17 @@ void rpmSetMachine(const char * arch, const char * os) {
 			    tables[currTables[OS]].defaults,
 			    tables[currTables[OS]].defaultsLength);
     }
+    if (os == NULL) return;	/* XXX can't happen */
 
     if (!current[ARCH] || strcmp(arch, current[ARCH])) {
-	if (current[ARCH]) free((void *)current[ARCH]);
+	current[ARCH] = _free(current[ARCH]);
 	current[ARCH] = xstrdup(arch);
 	rebuildCompatTables(ARCH, host_cpu);
     }
 
     if (!current[OS] || strcmp(os, current[OS])) {
 	char * t = xstrdup(os);
-	if (current[OS]) free((void *)current[OS]);
+	current[OS] = _free(current[OS]);
 	/*
 	 * XXX Capitalizing the 'L' is needed to insure that old
 	 * XXX os-from-uname (e.g. "Linux") is compatible with the new
@@ -1291,16 +1403,19 @@ void rpmSetMachine(const char * arch, const char * os) {
     }
 }
 
-static void rebuildCompatTables(int type, const char * name) {
+static void rebuildCompatTables(int type, const char * name)
+	/*@*/
+{
     machFindEquivs(&tables[currTables[type]].cache,
 		   &tables[currTables[type]].equiv,
 		   name);
 }
 
-static void getMachineInfo(int type, /*@out@*/ const char ** name,
-			/*@out@*/int * num)
+static void getMachineInfo(int type, /*@null@*/ /*@out@*/ const char ** name,
+			/*@null@*/ /*@out@*/int * num)
+	/*@modifies *name, *num @*/
 {
-    const struct canonEntry * canon;
+    canonEntry canon;
     int which = currTables[type];
 
     /* use the normal canon tables, even if we're looking up build stuff */
@@ -1324,15 +1439,17 @@ static void getMachineInfo(int type, /*@out@*/ const char ** name,
     }
 }
 
-void rpmGetArchInfo(const char ** name, int * num) {
+void rpmGetArchInfo(const char ** name, int * num)
+{
     getMachineInfo(ARCH, name, num);
 }
 
-void rpmGetOsInfo(const char ** name, int * num) {
+void rpmGetOsInfo(const char ** name, int * num)
+{
     getMachineInfo(OS, name, num);
 }
 
-void rpmRebuildTargetVars(const char **buildtarget, const char ** canontarget)
+void rpmRebuildTargetVars(const char ** target, const char ** canontarget)
 {
 
     char *ca = NULL, *co = NULL, *ct = NULL;
@@ -1344,10 +1461,10 @@ void rpmRebuildTargetVars(const char **buildtarget, const char ** canontarget)
     rpmSetTables(RPM_MACHTABLE_INSTARCH, RPM_MACHTABLE_INSTOS);
     rpmSetTables(RPM_MACHTABLE_BUILDARCH, RPM_MACHTABLE_BUILDOS);
 
-    if (buildtarget && *buildtarget) {
+    if (target && *target) {
 	char *c;
 	/* Set arch and os from specified build target */
-	ca = xstrdup(*buildtarget);
+	ca = xstrdup(*target);
 	if ((c = strchr(ca, '-')) != NULL) {
 	    *c++ = '\0';
 	    
@@ -1379,16 +1496,16 @@ void rpmRebuildTargetVars(const char **buildtarget, const char ** canontarget)
 	defaultMachine(&a, NULL);
 	ca = (a) ? xstrdup(a) : NULL;
     }
-    for (x = 0; ca[x]; x++)
-	ca[x] = tolower(ca[x]);
+    for (x = 0; ca[x] != '\0'; x++)
+	ca[x] = xtolower(ca[x]);
 
     if (co == NULL) {
 	const char *o = NULL;
 	defaultMachine(NULL, &o);
 	co = (o) ? xstrdup(o) : NULL;
     }
-    for (x = 0; co[x]; x++)
-	co[x] = tolower(co[x]);
+    for (x = 0; co[x] != '\0'; x++)
+	co[x] = xtolower(co[x]);
 
     /* XXX For now, set canonical target to arch-os */
     if (ct == NULL) {
@@ -1419,9 +1536,11 @@ void rpmRebuildTargetVars(const char **buildtarget, const char ** canontarget)
     if (canontarget)
 	*canontarget = ct;
     else
-	free(ct);
-    free(ca);
-    free(co);
+	ct = _free(ct);
+    ca = _free(ca);
+    /*@-usereleased@*/
+    co = _free(co);
+    /*@=usereleased@*/
 }
 
 void rpmFreeRpmrc(void)
@@ -1429,83 +1548,71 @@ void rpmFreeRpmrc(void)
     int i, j, k;
 
     for (i = 0; i < RPM_MACHTABLE_COUNT; i++) {
-	struct tableType *t;
+	tableType t;
 	t = tables + i;
 	if (t->equiv.list) {
-	    for (j = 0; j < t->equiv.count; j++) {
-		if (t->equiv.list[j].name) free((void *)t->equiv.list[j].name);
-	    }
-	    free((void *)t->equiv.list);
-	    t->equiv.list = NULL;
+	    for (j = 0; j < t->equiv.count; j++)
+		t->equiv.list[j].name = _free(t->equiv.list[j].name);
+	    t->equiv.list = _free(t->equiv.list);
 	    t->equiv.count = 0;
 	}
 	if (t->cache.cache) {
 	    for (j = 0; j < t->cache.size; j++) {
-		struct machCacheEntry *e;
+		machCacheEntry e;
 		e = t->cache.cache + j;
 		if (e == NULL)	continue;
-		if (e->name)		free((void *)e->name);
+		e->name = _free(e->name);
 		if (e->equivs) {
-		    for (k = 0; k < e->count; k++) {
-			if (e->equivs[k])	free((void *)e->equivs[k]);
-		    }
-		    free((void *)e->equivs);
+		    for (k = 0; k < e->count; k++)
+			e->equivs[k] = _free(e->equivs[k]);
+		    e->equivs = _free(e->equivs);
 		}
 	    }
-	    free((void *)t->cache.cache);
-	    t->cache.cache = NULL;
+	    t->cache.cache = _free(t->cache.cache);
 	    t->cache.size = 0;
 	}
 	if (t->defaults) {
 	    for (j = 0; j < t->defaultsLength; j++) {
-		if (t->defaults[j].name) free((void *)t->defaults[j].name);
-		if (t->defaults[j].defName) free((void *)t->defaults[j].defName);
+		t->defaults[j].name = _free(t->defaults[j].name);
+		t->defaults[j].defName = _free(t->defaults[j].defName);
 	    }
-	    free((void *)t->defaults);
-	    t->defaults = NULL;
+	    t->defaults = _free(t->defaults);
 	    t->defaultsLength = 0;
 	}
 	if (t->canons) {
 	    for (j = 0; j < t->canonsLength; j++) {
-		if (t->canons[j].name)	free((void *)t->canons[j].name);
-		if (t->canons[j].short_name)	free((void *)t->canons[j].short_name);
+		t->canons[j].name = _free(t->canons[j].name);
+		t->canons[j].short_name = _free(t->canons[j].short_name);
 	    }
-	    free((void *)t->canons);
-	    t->canons = NULL;
+	    t->canons = _free(t->canons);
 	    t->canonsLength = 0;
 	}
     }
 
     for (i = 0; i < RPMVAR_NUM; i++) {
-	struct rpmvarValue *this;
-	while ((this = values[i].next) != NULL) {
-	    values[i].next = this->next;
-	    if (this->value)	free((void *)this->value);
-	    if (this->arch)	free((void *)this->arch);
-	    free((void *)this);
+	/*@only@*/ /*@null@*/ struct rpmvarValue * vp;
+	while ((vp = values[i].next) != NULL) {
+	    values[i].next = vp->next;
+	    vp->value = _free(vp->value);
+	    vp->arch = _free(vp->arch);
+	    vp = _free(vp);
 	}
-	if (values[i].value)
-	    free((void *)values[i].value);
-	values[i].value = NULL;
-	if (values[i].arch)
-	    free((void *)values[i].arch);
-	values[i].arch = NULL;
+	values[i].value = _free(values[i].value);
+	values[i].arch = _free(values[i].arch);
     }
-    if (current[OS])
-	free((void *)current[OS]);
-    current[OS] = NULL;
-    if (current[ARCH])
-	free((void *)current[ARCH]);
-    current[ARCH] = NULL;
+    current[OS] = _free(current[OS]);
+    current[ARCH] = _free(current[ARCH]);
     defaultsInitialized = 0;
+/*@-nullstate@*/ /* FIX: current may be NULL */
     return;
+/*@=nullstate@*/
 }
 
-int rpmShowRC(FILE *fp)
+int rpmShowRC(FILE * fp)
 {
     struct rpmOption *opt;
     int i;
-    struct machEquivTable * equivTable;
+    machEquivTable equivTable;
 
     /* the caller may set the build arch which should be printed here */
     fprintf(fp, "ARCHITECTURE AND OS:\n");
