@@ -18,15 +18,16 @@
 #endif
 
 #include <rpmlib.h>
-#include <rpmmacro.h>	/* XXX for rpmGetPath */
+#include <rpmmacro.h>	/* XXX for rpmGetPath() */
 
 #include "md5.h"
-#include "misc.h"
+#include "misc.h"	/* XXX for dosetenv() and makeTempFile() */
 #include "rpmlead.h"
 #include "signature.h"
 #include "debug.h"
 
 /*@access Header@*/		/* XXX compared with NULL */
+/*@access FD_t@*/		/* XXX compared with NULL */
 
 typedef	unsigned char byte;
 
@@ -43,7 +44,6 @@ int rpmLookupSignatureType(int action)
 	break;
     case RPMLOOKUPSIG_ENABLE:
 	disabled = 0;
-	/* fall through */
 	/*@fallthrough@*/
     case RPMLOOKUPSIG_QUERY:
 	if (disabled)
@@ -61,7 +61,7 @@ int rpmLookupSignatureType(int action)
 	    rc = RPMSIGTAG_GPG;
 	else
 	    rc = -1;	/* Invalid %_signature spec in macro file */
-	free((void *)name);
+	name = _free(name);
       }	break;
     }
     return rc;
@@ -70,7 +70,7 @@ int rpmLookupSignatureType(int action)
 /* rpmDetectPGPVersion() returns the absolute path to the "pgp"  */
 /* executable of the requested version, or NULL when none found. */
 
-const char * rpmDetectPGPVersion(pgpVersion *pgpVer)
+const char * rpmDetectPGPVersion(pgpVersion * pgpVer)
 {
     /* Actually this should support having more then one pgp version. */ 
     /* At the moment only one version is possible since we only       */
@@ -84,7 +84,7 @@ const char * rpmDetectPGPVersion(pgpVersion *pgpVer)
 	struct stat st;
 	
 	if (!(pgpbin && pgpbin[0] != '%')) {
-	  if (pgpbin) free((void *)pgpbin);
+	  pgpbin = _free(pgpbin);
 	  saved_pgp_version = -1;
 	  return NULL;
 	}
@@ -99,69 +99,88 @@ const char * rpmDetectPGPVersion(pgpVersion *pgpVer)
 	  saved_pgp_version = PGP_NOTDETECTED;
     }
 
-    if (pgpbin && pgpVer)
+    if (pgpVer && pgpbin)
 	*pgpVer = saved_pgp_version;
     return pgpbin;
 }
 
-static inline int checkSize(FD_t fd, int siglen, int pad, int datalen)
+/**
+ * Check package size.
+ * @todo rpmio: use fdSize rather than fstat(2) to get file size.
+ * @param fd			package file handle
+ * @param siglen		signature header size
+ * @param pad			signature padding
+ * @param datalen		length of header+payload
+ * @return 			rpmRC return code
+ */
+static inline rpmRC checkSize(FD_t fd, int siglen, int pad, int datalen)
+	/*@modifies fileSystem @*/
 {
     struct stat st;
+    rpmRC rc;
 
-    fstat(Fileno(fd), &st);
+    if (fstat(Fileno(fd), &st))
+	return RPMRC_FAIL;
 
     if (!S_ISREG(st.st_mode)) {
 	rpmMessage(RPMMESS_DEBUG,
 	    _("file is not regular -- skipping size check\n"));
-	return 0;
+	return RPMRC_OK;
     }
-    rpmMessage(RPMMESS_DEBUG,
-	_("Expected size: %12d = lead(%d)+sigs(%d)+pad(%d)+data(%d)\n"),
-		sizeof(struct rpmlead)+siglen+pad+datalen,
-		sizeof(struct rpmlead), siglen, pad, datalen);
-    rpmMessage(RPMMESS_DEBUG,
-	_("  Actual size: %12d\n"), st.st_size);
 
-    return ((sizeof(struct rpmlead) + siglen + pad + datalen) - st.st_size);
+    rc = (((sizeof(struct rpmlead) + siglen + pad + datalen) - st.st_size)
+	? RPMRC_BADSIZE : RPMRC_OK);
+
+    rpmMessage((rc == RPMRC_OK ? RPMMESS_DEBUG : RPMMESS_WARNING),
+	_("Expected size: %12d = lead(%d)+sigs(%d)+pad(%d)+data(%d)\n"),
+		(int)sizeof(struct rpmlead)+siglen+pad+datalen,
+		(int)sizeof(struct rpmlead), siglen, pad, datalen);
+    rpmMessage((rc == RPMRC_OK ? RPMMESS_DEBUG : RPMMESS_WARNING),
+	_("  Actual size: %12d\n"), (int)st.st_size);
+
+    return rc;
 }
 
-int rpmReadSignature(FD_t fd, Header *headerp, short sigType)
+rpmRC rpmReadSignature(FD_t fd, Header * headerp, sigType sig_type)
 {
     byte buf[2048];
     int sigSize, pad;
     int_32 type, count;
     int_32 *archSize;
     Header h = NULL;
-    int nb;
-    int rc = 1;		/* assume failure */
+    rpmRC rc = RPMRC_FAIL;		/* assume failure */
 
     if (headerp)
 	*headerp = NULL;
 
-    switch (sigType) {
-      case RPMSIG_NONE:
+    buf[0] = 0;
+    switch (sig_type) {
+    case RPMSIGTYPE_NONE:
 	rpmMessage(RPMMESS_DEBUG, _("No signature\n"));
-	rc = 0;
+	rc = RPMRC_OK;
 	break;
-      case RPMSIG_PGP262_1024:
+    case RPMSIGTYPE_PGP262_1024:
 	rpmMessage(RPMMESS_DEBUG, _("Old PGP signature\n"));
 	/* These are always 256 bytes */
 	if (timedRead(fd, buf, 256) != 256)
 	    break;
 	h = headerNew();
-	headerAddEntry(h, RPMSIGTAG_PGP, RPM_BIN_TYPE, buf, 152);
-	rc = 0;
+	(void) headerAddEntry(h, RPMSIGTAG_PGP, RPM_BIN_TYPE, buf, 152);
+	rc = RPMRC_OK;
 	break;
-      case RPMSIG_MD5:
-      case RPMSIG_MD5_PGP:
+    case RPMSIGTYPE_MD5:
+    case RPMSIGTYPE_MD5_PGP:
 	rpmError(RPMERR_BADSIGTYPE,
 	      _("Old (internal-only) signature!  How did you get that!?\n"));
 	break;
-      case RPMSIG_HEADERSIG:
+    case RPMSIGTYPE_HEADERSIG:
+    case RPMSIGTYPE_DISABLE:
 	/* This is a new style signature */
 	h = headerRead(fd, HEADER_MAGIC_YES);
 	if (h == NULL)
 	    break;
+
+	rc = RPMRC_OK;
 	sigSize = headerSizeof(h, HEADER_MAGIC_YES);
 
 	/* XXX Legacy headers have a HEADER_IMAGE tag added. */
@@ -169,47 +188,46 @@ int rpmReadSignature(FD_t fd, Header *headerp, short sigType)
 	    sigSize -= (16 + 16);
 
 	pad = (8 - (sigSize % 8)) % 8; /* 8-byte pad */
-	if (! headerGetEntry(h, RPMSIGTAG_SIZE, &type, (void **)&archSize, &count))
-	    break;
-	nb = checkSize(fd, sigSize, pad, *archSize);
-	if (!(nb == 0 || nb == -(16+16)))
-	    break;
-	if (pad) {
-	    if (timedRead(fd, buf, pad) != pad)
+	if (sig_type == RPMSIGTYPE_HEADERSIG) {
+	    if (! headerGetEntry(h, RPMSIGTAG_SIZE, &type,
+				(void **)&archSize, &count))
 		break;
+	    rc = checkSize(fd, sigSize, pad, *archSize);
 	}
-	rc = 0;
+	if (pad && timedRead(fd, buf, pad) != pad)
+	    rc = RPMRC_SHORTREAD;
 	break;
-      default:
+    default:
 	break;
     }
 
     if (rc == 0 && headerp)
+	/*@-nullderef@*/
 	*headerp = h;
+	/*@=nullderef@*/
     else if (h)
-	headerFree(h);
+	h = headerFree(h);
 
     return rc;
 }
 
-int rpmWriteSignature(FD_t fd, Header header)
+int rpmWriteSignature(FD_t fd, Header h)
 {
-    int sigSize, pad;
     static byte buf[8] = "\000\000\000\000\000\000\000\000";
-    int rc = 0;
+    int sigSize, pad;
+    int rc;
     
-    rc = headerWrite(fd, header, HEADER_MAGIC_YES);
+    rc = headerWrite(fd, h, HEADER_MAGIC_YES);
     if (rc)
 	return rc;
 
-    sigSize = headerSizeof(header, HEADER_MAGIC_YES);
+    sigSize = headerSizeof(h, HEADER_MAGIC_YES);
     pad = (8 - (sigSize % 8)) % 8;
     if (pad) {
-	rpmMessage(RPMMESS_DEBUG, _("Signature size: %d\n"), sigSize);
-	rpmMessage(RPMMESS_DEBUG, _("Signature pad : %d\n"), pad);
 	if (Fwrite(buf, sizeof(buf[0]), pad, fd) != pad)
 	    rc = 1;
     }
+    rpmMessage(RPMMESS_DEBUG, _("Signature: size(%d)+pad(%d)\n"), sigSize, pad);
     return rc;
 }
 
@@ -219,23 +237,24 @@ Header rpmNewSignature(void)
     return h;
 }
 
-void rpmFreeSignature(Header h)
+Header rpmFreeSignature(Header h)
 {
-    headerFree(h);
+    return headerFree(h);
 }
 
-static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_32 *size,
-			    const char *passPhrase)
+static int makePGPSignature(const char * file, /*@out@*/ void ** sig,
+		/*@out@*/ int_32 * size, /*@null@*/ const char * passPhrase)
+	/*@modifies *sig, *size, fileSystem @*/
 {
-    char sigfile[1024];
+    char * sigfile = alloca(1024);
     int pid, status;
     int inpipe[2];
     struct stat st;
 
-    sprintf(sigfile, "%s.sig", file);
+    (void) stpcpy( stpcpy(sigfile, file), ".sig");
 
     inpipe[0] = inpipe[1] = 0;
-    pipe(inpipe);
+    (void) pipe(inpipe);
     
     if (!(pid = fork())) {
 	const char *pgp_path = rpmExpand("%{_pgp_path}", NULL);
@@ -243,24 +262,24 @@ static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 	const char *path;
 	pgpVersion pgpVer;
 
-	close(STDIN_FILENO);
-	dup2(inpipe[0], 3);
-	close(inpipe[1]);
+	(void) close(STDIN_FILENO);
+	(void) dup2(inpipe[0], 3);
+	(void) close(inpipe[1]);
 
-	dosetenv("PGPPASSFD", "3", 1);
+	(void) dosetenv("PGPPASSFD", "3", 1);
 	if (pgp_path && *pgp_path != '%')
-	    dosetenv("PGPPATH", pgp_path, 1);
+	    (void) dosetenv("PGPPATH", pgp_path, 1);
 
 	/* dosetenv("PGPPASS", passPhrase, 1); */
 
 	if ((path = rpmDetectPGPVersion(&pgpVer)) != NULL) {
 	    switch(pgpVer) {
 	    case PGP_2:
-		execlp(path, "pgp", "+batchmode=on", "+verbose=0", "+armor=off",
+		(void) execlp(path, "pgp", "+batchmode=on", "+verbose=0", "+armor=off",
 		    name, "-sb", file, sigfile, NULL);
 		break;
 	    case PGP_5:
-		execlp(path,"pgps", "+batchmode=on", "+verbose=0", "+armor=off",
+		(void) execlp(path,"pgps", "+batchmode=on", "+verbose=0", "+armor=off",
 		    name, "-b", file, "-o", sigfile, NULL);
 		break;
 	    case PGP_UNKNOWN:
@@ -268,14 +287,16 @@ static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 		break;
 	    }
 	}
-	rpmError(RPMERR_EXEC, _("Couldn't exec pgp (%s)\n"), path);
+	rpmError(RPMERR_EXEC, _("Couldn't exec pgp (%s)\n"),
+			(path ? path : NULL));
 	_exit(RPMERR_EXEC);
     }
 
-    close(inpipe[0]);
-    (void)write(inpipe[1], passPhrase, strlen(passPhrase));
-    (void)write(inpipe[1], "\n", 1);
-    close(inpipe[1]);
+    (void) close(inpipe[0]);
+    if (passPhrase)
+	(void) write(inpipe[1], passPhrase, strlen(passPhrase));
+    (void) write(inpipe[1], "\n", 1);
+    (void) close(inpipe[1]);
 
     (void)waitpid(pid, &status, 0);
     if (!WIFEXITED(status) || WEXITSTATUS(status)) {
@@ -285,7 +306,7 @@ static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 
     if (stat(sigfile, &st)) {
 	/* PGP failed to write signature */
-	unlink(sigfile);  /* Just in case */
+	if (sigfile) (void) unlink(sigfile);  /* Just in case */
 	rpmError(RPMERR_SIGGEN, _("pgp failed to write signature\n"));
 	return 1;
     }
@@ -295,13 +316,15 @@ static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
     *sig = xmalloc(*size);
     
     {	FD_t fd;
-	int rc;
+	int rc = 0;
 	fd = Fopen(sigfile, "r.fdio");
-	rc = timedRead(fd, *sig, *size);
-	unlink(sigfile);
-	Fclose(fd);
+	if (fd != NULL && !Ferror(fd)) {
+	    rc = timedRead(fd, *sig, *size);
+	    if (sigfile) (void) unlink(sigfile);
+	    (void) Fclose(fd);
+	}
 	if (rc != *size) {
-	    free(*sig);
+	    *sig = _free(*sig);
 	    rpmError(RPMERR_SIGGEN, _("unable to read the signature\n"));
 	    return 1;
 	}
@@ -317,31 +340,32 @@ static int makePGPSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
  * but this could be a good place to start looking if errors in GPG signature
  * creation crop up.
  */
-static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_32 *size,
-			    const char *passPhrase)
+static int makeGPGSignature(const char * file, /*@out@*/ void ** sig,
+		/*@out@*/ int_32 * size, /*@null@*/ const char * passPhrase)
+	/*@modifies *sig, *size, fileSystem @*/
 {
-    char sigfile[1024];
+    char * sigfile = alloca(1024);
     int pid, status;
     int inpipe[2];
-    FILE *fpipe;
+    FILE * fpipe;
     struct stat st;
 
-    sprintf(sigfile, "%s.sig", file);
+    (void) stpcpy( stpcpy(sigfile, file), ".sig");
 
     inpipe[0] = inpipe[1] = 0;
-    pipe(inpipe);
+    (void) pipe(inpipe);
     
     if (!(pid = fork())) {
 	const char *gpg_path = rpmExpand("%{_gpg_path}", NULL);
 	const char *name = rpmExpand("%{_gpg_name}", NULL);
 
-	close(STDIN_FILENO);
-	dup2(inpipe[0], 3);
-	close(inpipe[1]);
+	(void) close(STDIN_FILENO);
+	(void) dup2(inpipe[0], 3);
+	(void) close(inpipe[1]);
 
 	if (gpg_path && *gpg_path != '%')
-	    dosetenv("GNUPGHOME", gpg_path, 1);
-	execlp("gpg", "gpg",
+	    (void) dosetenv("GNUPGHOME", gpg_path, 1);
+	(void) execlp("gpg", "gpg",
 	       "--batch", "--no-verbose", "--no-armor", "--passphrase-fd", "3",
 	       "-u", name, "-sbo", sigfile, file,
 	       NULL);
@@ -350,9 +374,11 @@ static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
     }
 
     fpipe = fdopen(inpipe[1], "w");
-    close(inpipe[0]);
-    fprintf(fpipe, "%s\n", passPhrase);
-    fclose(fpipe);
+    (void) close(inpipe[0]);
+    if (fpipe) {
+	fprintf(fpipe, "%s\n", (passPhrase ? passPhrase : ""));
+	(void) fclose(fpipe);
+    }
 
     (void)waitpid(pid, &status, 0);
     if (!WIFEXITED(status) || WEXITSTATUS(status)) {
@@ -362,7 +388,7 @@ static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
 
     if (stat(sigfile, &st)) {
 	/* GPG failed to write signature */
-	unlink(sigfile);  /* Just in case */
+	if (sigfile) (void) unlink(sigfile);  /* Just in case */
 	rpmError(RPMERR_SIGGEN, _("gpg failed to write signature\n"));
 	return 1;
     }
@@ -372,13 +398,15 @@ static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
     *sig = xmalloc(*size);
     
     {	FD_t fd;
-	int rc;
+	int rc = 0;
 	fd = Fopen(sigfile, "r.fdio");
-	rc = timedRead(fd, *sig, *size);
-	unlink(sigfile);
-	Fclose(fd);
+	if (fd != NULL && !Ferror(fd)) {
+	    rc = timedRead(fd, *sig, *size);
+	    if (sigfile) (void) unlink(sigfile);
+	    (void) Fclose(fd);
+	}
 	if (rc != *size) {
-	    free(*sig);
+	    *sig = _free(*sig);
 	    rpmError(RPMERR_SIGGEN, _("unable to read the signature\n"));
 	    return 1;
 	}
@@ -389,7 +417,8 @@ static int makeGPGSignature(const char *file, /*@out@*/void **sig, /*@out@*/int_
     return 0;
 }
 
-int rpmAddSignature(Header header, const char *file, int_32 sigTag, const char *passPhrase)
+int rpmAddSignature(Header h, const char * file, int_32 sigTag,
+		const char *passPhrase)
 {
     struct stat st;
     int_32 size;
@@ -399,28 +428,28 @@ int rpmAddSignature(Header header, const char *file, int_32 sigTag, const char *
     
     switch (sigTag) {
     case RPMSIGTAG_SIZE:
-	stat(file, &st);
+	(void) stat(file, &st);
 	size = st.st_size;
 	ret = 0;
-	headerAddEntry(header, RPMSIGTAG_SIZE, RPM_INT32_TYPE, &size, 1);
+	(void) headerAddEntry(h, RPMSIGTAG_SIZE, RPM_INT32_TYPE, &size, 1);
 	break;
     case RPMSIGTAG_MD5:
 	ret = mdbinfile(file, buf);
 	if (ret == 0)
-	    headerAddEntry(header, sigTag, RPM_BIN_TYPE, buf, 16);
+	    (void) headerAddEntry(h, sigTag, RPM_BIN_TYPE, buf, 16);
 	break;
     case RPMSIGTAG_PGP5:	/* XXX legacy */
     case RPMSIGTAG_PGP:
 	rpmMessage(RPMMESS_VERBOSE, _("Generating signature using PGP.\n"));
 	ret = makePGPSignature(file, &sig, &size, passPhrase);
 	if (ret == 0)
-	    headerAddEntry(header, sigTag, RPM_BIN_TYPE, sig, size);
+	    (void) headerAddEntry(h, sigTag, RPM_BIN_TYPE, sig, size);
 	break;
     case RPMSIGTAG_GPG:
 	rpmMessage(RPMMESS_VERBOSE, _("Generating signature using GPG.\n"));
         ret = makeGPGSignature(file, &sig, &size, passPhrase);
 	if (ret == 0)
-	    headerAddEntry(header, sigTag, RPM_BIN_TYPE, sig, size);
+	    (void) headerAddEntry(h, sigTag, RPM_BIN_TYPE, sig, size);
 	break;
     }
 
@@ -428,11 +457,12 @@ int rpmAddSignature(Header header, const char *file, int_32 sigTag, const char *
 }
 
 static rpmVerifySignatureReturn
-verifySizeSignature(const char *datafile, int_32 size, char *result)
+verifySizeSignature(const char * datafile, int_32 size, /*@out@*/ char * result)
+	/*@modifies *result, fileSystem @*/
 {
     struct stat st;
 
-    stat(datafile, &st);
+    (void) stat(datafile, &st);
     if (size != st.st_size) {
 	sprintf(result, "Header+Archive size mismatch.\n"
 		"Expected %d, saw %d.\n",
@@ -447,12 +477,14 @@ verifySizeSignature(const char *datafile, int_32 size, char *result)
 #define	X(_x)	(unsigned)((_x) & 0xff)
 
 static rpmVerifySignatureReturn
-verifyMD5Signature(const char *datafile, const byte *sig, 
-			      char *result, md5func fn)
+verifyMD5Signature(const char * datafile, const byte * sig, 
+			      /*@out@*/ char * result, md5func fn)
+	/*@modifies *result, fileSystem @*/
 {
     byte md5sum[16];
 
-    fn(datafile, md5sum);
+    memset(md5sum, 0, sizeof(md5sum));
+    (void) fn(datafile, md5sum);
     if (memcmp(md5sum, sig, 16)) {
 	sprintf(result, "MD5 sum mismatch\n"
 		"Expected: %02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x"
@@ -481,11 +513,12 @@ verifyMD5Signature(const char *datafile, const byte *sig,
 }
 
 static rpmVerifySignatureReturn
-verifyPGPSignature(const char *datafile, const void * sig, int count, char *result)
+verifyPGPSignature(const char * datafile, const void * sig, int count,
+		/*@out@*/ char * result)
+	/*@modifies *result, fileSystem @*/
 {
     int pid, status, outpipe[2];
-    FD_t sfd;
-    char *sigfile;
+/*@only@*/ /*@null@*/ const char * sigfile = NULL;
     byte buf[BUFSIZ];
     FILE *file;
     int res = RPMSIG_OK;
@@ -508,27 +541,41 @@ verifyPGPSignature(const char *datafile, const void * sig, int count, char *resu
 	res = RPMSIG_BAD;
 
     /* Write out the signature */
+#ifdef	DYING
   { const char *tmppath = rpmGetPath("%{_tmppath}", NULL);
     sigfile = tempnam(tmppath, "rpmsig");
-    free((void *)tmppath);
+    tmppath = _free(tmppath);
   }
     sfd = Fopen(sigfile, "w.fdio");
-    (void)Fwrite(sig, sizeof(char), count, sfd);
-    Fclose(sfd);
+    if (sfd != NULL && !Ferror(sfd)) {
+	(void) Fwrite(sig, sizeof(char), count, sfd);
+	(void) Fclose(sfd);
+    }
+#else
+    {	FD_t sfd;
+	if (!makeTempFile(NULL, &sigfile, &sfd)) {
+	    (void) Fwrite(sig, sizeof(char), count, sfd);
+	    (void) Fclose(sfd);
+	    sfd = NULL;
+	}
+    }
+#endif
+    if (sigfile == NULL)
+	return RPMSIG_BAD;
 
     /* Now run PGP */
     outpipe[0] = outpipe[1] = 0;
-    pipe(outpipe);
+    (void) pipe(outpipe);
 
     if (!(pid = fork())) {
 	const char *pgp_path = rpmExpand("%{_pgp_path}", NULL);
 
-	close(outpipe[0]);
-	close(STDOUT_FILENO);	/* XXX unnecessary */
-	dup2(outpipe[1], STDOUT_FILENO);
+	(void) close(outpipe[0]);
+	(void) close(STDOUT_FILENO);	/* XXX unnecessary */
+	(void) dup2(outpipe[1], STDOUT_FILENO);
 
 	if (pgp_path && *pgp_path != '%')
-	    dosetenv("PGPPATH", pgp_path, 1);
+	    (void) dosetenv("PGPPATH", pgp_path, 1);
 
 	switch (pgpVer) {
 	case PGP_5:
@@ -536,19 +583,19 @@ verifyPGPSignature(const char *datafile, const void * sig, int count, char *resu
 	    /* another message") is _always_ written to stderr; we   */
 	    /* want to catch that output, so dup stdout to stderr:   */
 	{   int save_stderr = dup(2);
-	    dup2(1, 2);
-	    execlp(path, "pgpv", "+batchmode=on", "+verbose=0",
+	    (void) dup2(1, 2);
+	    (void) execlp(path, "pgpv", "+batchmode=on", "+verbose=0",
 		   /* Write "Good signature..." to stdout: */
 		   "+OutputInformationFD=1",
 		   /* Write "WARNING: ... is not trusted to... to stdout: */
 		   "+OutputWarningFD=1",
 		   sigfile, "-o", datafile, NULL);
 	    /* Restore stderr so we can print the error message below. */
-	    dup2(save_stderr, 2);
-	    close(save_stderr);
+	    (void) dup2(save_stderr, 2);
+	    (void) close(save_stderr);
 	}   break;
 	case PGP_2:
-	    execlp(path, "pgp", "+batchmode=on", "+verbose=0",
+	    (void) execlp(path, "pgp", "+batchmode=on", "+verbose=0",
 		   sigfile, datafile, NULL);
 	    break;
 	case PGP_UNKNOWN:
@@ -561,29 +608,32 @@ verifyPGPSignature(const char *datafile, const void * sig, int count, char *resu
 	_exit(RPMERR_EXEC);
     }
 
-    close(outpipe[1]);
+    (void) close(outpipe[1]);
     file = fdopen(outpipe[0], "r");
     result[0] = '\0';
-    while (fgets(buf, 1024, file)) {
-	if (strncmp("File '", buf, 6) &&
-	    strncmp("Text is assu", buf, 12) &&
-	    strncmp("This signature applies to another message", buf, 41) &&
-	    buf[0] != '\n') {
-	    strcat(result, buf);
+    if (file) {
+	while (fgets(buf, 1024, file)) {
+	    if (strncmp("File '", buf, 6) &&
+		strncmp("Text is assu", buf, 12) &&
+		strncmp("This signature applies to another message", buf, 41) &&
+		buf[0] != '\n') {
+		strcat(result, buf);
+	    }
+	    if (!strncmp("WARNING: Can't find the right public key", buf, 40))
+		res = RPMSIG_NOKEY;
+	    else if (!strncmp("Signature by unknown keyid:", buf, 27))
+		res = RPMSIG_NOKEY;
+	    else if (!strncmp("WARNING: The signing key is not trusted", buf, 39))
+		res = RPMSIG_NOTTRUSTED;
+	    else if (!strncmp("Good signature", buf, 14))
+		res = RPMSIG_OK;
 	}
-	if (!strncmp("WARNING: Can't find the right public key", buf, 40))
-	    res = RPMSIG_NOKEY;
-	else if (!strncmp("Signature by unknown keyid:", buf, 27))
-	    res = RPMSIG_NOKEY;
-	else if (!strncmp("WARNING: The signing key is not trusted", buf, 39))
-	    res = RPMSIG_NOTTRUSTED;
-	else if (!strncmp("Good signature", buf, 14))
-	    res = RPMSIG_OK;
+	(void) fclose(file);
     }
-    fclose(file);
 
-    (void)waitpid(pid, &status, 0);
-    unlink(sigfile);
+    (void) waitpid(pid, &status, 0);
+    if (sigfile) (void) unlink(sigfile);
+    sigfile = _free(sigfile);
     if (!res && (!WIFEXITED(status) || WEXITSTATUS(status))) {
 	res = RPMSIG_BAD;
     }
@@ -592,39 +642,54 @@ verifyPGPSignature(const char *datafile, const void * sig, int count, char *resu
 }
 
 static rpmVerifySignatureReturn
-verifyGPGSignature(const char *datafile, const void * sig, int count, char *result)
+verifyGPGSignature(const char * datafile, const void * sig, int count,
+		/*@out@*/ char * result)
+	/*@modifies *result, fileSystem @*/
 {
     int pid, status, outpipe[2];
-    FD_t sfd;
-    char *sigfile;
+/*@only@*/ /*@null@*/ const char * sigfile = NULL;
     byte buf[BUFSIZ];
     FILE *file;
     int res = RPMSIG_OK;
   
     /* Write out the signature */
+#ifdef	DYING
   { const char *tmppath = rpmGetPath("%{_tmppath}", NULL);
     sigfile = tempnam(tmppath, "rpmsig");
-    free((void *)tmppath);
+    tmppath = _free(tmppath);
   }
     sfd = Fopen(sigfile, "w.fdio");
-    (void)Fwrite(sig, sizeof(char), count, sfd);
-    Fclose(sfd);
+    if (sfd != NULL && !Ferror(sfd)) {
+	(void) Fwrite(sig, sizeof(char), count, sfd);
+	(void) Fclose(sfd);
+    }
+#else
+    {	FD_t sfd;
+	if (!makeTempFile(NULL, &sigfile, &sfd)) {
+	    (void) Fwrite(sig, sizeof(char), count, sfd);
+	    (void) Fclose(sfd);
+	    sfd = NULL;
+	}
+    }
+#endif
+    if (sigfile == NULL)
+	return RPMSIG_BAD;
 
     /* Now run GPG */
     outpipe[0] = outpipe[1] = 0;
-    pipe(outpipe);
+    (void) pipe(outpipe);
 
     if (!(pid = fork())) {
 	const char *gpg_path = rpmExpand("%{_gpg_path}", NULL);
 
-	close(outpipe[0]);
+	(void) close(outpipe[0]);
 	/* gpg version 0.9 sends its output to stderr. */
-	dup2(outpipe[1], STDERR_FILENO);
+	(void) dup2(outpipe[1], STDERR_FILENO);
 
 	if (gpg_path && *gpg_path != '%')
-	    dosetenv("GNUPGHOME", gpg_path, 1);
+	    (void) dosetenv("GNUPGHOME", gpg_path, 1);
 
-	execlp("gpg", "gpg",
+	(void) execlp("gpg", "gpg",
 	       "--batch", "--no-verbose", 
 	       "--verify", sigfile, datafile,
 	       NULL);
@@ -633,19 +698,22 @@ verifyGPGSignature(const char *datafile, const void * sig, int count, char *resu
 	_exit(RPMERR_EXEC);
     }
 
-    close(outpipe[1]);
+    (void) close(outpipe[1]);
     file = fdopen(outpipe[0], "r");
     result[0] = '\0';
-    while (fgets(buf, 1024, file)) {
-	strcat(result, buf);
-	if (!xstrncasecmp("gpg: Can't check signature: Public key not found", buf, 48)) {
-	    res = RPMSIG_NOKEY;
+    if (file) {
+	while (fgets(buf, 1024, file)) {
+	    strcat(result, buf);
+	    if (!xstrncasecmp("gpg: Can't check signature: Public key not found", buf, 48)) {
+		res = RPMSIG_NOKEY;
+	    }
 	}
+	(void) fclose(file);
     }
-    fclose(file);
   
-    (void)waitpid(pid, &status, 0);
-    unlink(sigfile);
+    (void) waitpid(pid, &status, 0);
+    if (sigfile) (void) unlink(sigfile);
+    sigfile = _free(sigfile);
     if (!res && (!WIFEXITED(status) || WEXITSTATUS(status))) {
 	res = RPMSIG_BAD;
     }
@@ -653,38 +721,39 @@ verifyGPGSignature(const char *datafile, const void * sig, int count, char *resu
     return res;
 }
 
-static int checkPassPhrase(const char *passPhrase, const int sigTag)
+static int checkPassPhrase(const char * passPhrase, const int sigTag)
+	/*@modifies fileSystem @*/
 {
     int passPhrasePipe[2];
     int pid, status;
     int fd;
 
     passPhrasePipe[0] = passPhrasePipe[1] = 0;
-    pipe(passPhrasePipe);
+    (void) pipe(passPhrasePipe);
     if (!(pid = fork())) {
-	close(STDIN_FILENO);
-	close(STDOUT_FILENO);
-	close(passPhrasePipe[1]);
+	(void) close(STDIN_FILENO);
+	(void) close(STDOUT_FILENO);
+	(void) close(passPhrasePipe[1]);
 	if (! rpmIsVerbose()) {
-	    close(STDERR_FILENO);
+	    (void) close(STDERR_FILENO);
 	}
 	if ((fd = open("/dev/null", O_RDONLY)) != STDIN_FILENO) {
-	    dup2(fd, STDIN_FILENO);
-	    close(fd);
+	    (void) dup2(fd, STDIN_FILENO);
+	    (void) close(fd);
 	}
 	if ((fd = open("/dev/null", O_WRONLY)) != STDOUT_FILENO) {
-	    dup2(fd, STDOUT_FILENO);
-	    close(fd);
+	    (void) dup2(fd, STDOUT_FILENO);
+	    (void) close(fd);
 	}
-	dup2(passPhrasePipe[0], 3);
+	(void) dup2(passPhrasePipe[0], 3);
 
 	switch (sigTag) {
 	case RPMSIGTAG_GPG:
 	{   const char *gpg_path = rpmExpand("%{_gpg_path}", NULL);
 	    const char *name = rpmExpand("%{_gpg_name}", NULL);
 	    if (gpg_path && *gpg_path != '%')
-		dosetenv("GNUPGHOME", gpg_path, 1);
-	    execlp("gpg", "gpg",
+		(void) dosetenv("GNUPGHOME", gpg_path, 1);
+	    (void) execlp("gpg", "gpg",
 	           "--batch", "--no-verbose", "--passphrase-fd", "3",
 	           "-u", name, "-so", "-",
 	           NULL);
@@ -698,18 +767,18 @@ static int checkPassPhrase(const char *passPhrase, const int sigTag)
 	    const char *path;
 	    pgpVersion pgpVer;
 
-	    dosetenv("PGPPASSFD", "3", 1);
+	    (void) dosetenv("PGPPASSFD", "3", 1);
 	    if (pgp_path && *pgp_path != '%')
-		dosetenv("PGPPATH", pgp_path, 1);
+		(void) dosetenv("PGPPATH", pgp_path, 1);
 
 	    if ((path = rpmDetectPGPVersion(&pgpVer)) != NULL) {
 		switch(pgpVer) {
 		case PGP_2:
-		    execlp(path, "pgp", "+batchmode=on", "+verbose=0",
+		    (void) execlp(path, "pgp", "+batchmode=on", "+verbose=0",
 			name, "-sf", NULL);
 		    break;
 		case PGP_5:	/* XXX legacy */
-		    execlp(path,"pgps", "+batchmode=on", "+verbose=0",
+		    (void) execlp(path,"pgps", "+batchmode=on", "+verbose=0",
 			name, "-f", NULL);
 		    break;
 		case PGP_UNKNOWN:
@@ -727,10 +796,10 @@ static int checkPassPhrase(const char *passPhrase, const int sigTag)
 	}
     }
 
-    close(passPhrasePipe[0]);
-    (void)write(passPhrasePipe[1], passPhrase, strlen(passPhrase));
-    (void)write(passPhrasePipe[1], "\n", 1);
-    close(passPhrasePipe[1]);
+    (void) close(passPhrasePipe[0]);
+    (void) write(passPhrasePipe[1], passPhrase, strlen(passPhrase));
+    (void) write(passPhrasePipe[1], "\n", 1);
+    (void) close(passPhrasePipe[1]);
 
     (void)waitpid(pid, &status, 0);
     if (!WIFEXITED(status) || WEXITSTATUS(status)) {
@@ -741,7 +810,7 @@ static int checkPassPhrase(const char *passPhrase, const int sigTag)
     return 0;
 }
 
-char *rpmGetPassPhrase(const char *prompt, const int sigTag)
+char * rpmGetPassPhrase(const char * prompt, const int sigTag)
 {
     char *pass;
     int aok;
@@ -750,7 +819,7 @@ char *rpmGetPassPhrase(const char *prompt, const int sigTag)
     case RPMSIGTAG_GPG:
       { const char *name = rpmExpand("%{_gpg_name}", NULL);
 	aok = (name && *name != '%');
-	free((void *)name);
+	name = _free(name);
       }
 	if (!aok) {
 	    rpmError(RPMERR_SIGGEN,
@@ -762,7 +831,7 @@ char *rpmGetPassPhrase(const char *prompt, const int sigTag)
     case RPMSIGTAG_PGP: 
       { const char *name = rpmExpand("%{_pgp_name}", NULL);
 	aok = (name && *name != '%');
-	free((void *)name);
+	name = _free(name);
       }
 	if (!aok) {
 	    rpmError(RPMERR_SIGGEN,
@@ -788,8 +857,8 @@ char *rpmGetPassPhrase(const char *prompt, const int sigTag)
 }
 
 rpmVerifySignatureReturn
-rpmVerifySignature(const char *file, int_32 sigTag, const void * sig, int count,
-		    char *result)
+rpmVerifySignature(const char * file, int_32 sigTag, const void * sig,
+		int count, char * result)
 {
     switch (sigTag) {
     case RPMSIGTAG_SIZE:
