@@ -42,6 +42,7 @@ char *alloca ();
 #define alloca __builtin_alloca
 #endif
 
+#if !defined(__LCLINT__)
 /*@only@*/ char * xstrdup (const char *str);
 
 #if HAVE_MCHECK_H && defined(__GNUC__)
@@ -50,6 +51,7 @@ char *alloca ();
 #else
 #define	xstrdup(_str)	strdup(_str)
 #endif  /* HAVE_MCHECK_H && defined(__GNUC__) */
+#endif /* !__LCLINT__ */
 
 
 #include "popt.h"
