@@ -5,21 +5,34 @@
 #include <alloca.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <time.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 #include <unistd.h>
 #include <glob.h>	/* XXX rpmio.h */
 #include <dirent.h>	/* XXX rpmio.h */
 #include <locale.h>
+#include <time.h>
 
 #include "Python.h"
-#include "rpmlib.h"
+#include "rpmcli.h"	/* XXX for rpmCheckSig */
 #include "misc.h"
-#include "rpmmacro.h"
+#include "rpmio_internal.h"
+#include "header_internal.h"
 #include "upgrade.h"
 
 extern int _rpmio_debug;
 
+/*@unused@*/ static inline Header headerAllocated(Header h) {
+    h->flags |= HEADERFLAG_ALLOCATED;
+    return 0;
+}
+
+#ifdef __LCLINT__
+#undef	PyObject_HEAD
+#define	PyObject_HEAD	int _PyObjectHead
+#endif
+
 extern int mdfile(const char *fn, unsigned char *digest);
 
 void initrpm(void);
@@ -67,7 +80,7 @@ static PyObject * pyrpmError;
  * 	import os, rpm
  *  
  * 	fd = os.open("/tmp/foo-1.0-1.i386.rpm", os.O_RDONLY)
- * 	(isSource, header) = rpm.headerFromPackage(fd)
+ * 	(header, isSource) = rpm.headerFromPackage(fd)
  * 	fd.close()
  * \endcode
  * The Python interface to the header data is quite elegant.  It
@@ -103,13 +116,13 @@ static PyObject * pyrpmError;
 struct hdrObject_s {
     PyObject_HEAD;
     Header h;
-    Header sigs;
+    Header sigs;		/* XXX signature tags are in header */
     char ** md5list;
     char ** fileList;
     char ** linkList;
     int_32 * fileSizes;
     int_32 * mtimes;
-    int_32 * uids, * gids;
+    int_32 * uids, * gids;	/* XXX these tags are not used anymore */
     unsigned short * rdevs;
     unsigned short * modes;
 } ;
@@ -149,13 +162,30 @@ static PyObject * hdrKeyList(hdrObject * s, PyObject * args) {
 
 /** \ingroup python
  */
-static PyObject * hdrUnload(hdrObject * s, PyObject * args) {
+static PyObject * hdrUnload(hdrObject * s, PyObject * args, PyObject *keywords) {
     char * buf;
-    int len;
     PyObject * rc;
+    int len, legacy = 0;
+    Header h;
+    static char *kwlist[] = { "legacyHeader", NULL};
 
-    len = headerSizeof(s->h, 0);
-    buf = headerUnload(s->h);
+    if (!PyArg_ParseTupleAndKeywords(args, keywords, "|i", kwlist, &legacy))
+	return NULL; 
+
+    h = headerLink(s->h);
+    /* XXX this legacy switch is a hack, needs to be removed. */
+    if (legacy) {
+	h = headerCopy(s->h);	/* XXX strip region tags, etc */
+	headerFree(s->h);
+    }
+    len = headerSizeof(h, 0);
+    buf = headerUnload(h);
+    h = headerFree(h);
+
+    if (buf == NULL || len == 0) {
+	PyErr_SetString(pyrpmError, "can't unload bad header\n");
+	return NULL;
+    }
 
     rc = PyString_FromStringAndSize(buf, len);
     free(buf);
@@ -163,13 +193,10 @@ static PyObject * hdrUnload(hdrObject * s, PyObject * args) {
     return rc;
 }
 
-/* Returns a list of these tuple for each part which failed:
-
-	(attr_name, correctValue, currentValue)
-
-	It should be passwd the file number to verify.
-*/
 /** \ingroup python
+ * Returns a list of these tuples for each item that failed:
+ *	(attr_name, correctValue, currentValue)
+ * It should be passed the file number to verify.
  */
 static PyObject * hdrVerifyFile(hdrObject * s, PyObject * args) {
     int fileNumber;
@@ -189,6 +216,7 @@ static PyObject * hdrVerifyFile(hdrObject * s, PyObject * args) {
 
     fileNumber = (int) PyInt_AsLong(args);
 
+    /* XXX this routine might use callbacks intelligently. */
     if (rpmVerifyFile("", s->h, fileNumber, &verifyResult, RPMVERIFY_NONE)) {
 	Py_INCREF(Py_None);
 	return Py_None;
@@ -198,6 +226,7 @@ static PyObject * hdrVerifyFile(hdrObject * s, PyObject * args) {
 
     if (!verifyResult) return list;
 
+    /* XXX Legacy tag needs to go away. */
     if (!s->fileList) {
 	headerGetEntry(s->h, RPMTAG_OLDFILENAMES, &type, (void **) &s->fileList,
 		 &count);
@@ -307,8 +336,14 @@ static PyObject * hdrVerifyFile(hdrObject * s, PyObject * args) {
 	Py_DECREF(tuple);
     }
 
-    /* RPMVERIFY_USER and RPM_VERIFY_GROUP are handled wrong here, but rpmlib.a
-       doesn't do these correctly either. At least this is consisten */
+    /*
+     * RPMVERIFY_USER and RPM_VERIFY_GROUP are handled wrong here, but rpmlib.a
+     * doesn't do these correctly either. At least this is consistent.
+     *
+     * XXX Consistent? rpmlib.a verifies user/group quite well, thank you.
+     * XXX The code below does nothing useful. FILEUSERNAME needs to be
+     * XXX retrieved and looked up.
+     */
     if (verifyResult & RPMVERIFY_USER) {
 	if (!s->uids) {
 	    headerGetEntry(s->h, RPMTAG_FILEUIDS, &type, (void **) &s->uids,
@@ -326,6 +361,10 @@ static PyObject * hdrVerifyFile(hdrObject * s, PyObject * args) {
 	Py_DECREF(tuple);
     }
 
+    /*
+     * XXX The code below does nothing useful. FILEGROUPNAME needs to be
+     * XXX retrieved and looked up.
+     */
     if (verifyResult & RPMVERIFY_GROUP) {
 	if (!s->gids) {
 	    headerGetEntry(s->h, RPMTAG_FILEGIDS, &type, (void **) &s->gids,
@@ -399,12 +438,73 @@ static void mungeFilelist(Header h)
     if (fileNames == NULL || count <= 0)
 	return;
 
+    /* XXX Legacy tag needs to go away. */
     headerAddEntry(h, RPMTAG_OLDFILENAMES, RPM_STRING_ARRAY_TYPE,
 			fileNames, count);
 
     free((void *)fileNames);
 }
 
+/** 
+ */
+static PyObject * rhnUnload(hdrObject * s, PyObject * args) {
+    int len;
+    char * uh;
+    PyObject * rc;
+    Header h;
+
+    if (!PyArg_ParseTuple(args, ""))
+        return NULL;
+
+    h = headerLink(s->h);
+
+    /* Retrofit a RHNPlatform: tag. */
+    if (!headerIsEntry(h, RPMTAG_RHNPLATFORM)) {
+	const char * arch;
+	int_32 at;
+	if (headerGetEntry(h, RPMTAG_ARCH, &at, (void **)&arch, NULL))
+	    headerAddEntry(h, RPMTAG_RHNPLATFORM, at, arch, 1);
+    }
+
+    /* Legacy headers are forced into immutable region. */
+    if (!headerIsEntry(h, RPMTAG_HEADERIMMUTABLE)) {
+	Header nh = headerReload(h, RPMTAG_HEADERIMMUTABLE);
+	/* XXX Another unload/load cycle to "seal" the immutable region. */
+	uh = headerUnload(nh);
+	headerFree(nh);
+	h = headerLoad(uh);
+	headerAllocated(h);
+    }
+
+    /* All headers have SHA1 digest, compute and add if necessary. */
+    if (!headerIsEntry(h, RPMTAG_SHA1HEADER)) {
+	int_32 uht, uhc;
+	const char * digest;
+        size_t digestlen;
+        DIGEST_CTX ctx;
+
+	headerGetEntry(h, RPMTAG_HEADERIMMUTABLE, &uht, (void **)&uh, &uhc);
+
+	ctx = rpmDigestInit(RPMDIGEST_SHA1);
+        rpmDigestUpdate(ctx, uh, uhc);
+        rpmDigestFinal(ctx, (void **)&digest, &digestlen, 1);
+
+	headerAddEntry(h, RPMTAG_SHA1RHN, RPM_STRING_TYPE, digest, 1);
+
+	uh = headerFreeData(uh, uht);
+	digest = _free(digest);
+    }
+
+    len = headerSizeof(h, 0);
+    uh = headerUnload(h);
+    headerFree(h);
+
+    rc = PyString_FromStringAndSize(uh, len);
+    free(uh);
+
+    return rc;
+}
+
 /** \ingroup python
  */
 static PyObject * hdrFullFilelist(hdrObject * s, PyObject * args) {
@@ -418,11 +518,12 @@ static PyObject * hdrFullFilelist(hdrObject * s, PyObject * args) {
  */
 static struct PyMethodDef hdrMethods[] = {
 	{"keys",	(PyCFunction) hdrKeyList,	1 },
-	{"unload",	(PyCFunction) hdrUnload,	1 },
+	{"unload",	(PyCFunction) hdrUnload,	METH_VARARGS|METH_KEYWORDS },
 	{"verifyFile",	(PyCFunction) hdrVerifyFile,	1 },
 	{"expandFilelist",	(PyCFunction) hdrExpandFilelist,	1 },
 	{"compressFilelist",	(PyCFunction) hdrCompressFilelist,	1 },
 	{"fullFilelist",	(PyCFunction) hdrFullFilelist,	1 },
+	{"rhnUnload",	(PyCFunction) rhnUnload, METH_VARARGS },
 	{NULL,		NULL}		/* sentinel */
 };
 
@@ -471,8 +572,8 @@ static PyObject * hdrSubscript(hdrObject * s, PyObject * item) {
     int forceArray = 0;
     int freeData = 0;
     char * str;
-    struct headerSprintfExtension * ext = NULL;
-    const struct headerSprintfExtension * extensions = rpmHeaderFormats;
+    struct headerSprintfExtension_s * ext = NULL;
+    const struct headerSprintfExtension_s * extensions = rpmHeaderFormats;
 
     if (PyCObject_Check (item))
         ext = PyCObject_AsVoidPtr(item);
@@ -499,6 +600,7 @@ static PyObject * hdrSubscript(hdrObject * s, PyObject * item) {
             return NULL;
         }
         
+	/* XXX signature tags are appended to header, this API is gonna die */
         if (!rpmPackageGetEntry(NULL, s->sigs, s->h, tag, &type, &data, &count))
 	{
             Py_INCREF(Py_None);
@@ -630,7 +732,7 @@ static PyMappingMethods hdrAsMapping = {
 /** \ingroup python
  */
 static PyTypeObject hdrType = {
-	PyObject_HEAD_INIT(&PyType_Type)
+	PyObject_HEAD_INIT(NULL)
 	0,				/* ob_size */
 	"header",			/* tp_name */
 	sizeof(hdrObject),		/* tp_size */
@@ -728,7 +830,7 @@ static void rpmdbMIDealloc(rpmdbMIObject * s) {
 /** \ingroup python
  */
 static PyTypeObject rpmdbMIType = {
-	PyObject_HEAD_INIT(&PyType_Type)
+	PyObject_HEAD_INIT(NULL)
 	0,				/* ob_size */
 	"rpmdbMatchIterator",		/* tp_name */
 	sizeof(rpmdbMIObject),	/* tp_size */
@@ -813,7 +915,7 @@ static PyTypeObject rpmdbMIType = {
  */
 /*@{*/
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmdbFirst(rpmdbObject * s, PyObject * args) {
     int first;
@@ -852,7 +954,7 @@ static PyObject * rpmdbFirst(rpmdbObject * s, PyObject * args) {
     return Py_BuildValue("i", first);
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmdbNext(rpmdbObject * s, PyObject * args) {
     int where;
@@ -874,7 +976,7 @@ static PyObject * rpmdbNext(rpmdbObject * s, PyObject * args) {
     return Py_BuildValue("i", where);
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * handleDbResult(rpmdbMatchIterator mi) {
     PyObject * list, *o;
@@ -893,7 +995,7 @@ static PyObject * handleDbResult(rpmdbMatchIterator mi) {
     return list;
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmdbByFile(rpmdbObject * s, PyObject * args) {
     char * str;
@@ -903,7 +1005,7 @@ static PyObject * rpmdbByFile(rpmdbObject * s, PyObject * args) {
     return handleDbResult(rpmdbInitIterator(s->db, RPMTAG_BASENAMES, str, 0));
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmdbByName(rpmdbObject * s, PyObject * args) {
     char * str;
@@ -913,7 +1015,7 @@ static PyObject * rpmdbByName(rpmdbObject * s, PyObject * args) {
     return handleDbResult(rpmdbInitIterator(s->db, RPMTAG_NAME, str, 0));
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmdbByProvides(rpmdbObject * s, PyObject * args) {
     char * str;
@@ -923,7 +1025,7 @@ static PyObject * rpmdbByProvides(rpmdbObject * s, PyObject * args) {
     return handleDbResult(rpmdbInitIterator(s->db, RPMTAG_PROVIDENAME, str, 0));
 }
 
-/** \ingroup python
+/**
  */
 static rpmdbMIObject *
 py_rpmdbInitIterator (rpmdbObject * s, PyObject * args) {
@@ -955,7 +1057,7 @@ py_rpmdbInitIterator (rpmdbObject * s, PyObject * args) {
     return mio;
 }
 
-/** \ingroup python
+/**
  */
 static struct PyMethodDef rpmdbMethods[] = {
 	{"firstkey",	    (PyCFunction) rpmdbFirst,	1 },
@@ -967,13 +1069,13 @@ static struct PyMethodDef rpmdbMethods[] = {
 	{NULL,		NULL}		/* sentinel */
 };
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmdbGetAttr(rpmdbObject * s, char * name) {
     return Py_FindMethod(rpmdbMethods, (PyObject * ) s, name);
 }
 
-/** \ingroup python
+/**
  */
 static void rpmdbDealloc(rpmdbObject * s) {
     if (s->offsets) {
@@ -985,8 +1087,8 @@ static void rpmdbDealloc(rpmdbObject * s) {
     PyMem_DEL(s);
 }
 
-#ifndef DYINGSOON
-/** \ingroup python
+#ifndef DYINGSOON	/* XXX OK, when? */
+/**
  */
 static int
 rpmdbLength(rpmdbObject * s) {
@@ -996,7 +1098,6 @@ rpmdbLength(rpmdbObject * s) {
 
 	/* RPMDBI_PACKAGES */
 	mi = rpmdbInitIterator(s->db, RPMDBI_PACKAGES, NULL, 0);
-	/* XXX FIXME: unnecessary header mallocs are side effect here */
 	while (rpmdbNextIterator(mi) != NULL)
 	    count++;
 	rpmdbFreeIterator(mi);
@@ -1005,7 +1106,7 @@ rpmdbLength(rpmdbObject * s) {
     return count;
 }
 
-/** \ingroup python
+/**
  */
 static hdrObject *
 rpmdbSubscript(rpmdbObject * s, PyObject * key) {
@@ -1040,7 +1141,7 @@ rpmdbSubscript(rpmdbObject * s, PyObject * key) {
     return h;
 }
 
-/** \ingroup python
+/**
  */
 static PyMappingMethods rpmdbAsMapping = {
 	(inquiry) rpmdbLength,		/* mp_length */
@@ -1049,10 +1150,10 @@ static PyMappingMethods rpmdbAsMapping = {
 };
 #endif
 
-/** \ingroup python
+/**
  */
 static PyTypeObject rpmdbType = {
-	PyObject_HEAD_INIT(&PyType_Type)
+	PyObject_HEAD_INIT(NULL)
 	0,				/* ob_size */
 	"rpmdb",			/* tp_name */
 	sizeof(rpmdbObject),		/* tp_size */
@@ -1255,7 +1356,7 @@ static PyObject * rpmtransRemove(rpmtransObject * s, PyObject * args) {
 /** \ingroup python
  */
 static PyObject * rpmtransDepCheck(rpmtransObject * s, PyObject * args) {
-    struct rpmDependencyConflict * conflicts;
+    rpmDependencyConflict conflicts;
     int numConflicts;
     PyObject * list, * cf;
     int i;
@@ -1266,6 +1367,7 @@ static PyObject * rpmtransDepCheck(rpmtransObject * s, PyObject * args) {
     if (numConflicts) {
 	list = PyList_New(0);
 
+	/* XXX TODO: rpmlib-4.0.3 can return multiple suggested packages. */
 	for (i = 0; i < numConflicts; i++) {
 	    cf = Py_BuildValue("((sss)(ss)iOi)", conflicts[i].byName,
 			       conflicts[i].byVersion, conflicts[i].byRelease,
@@ -1274,14 +1376,14 @@ static PyObject * rpmtransDepCheck(rpmtransObject * s, PyObject * args) {
 			       conflicts[i].needsVersion,
 
 			       conflicts[i].needsFlags,
-			       conflicts[i].suggestedPackage ?
-				   conflicts[i].suggestedPackage : Py_None,
+			       conflicts[i].suggestedPackages ?
+				   conflicts[i].suggestedPackages[0] : Py_None,
 			       conflicts[i].sense);
 	    PyList_Append(list, (PyObject *) cf);
 	    Py_DECREF(cf);
 	}
 
-	rpmdepFreeConflicts(conflicts, numConflicts);
+	conflicts = rpmdepFreeConflicts(conflicts, numConflicts);
 
 	return list;
     }
@@ -1336,6 +1438,7 @@ struct tsCallbackType {
 };
 
 /** \ingroup python
+ * @todo Remove, there's no headerLink refcount on the pointer.
  */
 static Header transactionSetHeader = NULL;
 
@@ -1492,7 +1595,7 @@ static int rpmtransSetAttr(rpmtransObject * o, char * name,
 /** \ingroup python
  */
 static PyTypeObject rpmtransType = {
-	PyObject_HEAD_INIT(&PyType_Type)
+	PyObject_HEAD_INIT(NULL)
 	0,				/* ob_size */
 	"rpmtrans",			/* tp_name */
 	sizeof(rpmtransObject),		/* tp_size */
@@ -1515,7 +1618,7 @@ static PyTypeObject rpmtransType = {
  */
 /*@{*/
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmtransCreate(PyObject * self, PyObject * args) {
     rpmtransObject * o;
@@ -1539,7 +1642,7 @@ static PyObject * rpmtransCreate(PyObject * self, PyObject * args) {
     return (void *) o;
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * doAddMacro(PyObject * self, PyObject * args) {
     char * name, * val;
@@ -1553,7 +1656,7 @@ static PyObject * doAddMacro(PyObject * self, PyObject * args) {
     return Py_None;
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * doDelMacro(PyObject * self, PyObject * args) {
     char * name;
@@ -1567,7 +1670,7 @@ static PyObject * doDelMacro(PyObject * self, PyObject * args) {
     return Py_None;
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * archScore(PyObject * self, PyObject * args) {
     char * arch;
@@ -1581,7 +1684,7 @@ static PyObject * archScore(PyObject * self, PyObject * args) {
     return Py_BuildValue("i", score);
 }
 
-/** \ingroup python
+/**
  */
 static int psGetArchScore(Header h) {
     void * pkgArch;
@@ -1594,7 +1697,7 @@ static int psGetArchScore(Header h) {
         return rpmMachineScore(RPM_MACHTABLE_INSTARCH, pkgArch);
 }
 
-/** \ingroup python
+/**
  */
 static int pkgCompareVer(void * first, void * second) {
     struct packageInfo ** a = first;
@@ -1616,12 +1719,15 @@ static int pkgCompareVer(void * first, void * second) {
     return rpmVersionCompare((*b)->h, (*a)->h);
 }
 
-/** \ingroup python
+/**
  */
 static void pkgSort(struct pkgSet * psp) {
     int i;
     char *name;
 
+    if (psp->numPackages <= 0)
+	return;
+
     qsort(psp->packages, psp->numPackages, sizeof(*psp->packages),
 	 (void *) pkgCompareVer);
 
@@ -1646,7 +1752,7 @@ static void pkgSort(struct pkgSet * psp) {
     psp->numPackages = i;
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * findUpgradeSet(PyObject * self, PyObject * args) {
     PyObject * hdrList, * result;
@@ -1697,16 +1803,16 @@ static PyObject * findUpgradeSet(PyObject * self, PyObject * args) {
     return result;
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmHeaderFromPackage(PyObject * self, PyObject * args) {
     hdrObject * h;
     Header header;
     Header sigs;
-    int rc;
     FD_t fd;
     int rawFd;
     int isSource = 0;
+    rpmRC rc;
 
     if (!PyArg_ParseTuple(args, "i", &rawFd)) return NULL;
     fd = fdDup(rawFd);
@@ -1715,7 +1821,8 @@ static PyObject * rpmHeaderFromPackage(PyObject * self, PyObject * args) {
     Fclose(fd);
 
     switch (rc) {
-      case 0:
+    case RPMRC_BADSIZE:
+    case RPMRC_OK:
 	h = (hdrObject *) PyObject_NEW(PyObject, &hdrType);
 	h->h = header;
 	h->sigs = sigs;
@@ -1726,12 +1833,14 @@ static PyObject * rpmHeaderFromPackage(PyObject * self, PyObject * args) {
 	    isSource = 1;
 	break;
 
-      case 1:
+    case RPMRC_BADMAGIC:
 	Py_INCREF(Py_None);
 	h = (hdrObject *) Py_None;
 	break;
 
-      default:
+    case RPMRC_FAIL:
+    case RPMRC_SHORTREAD:
+    default:
 	PyErr_SetString(pyrpmError, "error reading package");
 	return NULL;
     }
@@ -1739,7 +1848,7 @@ static PyObject * rpmHeaderFromPackage(PyObject * self, PyObject * args) {
     return Py_BuildValue("(Ni)", h, isSource);
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * hdrLoad(PyObject * self, PyObject * args) {
     char * obj, * copy=NULL;
@@ -1749,12 +1858,12 @@ static PyObject * hdrLoad(PyObject * self, PyObject * args) {
 
     if (!PyArg_ParseTuple(args, "s#", &obj, &len)) return NULL;
     
+    /* malloc is needed to avoid surprises from data swab in headerLoad(). */
     copy = malloc(len);
     if (copy == NULL) {
 	PyErr_SetString(pyrpmError, "out of memory");
 	return NULL;
     }
-
     memcpy (copy, obj, len);
 
     hdr = headerLoad(copy);
@@ -1762,6 +1871,7 @@ static PyObject * hdrLoad(PyObject * self, PyObject * args) {
 	PyErr_SetString(pyrpmError, "bad header");
 	return NULL;
     }
+    headerAllocated(hdr);
     compressFilelist (hdr);
     providePackageNVR (hdr);
 
@@ -1775,7 +1885,71 @@ static PyObject * hdrLoad(PyObject * self, PyObject * args) {
     return (PyObject *) h;
 }
 
-/** \ingroup python
+/**
+ */
+static PyObject * rhnLoad(PyObject * self, PyObject * args) {
+    char * obj, * copy=NULL;
+    Header hdr;
+    hdrObject * h;
+    int len;
+
+    if (!PyArg_ParseTuple(args, "s#", &obj, &len)) return NULL;
+    
+    /* malloc is needed to avoid surprises from data swab in headerLoad(). */
+    copy = malloc(len);
+    if (copy == NULL) {
+	PyErr_SetString(pyrpmError, "out of memory");
+	return NULL;
+    }
+    memcpy (copy, obj, len);
+
+    hdr = headerLoad(copy);
+    if (!hdr) {
+	PyErr_SetString(pyrpmError, "bad header");
+	return NULL;
+    }
+    headerAllocated(hdr);
+
+    /* XXX avoid the false OK's from rpmverifyDigest() with missing tags. */
+    if (!headerIsEntry(hdr, RPMTAG_HEADERIMMUTABLE)) {
+	PyErr_SetString(pyrpmError, "bad header, not immutable");
+	headerFree(hdr);
+	return NULL;
+    }
+
+    /* XXX avoid the false OK's from rpmverifyDigest() with missing tags. */
+    if (!headerIsEntry(hdr, RPMTAG_SHA1HEADER)
+    &&  !headerIsEntry(hdr, RPMTAG_SHA1RHN)) {
+	PyErr_SetString(pyrpmError, "bad header, no digest");
+	headerFree(hdr);
+	return NULL;
+    }
+
+    if (rpmVerifyDigest(hdr)) {
+	PyErr_SetString(pyrpmError, "bad header, digest check failed");
+	headerFree(hdr);
+	return NULL;
+    }
+
+    /* Retrofit a RHNPlatform: tag. */
+    if (!headerIsEntry(hdr, RPMTAG_RHNPLATFORM)) {
+	const char * arch;
+	int_32 at;
+	if (headerGetEntry(hdr, RPMTAG_ARCH, &at, (void **)&arch, NULL))
+	    headerAddEntry(hdr, RPMTAG_RHNPLATFORM, at, arch, 1);
+    }
+
+    h = (hdrObject *) PyObject_NEW(PyObject, &hdrType);
+    h->h = hdr;
+    h->sigs = NULL;
+    h->fileList = h->linkList = h->md5list = NULL;
+    h->uids = h->gids = h->mtimes = h->fileSizes = NULL;
+    h->modes = h->rdevs = NULL;
+
+    return (PyObject *) h;
+}
+
+/**
  */
 static PyObject * rpmInitDB(PyObject * self, PyObject * args) {
     char *root;
@@ -1799,7 +1973,7 @@ static PyObject * rpmInitDB(PyObject * self, PyObject * args) {
     return(Py_None);
 }
 
-/** \ingroup python
+/**
  */
 static rpmdbObject * rpmOpenDB(PyObject * self, PyObject * args) {
     rpmdbObject * o;
@@ -1831,7 +2005,7 @@ static rpmdbObject * rpmOpenDB(PyObject * self, PyObject * args) {
     return o;
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * rebuildDB (PyObject * self, PyObject * args) {
     char * root = "";
@@ -1841,7 +2015,7 @@ static PyObject * rebuildDB (PyObject * self, PyObject * args) {
     return Py_BuildValue("i", rpmdbRebuild(root));
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmReadHeaders (FD_t fd) {
     PyObject * list;
@@ -1883,7 +2057,7 @@ static PyObject * rpmReadHeaders (FD_t fd) {
     return list;
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmHeaderFromFD(PyObject * self, PyObject * args) {
     FD_t fd;
@@ -1899,7 +2073,7 @@ static PyObject * rpmHeaderFromFD(PyObject * self, PyObject * args) {
     return list;
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * rpmHeaderFromFile(PyObject * self, PyObject * args) {
     char * filespec;
@@ -1920,11 +2094,10 @@ static PyObject * rpmHeaderFromFile(PyObject * self, PyObject * args) {
     return list;
 }
 
-/** \ingroup python
+/**
+ * This assumes the order of list matches the order of the new headers, and
+ * throws an exception if that isn't true.
  */
-
-/* this assumes the order of list matches the order of the new headers, and
-   throws an exception if that isn't true */
 static int rpmMergeHeaders(PyObject * list, FD_t fd, int matchTag) {
     Header newH;
     HeaderIterator iter;
@@ -2013,11 +2186,12 @@ static PyObject * rpmMergeHeadersFromFD(PyObject * self, PyObject * args) {
     return Py_None;
 }
 
-/** \ingroup python
+
+/**
  */
 static PyObject * errorCB = NULL, * errorData = NULL;
 
-/** \ingroup python
+/**
  */
 static void errorcb (void)
 {
@@ -2036,26 +2210,18 @@ static void errorcb (void)
     Py_DECREF (result);
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * errorSetCallback (PyObject * self, PyObject * args) {
-    if (errorCB != NULL) {
-	Py_DECREF (errorCB);
-	errorCB = NULL;
-    }
+    PyObject *newCB = NULL, *newData = NULL;
 
-    if (errorData != NULL) {
-	Py_DECREF (errorData);
-	errorData = NULL;
-    }
-
-    if (!PyArg_ParseTuple(args, "O|O", &errorCB, &errorData)) return NULL;
+    if (!PyArg_ParseTuple(args, "O|O", &newCB, &newData)) return NULL;
 
     /* if we're getting a void*, set the error callback to this. */
     /* also, we can possibly decref any python callbacks we had  */
     /* and set them to NULL.                                     */
-    if (PyCObject_Check (errorCB)) {
-	rpmErrorSetCallback (PyCObject_AsVoidPtr(errorCB));
+    if (PyCObject_Check (newCB)) {
+	rpmErrorSetCallback (PyCObject_AsVoidPtr(newCB));
 
 	Py_XDECREF (errorCB);
 	Py_XDECREF (errorData);
@@ -2067,24 +2233,30 @@ static PyObject * errorSetCallback (PyObject * self, PyObject * args) {
 	return Py_None;
     }
     
-    if (!PyCallable_Check (errorCB)) {
+    if (!PyCallable_Check (newCB)) {
 	PyErr_SetString(PyExc_TypeError, "parameter must be callable");
 	return NULL;
     }
 
+    Py_XDECREF(errorCB);
+    Py_XDECREF(errorData);
+
+    errorCB = newCB;
+    errorData = newData;
+    
     Py_INCREF (errorCB);
     Py_XINCREF (errorData);
 
     return PyCObject_FromVoidPtr(rpmErrorSetCallback (errorcb), NULL);
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * errorString (PyObject * self, PyObject * args) {
     return PyString_FromString(rpmErrorString ());
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * versionCompare (PyObject * self, PyObject * args) {
     hdrObject * h1, * h2;
@@ -2125,7 +2297,7 @@ static PyObject * labelCompare (PyObject * self, PyObject * args) {
     return Py_BuildValue("i", rpmvercmp(r1, r2));
 }
 
-/** \ingroup python
+/**
  */
 static PyObject * checkSig (PyObject * self, PyObject * args) {
     char * filename;
@@ -2142,7 +2314,7 @@ static PyObject * checkSig (PyObject * self, PyObject * args) {
 }
 
 /* hack to get the current header that's in the transaction set */
-/** \ingroup python
+/**
  */
 static PyObject * getTsHeader (PyObject * self, PyObject * args) {
     hdrObject * h;
@@ -2160,11 +2332,11 @@ static PyObject * getTsHeader (PyObject * self, PyObject * args) {
     return (PyObject *) Py_None;
 }
 
-/** \ingroup python
+/**
  */
 typedef struct FDlist_t FDlist;
 
-/** \ingroup python
+/**
  */
 struct FDlist_t {
     FILE *f;
@@ -2173,15 +2345,15 @@ struct FDlist_t {
     FDlist *next;
 } ;
 
-/** \ingroup python
+/**
  */
 static FDlist *fdhead = NULL;
 
-/** \ingroup python
+/**
  */
 static FDlist *fdtail = NULL;
 
-/** \ingroup python
+/**
  */
 static int closeCallback(FILE * f) {
     FDlist *node, *last;
@@ -2212,8 +2384,7 @@ static int closeCallback(FILE * f) {
     return 0; 
 }
 
-#if 0
-/** \ingroup python
+/**
  */
 static PyObject * doFopen(PyObject * self, PyObject * args) {
     char * path, * mode;
@@ -2262,9 +2433,8 @@ static PyObject * doFopen(PyObject * self, PyObject * args) {
     
     return PyFile_FromFile (node->f, path, mode, closeCallback);
 }
-#endif
 
-/** \ingroup python
+/**
  */
 static PyMethodDef rpmModuleMethods[] = {
     { "TransactionSet", (PyCFunction) rpmtransCreate, METH_VARARGS, NULL },
@@ -2274,11 +2444,12 @@ static PyMethodDef rpmModuleMethods[] = {
     { "findUpgradeSet", (PyCFunction) findUpgradeSet, METH_VARARGS, NULL },
     { "headerFromPackage", (PyCFunction) rpmHeaderFromPackage, METH_VARARGS, NULL },
     { "headerLoad", (PyCFunction) hdrLoad, METH_VARARGS, NULL },
+    { "rhnLoad", (PyCFunction) rhnLoad, METH_VARARGS, NULL },
     { "initdb", (PyCFunction) rpmInitDB, METH_VARARGS, NULL },
     { "opendb", (PyCFunction) rpmOpenDB, METH_VARARGS, NULL },
     { "rebuilddb", (PyCFunction) rebuildDB, METH_VARARGS, NULL },
-    { "readHeaderListFromFD", (PyCFunction) rpmHeaderFromFD, METH_VARARGS, NULL },
     { "mergeHeaderListFromFD", (PyCFunction) rpmMergeHeadersFromFD, METH_VARARGS, NULL },
+    { "readHeaderListFromFD", (PyCFunction) rpmHeaderFromFD, METH_VARARGS, NULL },
     { "readHeaderListFromFile", (PyCFunction) rpmHeaderFromFile, METH_VARARGS, NULL },
     { "errorSetCallback", (PyCFunction) errorSetCallback, METH_VARARGS, NULL },
     { "errorString", (PyCFunction) errorString, METH_VARARGS, NULL },
@@ -2290,18 +2461,27 @@ static PyMethodDef rpmModuleMethods[] = {
     { NULL }
 } ;
 
-/** \ingroup python
+/**
  */
 void initrpm(void) {
     PyObject * m, * d, *o, * tag = NULL, * dict;
     int i;
-    const struct headerSprintfExtension * extensions = rpmHeaderFormats;
-    struct headerSprintfExtension * ext;
+    const struct headerSprintfExtension_s * extensions = rpmHeaderFormats;
+    struct headerSprintfExtension_s * ext;
+
+    m = Py_InitModule("rpm", rpmModuleMethods);
+
+    hdrType.ob_type = &PyType_Type;
+    rpmdbMIType.ob_type = &PyType_Type;
+    rpmdbType.ob_type = &PyType_Type;
+    rpmtransType.ob_type = &PyType_Type;
+
+    if(!m)
+	return;
 
 /*      _rpmio_debug = -1; */
     rpmReadConfigFiles(NULL, NULL);
 
-    m = Py_InitModule("rpm", rpmModuleMethods);
     d = PyModule_GetDict(m);
 
     pyrpmError = PyString_FromString("rpm.error");
@@ -2334,7 +2514,7 @@ void initrpm(void) {
 
 
 #define REGISTER_ENUM(val) \
-    PyDict_SetItemString(d, #val, o=PyInt_FromLong(## val)); \
+    PyDict_SetItemString(d, #val, o=PyInt_FromLong( val )); \
     Py_DECREF(o);
     
     REGISTER_ENUM(RPMFILE_STATE_NORMAL);
