@@ -1,17 +1,16 @@
 #include "system.h"
 
+#include <rpmcli.h>
 #include <rpmbuild.h>
-#include <rpmurl.h>
 
 #include "build.h"
-#include "install.h"
 #include "signature.h"
 #include "debug.h"
 
 #define GETOPT_ADDSIGN		1005
 #define GETOPT_RESIGN		1006
 #define GETOPT_DBPATH		1010
-#define GETOPT_REBUILDDB        1013
+#define GETOPT_REBUILDDB	1013
 #define GETOPT_INSTALL		1014
 #define GETOPT_RELOCATE		1016
 #define GETOPT_SHOWRC		1018
@@ -19,12 +18,13 @@
 #define	GETOPT_DEFINEMACRO	1020
 #define	GETOPT_EVALMACRO	1021
 #define	GETOPT_RCFILE		1022
+#define GETOPT_VERIFYDB		1023
 
 enum modes {
     MODE_UNKNOWN	= 0,
     MODE_QUERY		= (1 <<  0),
     MODE_INSTALL	= (1 <<  1),
-    MODE_UNINSTALL	= (1 <<  2),
+    MODE_ERASE		= (1 <<  2),
     MODE_VERIFY		= (1 <<  3),
     MODE_BUILD		= (1 <<  4),
     MODE_REBUILD	= (1 <<  5),
@@ -34,13 +34,14 @@ enum modes {
     MODE_QUERYTAGS	= (1 <<  9),
     MODE_INITDB		= (1 << 10),
     MODE_TARBUILD	= (1 << 11),
-    MODE_REBUILDDB	= (1 << 12)
+    MODE_REBUILDDB	= (1 << 12),
+    MODE_VERIFYDB	= (1 << 13)
 };
 
 #define	MODES_QV (MODE_QUERY | MODE_VERIFY)
 #define	MODES_BT (MODE_BUILD | MODE_TARBUILD | MODE_REBUILD | MODE_RECOMPILE)
-#define	MODES_IE (MODE_INSTALL | MODE_UNINSTALL)
-#define	MODES_DB (MODE_INITDB | MODE_REBUILDDB)
+#define	MODES_IE (MODE_INSTALL | MODE_ERASE)
+#define	MODES_DB (MODE_INITDB | MODE_REBUILDDB | MODE_VERIFYDB)
 #define	MODES_K	 (MODE_CHECKSIG | MODES_RESIGN)
 
 #define	MODES_FOR_DBPATH	(MODES_BT | MODES_IE | MODES_QV | MODES_DB)
@@ -51,9 +52,12 @@ enum modes {
 /* the flags for the various options */
 static int allFiles;
 static int allMatches;
+static int applyOnly;
 static int badReloc;
+static int dirStash;
 static int excldocs;
 static int force;
+extern int _fsm_debug;
 extern int _ftp_debug;
 static int showHash;
 static int help;
@@ -69,14 +73,32 @@ extern int noLibio;
 static int noMd5;
 static int noOrder;
 static int noPgp;
+
 static int noScripts;
+static int noPre;
+static int noPost;
+static int noPreun;
+static int noPostun;
+
 static int noTriggers;
+static int noTPrein;
+static int noTIn;
+static int noTUn;
+static int noTPostun;
+
 static int noUsageMsg;
 static int oldPackage;
 static char * pipeOutput;
 static char * prefix;
 static int quiet;
 static char * rcfile;
+
+static int rePackage;
+static int pkgCommit;
+static int pkgUndo;
+static int tsCommit;
+static int tsUndo;
+
 static int replaceFiles;
 static int replacePackages;
 static char * rootdir;
@@ -94,18 +116,20 @@ extern const char * rpmNAME;
 extern const char * rpmEVR;
 extern int rpmFLAGS;
 
-extern MacroContext rpmCLIMacroContext;
+extern struct MacroContext_s rpmCLIMacroContext;
 
-extern struct rpmBuildArguments		rpmBTArgs;
+extern struct rpmBuildArguments_s	rpmBTArgs;
 
 /* the structure describing the options we take and the defaults */
 static struct poptOption optionsTable[] = {
  { "addsign", '\0', 0, 0, GETOPT_ADDSIGN,	NULL, NULL},
  { "allfiles", '\0', 0, &allFiles, 0,		NULL, NULL},
  { "allmatches", '\0', 0, &allMatches, 0,	NULL, NULL},
+ { "apply", '\0', 0, &applyOnly, 0,		NULL, NULL},
  { "badreloc", '\0', 0, &badReloc, 0,		NULL, NULL},
  { "checksig", 'K', 0, 0, 'K',			NULL, NULL},
  { "define", '\0', POPT_ARG_STRING, 0, GETOPT_DEFINEMACRO,NULL, NULL},
+ { "dirstash", '\0', POPT_ARG_VAL, &dirStash, 1,	NULL, NULL},
  { "dirtokens", '\0', POPT_ARG_VAL, &_noDirTokens, 0,	NULL, NULL},
  { "erase", 'e', 0, 0, 'e',			NULL, NULL},
  { "eval", '\0', POPT_ARG_STRING, 0, GETOPT_EVALMACRO, NULL, NULL},
@@ -113,6 +137,7 @@ static struct poptOption optionsTable[] = {
  { "excludepath", '\0', POPT_ARG_STRING, 0, GETOPT_EXCLUDEPATH,	NULL, NULL},
  { "force", '\0', 0, &force, 0,			NULL, NULL},
  { "freshen", 'F', 0, 0, 'F',			NULL, NULL},
+ { "fsmdebug", '\0', POPT_ARG_VAL, &_fsm_debug, -1,		NULL, NULL},
  { "ftpdebug", '\0', POPT_ARG_VAL, &_ftp_debug, -1,		NULL, NULL},
  { "hash", 'h', 0, &showHash, 0,		NULL, NULL},
  { "help", '\0', 0, &help, 0,			NULL, NULL},
@@ -135,11 +160,24 @@ static struct poptOption optionsTable[] = {
  { "nomd5", '\0', 0, &noMd5, 0,			NULL, NULL},
  { "noorder", '\0', 0, &noOrder, 0,		NULL, NULL},
  { "nopgp", '\0', 0, &noPgp, 0,			NULL, NULL},
+
  { "noscripts", '\0', 0, &noScripts, 0,		NULL, NULL},
+ { "nopre", '\0', 0, &noPre, 0,			NULL, NULL},
+ { "nopost", '\0', 0, &noPost, 0,		NULL, NULL},
+ { "nopreun", '\0', 0, &noPreun, 0,		NULL, NULL},
+ { "nopostun", '\0', 0, &noPostun, 0,		NULL, NULL},
+
  { "notriggers", '\0', 0, &noTriggers, 0,	NULL, NULL},
+ { "notriggerprein", '\0', 0, &noTPrein, 0,	NULL, NULL},
+ { "notriggerin", '\0', 0, &noTIn, 0,		NULL, NULL},
+ { "notriggerun", '\0', 0, &noTUn, 0,		NULL, NULL},
+ { "notriggerpostun", '\0', 0, &noTPostun, 0,	NULL, NULL},
+
  { "oldpackage", '\0', 0, &oldPackage, 0,	NULL, NULL},
  { "percent", '\0', 0, &showPercents, 0,	NULL, NULL},
  { "pipe", '\0', POPT_ARG_STRING, &pipeOutput, 0,	NULL, NULL},
+ { "pkgcommit", '\0', POPT_ARG_VAL, &pkgCommit, 1,	NULL, NULL},
+ { "pkgundo", '\0', POPT_ARG_VAL, &pkgUndo, 1,	NULL, NULL},
  { "prefix", '\0', POPT_ARG_STRING, &prefix, 0,	NULL, NULL},
  { "quiet", '\0', 0, &quiet, 0,			NULL, NULL},
 #ifndef	DYING
@@ -148,7 +186,9 @@ static struct poptOption optionsTable[] = {
  { "rcfile", '\0', 0, 0, GETOPT_RCFILE,		NULL, NULL},
 #endif
  { "rebuilddb", '\0', 0, 0, GETOPT_REBUILDDB,	NULL, NULL},
+ { "verifydb", '\0', 0, 0, GETOPT_VERIFYDB,	NULL, NULL},
  { "relocate", '\0', POPT_ARG_STRING, 0, GETOPT_RELOCATE,	NULL, NULL},
+ { "repackage", '\0', POPT_ARG_VAL, &rePackage, 1,	NULL, NULL},
  { "replacefiles", '\0', 0, &replaceFiles, 0,	NULL, NULL},
  { "replacepkgs", '\0', 0, &replacePackages, 0,	NULL, NULL},
  { "resign", '\0', 0, 0, GETOPT_RESIGN,		NULL, NULL},
@@ -157,6 +197,8 @@ static struct poptOption optionsTable[] = {
  { "showrc", '\0', 0, &showrc, GETOPT_SHOWRC,	NULL, NULL},
  { "sign", '\0', 0, &signIt, 0,			NULL, NULL},
  { "test", '\0', 0, &test, 0,			NULL, NULL},
+ { "commit", '\0', POPT_ARG_VAL, &tsCommit, 1,	NULL, NULL},
+ { "undo", '\0', POPT_ARG_VAL, &tsUndo, 1,	NULL, NULL},
  { "upgrade", 'U', 0, 0, 'U',			NULL, NULL},
  { "urldebug", '\0', POPT_ARG_VAL, &_url_debug, -1,		NULL, NULL},
  { "uninstall", 'u', 0, 0, 'u',			NULL, NULL},
@@ -251,6 +293,7 @@ static void printUsage(void) {
     puts(_("       rpm {--checksig -K} [--nopgp] [--nogpg] [--nomd5] [--rcfile <file>]"));
     puts(_("                           package1 ... packageN"));
     puts(_("       rpm {--rebuilddb} [--rcfile <file>] [--dbpath <dir>]"));
+    puts(_("       rpm {--verifydb} [--rcfile <file>] [--dbpath <dir>]"));
     puts(_("       rpm {--querytags}"));
 }
 
@@ -442,7 +485,7 @@ static void printHelp(void) {
 		  _("upgrade to an old version of the package (--force on upgrades does this automatically)"));
     puts("");
     puts(         _("    --erase <package>"));
-    printHelpLine(  "    -e <package>          ",
+    printHelpLine(_("    -e <package>          "),
 		  _("erase (uninstall) package"));
     printHelpLine(  "      --allmatches        ",
 		  _("remove all packages which match <package> (normally an error is generated if <package> specified multiple packages)"));
@@ -492,8 +535,6 @@ static void printHelp(void) {
 		  _("build the packages for the build targets platform1...platformN."));
     printHelpLine(  "      --nobuild           ",
 		  _("do not execute any stages"));
-    printHelpLine(_("      --timecheck <secs>  "),
-		  _("set the time check to <secs> seconds (0 disables)"));
     puts("");
     printHelpLine(_("    --rebuild <src_pkg>   "),
 		  _("install source package, build binary package and remove spec file, sources, patches, and icons."));
@@ -520,6 +561,8 @@ static void printHelp(void) {
 		  _("make sure a valid database exists"));
     printHelpLine(  "    --rebuilddb           ",
 		  _("rebuild database from existing database"));
+    printHelpLine(  "    --verifydb            ",
+		  _("verify database files"));
     printHelpLine(_("      --dbpath <dir>      "),
 		  _("use <dir> as the directory for the database"));
     printHelpLine(  "      --root <dir>        ",
@@ -538,9 +581,11 @@ static void printHelp(void) {
 int main(int argc, const char ** argv)
 {
     enum modes bigMode = MODE_UNKNOWN;
-    QVA_t *qva = &rpmQVArgs;
+    QVA_t qva = &rpmQVArgs;
     int arg;
-    int installFlags = 0, uninstallFlags = 0, interfaceFlags = 0;
+    rpmtransFlags transFlags = RPMTRANS_FLAG_NONE;
+    rpmInstallInterfaceFlags installInterfaceFlags = INSTALL_NONE;
+    rpmEraseInterfaceFlags eraseInterfaceFlags = UNINSTALL_NONE;
     int verifyFlags;
     int checksigFlags = 0;
     rpmResignFlags addSign = RESIGN_NEW_SIGNATURE;
@@ -568,6 +613,7 @@ int main(int argc, const char ** argv)
     /* set the defaults for the various command line options */
     allFiles = 0;
     allMatches = 0;
+    applyOnly = 0;
     badReloc = 0;
     excldocs = 0;
     force = 0;
@@ -590,8 +636,19 @@ int main(int argc, const char ** argv)
     noMd5 = 0;
     noOrder = 0;
     noPgp = 0;
+
     noScripts = 0;
+    noPre = 0;
+    noPost = 0;
+    noPreun = 0;
+    noPostun = 0;
+
     noTriggers = 0;
+    noTPrein = 0;
+    noTIn = 0;
+    noTUn = 0;
+    noTPostun = 0;
+
     noUsageMsg = 0;
     oldPackage = 0;
     showPercents = 0;
@@ -673,18 +730,18 @@ int main(int argc, const char ** argv)
 	    break;
 	    
 	  case 'u':
-	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_UNINSTALL)
+	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_ERASE)
 		argerror(_("only one major mode may be specified"));
-	    bigMode = MODE_UNINSTALL;
+	    bigMode = MODE_ERASE;
 	    rpmMessage(RPMMESS_ERROR, _("-u and --uninstall are deprecated and no"
 		    " longer work.\n"));
 	    rpmMessage(RPMMESS_ERROR, _("Use -e or --erase instead.\n"));
 	    exit(EXIT_FAILURE);
 	
 	  case 'e':
-	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_UNINSTALL)
+	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_ERASE)
 		argerror(_("only one major mode may be specified"));
-	    bigMode = MODE_UNINSTALL;
+	    bigMode = MODE_ERASE;
 	    break;
 	
 	  case 'v':
@@ -759,6 +816,12 @@ int main(int argc, const char ** argv)
 	    bigMode = MODE_REBUILDDB;
 	    break;
 
+	  case GETOPT_VERIFYDB:
+	    if (bigMode != MODE_UNKNOWN && bigMode != MODE_VERIFYDB)
+		argerror(_("only one major mode may be specified"));
+	    bigMode = MODE_VERIFYDB;
+	    break;
+
 	  case GETOPT_RELOCATE:
 	    if (*optArg != '/') 
 		argerror(_("relocations must begin with a /"));
@@ -896,7 +959,7 @@ int main(int argc, const char ** argv)
 	argerror(_("--ignoresize may only be specified during package "
 		   "installation"));
 
-    if (allMatches && bigMode != MODE_UNINSTALL)
+    if (allMatches && bigMode != MODE_ERASE)
 	argerror(_("--allmatches may only be specified during package "
 		   "erasure"));
 
@@ -904,18 +967,25 @@ int main(int argc, const char ** argv)
 	argerror(_("--allfiles may only be specified during package "
 		   "installation"));
 
-    if (justdb && bigMode != MODE_INSTALL && bigMode != MODE_UNINSTALL)
+    if (justdb && bigMode != MODE_INSTALL && bigMode != MODE_ERASE)
 	argerror(_("--justdb may only be specified during package "
 		   "installation and erasure"));
 
-    if (bigMode != MODE_INSTALL && bigMode != MODE_UNINSTALL && 
-	bigMode != MODE_VERIFY && noScripts)
-	argerror(_("--noscripts may only be specified during package "
+    if (bigMode != MODE_INSTALL && bigMode != MODE_ERASE && 
+	bigMode != MODE_VERIFY &&
+	(noScripts | noPre | noPost | noPreun | noPostun |
+	 noTriggers | noTPrein | noTIn | noTUn | noTPostun))
+	argerror(_("script disabling options may only be specified during package "
 		   "installation, erasure, and verification"));
 
-    if (bigMode != MODE_INSTALL && bigMode != MODE_UNINSTALL && noTriggers)
-	argerror(_("--notriggers may only be specified during package "
-		   "installation, erasure, and verification"));
+    if (bigMode != MODE_INSTALL && applyOnly)
+	argerror(_("--apply may only be specified during package "
+		   "installation"));
+
+    if (bigMode != MODE_INSTALL && bigMode != MODE_ERASE &&
+	(noTriggers | noTPrein | noTIn | noTUn | noTPostun))
+	argerror(_("trigger disabling options may only be specified during package "
+		   "installation and erasure"));
 
     if (noDeps & (bigMode & ~MODES_FOR_NODEPS))
 	argerror(_("--nodeps may only be specified during package "
@@ -1037,6 +1107,10 @@ int main(int argc, const char ** argv)
 	ec = rpmdbRebuild(rootdir);
 	break;
 
+      case MODE_VERIFYDB:
+	ec = rpmdbVerify(rootdir);
+	break;
+
       case MODE_QUERYTAGS:
 	if (argc != 2)
 	    argerror(_("unexpected arguments to --querytags "));
@@ -1075,19 +1149,36 @@ int main(int argc, const char ** argv)
       case MODE_TARBUILD:
 	break;
 
-      case MODE_UNINSTALL:
+      case MODE_ERASE:
 	if (!poptPeekArg(optCon))
 	    argerror(_("no packages given for uninstall"));
 
-	if (noScripts) uninstallFlags |= RPMTRANS_FLAG_NOSCRIPTS;
-	if (noTriggers) uninstallFlags |= RPMTRANS_FLAG_NOTRIGGERS;
-	if (test) uninstallFlags |= RPMTRANS_FLAG_TEST;
-	if (justdb) uninstallFlags |= RPMTRANS_FLAG_JUSTDB;
-	if (noDeps) interfaceFlags |= UNINSTALL_NODEPS;
-	if (allMatches) interfaceFlags |= UNINSTALL_ALLMATCHES;
+	if (noScripts) transFlags |= (_noTransScripts | _noTransTriggers);
+	if (noPre) transFlags |= RPMTRANS_FLAG_NOPRE;
+	if (noPost) transFlags |= RPMTRANS_FLAG_NOPOST;
+	if (noPreun) transFlags |= RPMTRANS_FLAG_NOPREUN;
+	if (noPostun) transFlags |= RPMTRANS_FLAG_NOPOSTUN;
+
+	if (noTriggers) transFlags |= _noTransTriggers;
+	if (noTPrein) transFlags |= RPMTRANS_FLAG_NOTRIGGERPREIN;
+	if (noTIn) transFlags |= RPMTRANS_FLAG_NOTRIGGERIN;
+	if (noTUn) transFlags |= RPMTRANS_FLAG_NOTRIGGERUN;
+	if (noTPostun) transFlags |= RPMTRANS_FLAG_NOTRIGGERPOSTUN;
+
+	if (test) transFlags |= RPMTRANS_FLAG_TEST;
+	if (justdb) transFlags |= RPMTRANS_FLAG_JUSTDB;
+	if (dirStash) transFlags |= RPMTRANS_FLAG_DIRSTASH;
+	if (rePackage) transFlags |= RPMTRANS_FLAG_REPACKAGE;
+	if (pkgCommit) transFlags |= RPMTRANS_FLAG_PKGCOMMIT;
+	if (pkgUndo) transFlags |= RPMTRANS_FLAG_PKGUNDO;
+	if (tsCommit) transFlags |= RPMTRANS_FLAG_COMMIT;
+	if (tsUndo) transFlags |= RPMTRANS_FLAG_UNDO;
+
+	if (noDeps) eraseInterfaceFlags |= UNINSTALL_NODEPS;
+	if (allMatches) eraseInterfaceFlags |= UNINSTALL_ALLMATCHES;
 
 	ec = rpmErase(rootdir, (const char **)poptGetArgs(optCon), 
-			 uninstallFlags, interfaceFlags);
+			 transFlags, eraseInterfaceFlags);
 	break;
 
       case MODE_INSTALL:
@@ -1106,26 +1197,47 @@ int main(int argc, const char ** argv)
 	if (ignoreOs) probFilter |= RPMPROB_FILTER_IGNOREOS;
 	if (ignoreSize) probFilter |= RPMPROB_FILTER_DISKSPACE;
 
-	if (test) installFlags |= RPMTRANS_FLAG_TEST;
+	if (applyOnly)
+	    transFlags = (_noTransScripts | _noTransTriggers |
+		RPMTRANS_FLAG_APPLYONLY | RPMTRANS_FLAG_PKGCOMMIT);
+
+	if (test) transFlags |= RPMTRANS_FLAG_TEST;
 	/* RPMTRANS_FLAG_BUILD_PROBS */
-	if (noScripts) installFlags |= RPMTRANS_FLAG_NOSCRIPTS;
-	if (justdb) installFlags |= RPMTRANS_FLAG_JUSTDB;
-	if (noTriggers) installFlags |= RPMTRANS_FLAG_NOTRIGGERS;
+
+	if (noScripts) transFlags |= (_noTransScripts | _noTransTriggers);
+	if (noPre) transFlags |= RPMTRANS_FLAG_NOPRE;
+	if (noPost) transFlags |= RPMTRANS_FLAG_NOPOST;
+	if (noPreun) transFlags |= RPMTRANS_FLAG_NOPREUN;
+	if (noPostun) transFlags |= RPMTRANS_FLAG_NOPOSTUN;
+
+	if (noTriggers) transFlags |= RPMTRANS_FLAG_NOTRIGGERS;
+	if (noTPrein) transFlags |= RPMTRANS_FLAG_NOTRIGGERPREIN;
+	if (noTIn) transFlags |= RPMTRANS_FLAG_NOTRIGGERIN;
+	if (noTUn) transFlags |= RPMTRANS_FLAG_NOTRIGGERUN;
+	if (noTPostun) transFlags |= RPMTRANS_FLAG_NOTRIGGERPOSTUN;
+
+	if (justdb) transFlags |= RPMTRANS_FLAG_JUSTDB;
 	if (!incldocs) {
 	    if (excldocs)
-		installFlags |= RPMTRANS_FLAG_NODOCS;
+		transFlags |= RPMTRANS_FLAG_NODOCS;
 	    else if (rpmExpandNumeric("%{_excludedocs}"))
-		installFlags |= RPMTRANS_FLAG_NODOCS;
+		transFlags |= RPMTRANS_FLAG_NODOCS;
 	}
-	if (allFiles) installFlags |= RPMTRANS_FLAG_ALLFILES;
+	if (allFiles) transFlags |= RPMTRANS_FLAG_ALLFILES;
+	if (dirStash) transFlags |= RPMTRANS_FLAG_DIRSTASH;
+	if (rePackage) transFlags |= RPMTRANS_FLAG_REPACKAGE;
+	if (pkgCommit) transFlags |= RPMTRANS_FLAG_PKGCOMMIT;
+	if (pkgUndo) transFlags |= RPMTRANS_FLAG_PKGUNDO;
+	if (tsCommit) transFlags |= RPMTRANS_FLAG_COMMIT;
+	if (tsUndo) transFlags |= RPMTRANS_FLAG_UNDO;
 	/* RPMTRANS_FLAG_KEEPOBSOLETE */
 
-	if (showPercents) interfaceFlags |= INSTALL_PERCENT;
-	if (showHash) interfaceFlags |= INSTALL_HASH;
-	if (noDeps) interfaceFlags |= INSTALL_NODEPS;
-	if (noOrder) interfaceFlags |= INSTALL_NOORDER;
-	if (upgrade) interfaceFlags |= INSTALL_UPGRADE;
-	if (freshen) interfaceFlags |= (INSTALL_UPGRADE|INSTALL_FRESHEN);
+	if (showPercents) installInterfaceFlags |= INSTALL_PERCENT;
+	if (showHash) installInterfaceFlags |= INSTALL_HASH;
+	if (noDeps) installInterfaceFlags |= INSTALL_NODEPS;
+	if (noOrder) installInterfaceFlags |= INSTALL_NOORDER;
+	if (upgrade) installInterfaceFlags |= INSTALL_UPGRADE;
+	if (freshen) installInterfaceFlags |= (INSTALL_UPGRADE|INSTALL_FRESHEN);
 
 	if (!poptPeekArg(optCon))
 	    argerror(_("no packages given for install"));
@@ -1144,7 +1256,8 @@ int main(int argc, const char ** argv)
 	}
 
 	ec += rpmInstall(rootdir, (const char **)poptGetArgs(optCon), 
-			installFlags, interfaceFlags, probFilter, relocations);
+			transFlags, installInterfaceFlags, probFilter,
+			relocations);
 	break;
 
       case MODE_QUERY:
@@ -1163,7 +1276,7 @@ int main(int argc, const char ** argv)
 	break;
 
       case MODE_VERIFY:
-	verifyFlags = (VERIFY_FILES|VERIFY_DEPS|VERIFY_SCRIPT|VERIFY_MD5);
+	verifyFlags = VERIFY_ALL;
 	verifyFlags &= ~qva->qva_flags;
 	if (noDeps)	verifyFlags &= ~VERIFY_DEPS;
 	if (noScripts)	verifyFlags &= ~VERIFY_SCRIPT;
@@ -1184,7 +1297,7 @@ int main(int argc, const char ** argv)
 	break;
     }
 
-    poptFreeContext(optCon);
+    optCon = poptFreeContext(optCon);
     rpmFreeMacros(NULL);
     rpmFreeMacros(&rpmCLIMacroContext);
     rpmFreeRpmrc();
