@@ -14,14 +14,14 @@ int main(int argc, char **argv)
     FD_t gzdi;
     
     setprogname(argv[0]);	/* Retrofit glibc __progname */
-    if (argc == 1) {
+    if (argc == 1)
 	fdi = fdDup(STDIN_FILENO);
-    } else {
+    else
 	fdi = Fopen(argv[1], "r.ufdio");
-    }
 
-    if (fdi == NULL || Ferror(fdi)) {
-	fprintf(stderr, _("cannot open package: %s\n"), Fstrerror(fdi));
+    if (Ferror(fdi)) {
+	fprintf(stderr, "%s: %s: %s\n", argv[0],
+		(argc == 1 ? "<stdin>" : argv[1]), Fstrerror(fdi));
 	exit(EXIT_FAILURE);
     }
     fdo = fdDup(STDOUT_FILENO);
@@ -61,25 +61,9 @@ int main(int argc, char **argv)
 	exit(EXIT_FAILURE);
     }
 
-#ifdef	DYING
-  { char buffer[BUFSIZ];
-    int ct;
-    while ((ct = Fread(buffer, sizeof(buffer[0]), sizeof(buffer), gzdi)) > 0) {
-	Fwrite(buffer, sizeof(buffer[0]), ct, fdo);
-    }
-
-    if (ct < 0) {
-        fprintf (stderr, "rpm2cpio: zlib: %s\n", Fstrerror(gzdi));
-	rc = EXIT_FAILURE;
-    } else {
-	rc = EXIT_SUCCESS;
-    }
-  }
-#else
     rc = ufdCopy(gzdi, fdo);
     rc = (rc <= 0) ? EXIT_FAILURE : EXIT_SUCCESS;
     Fclose(fdo);
-#endif
 
     Fclose(gzdi);	/* XXX gzdi == fdi */
 
