@@ -6,8 +6,12 @@
 #include "rpmio_internal.h"
 #include "debug.h"
 
+/*@-redef@*/
 typedef unsigned int uint32;
 typedef unsigned char byte;
+/*@=redef@*/
+
+/*@access DIGEST_CTX@*/
 
 /**
  * MD5/SHA1 digest private data.
@@ -138,6 +142,12 @@ SHA1Transform(DIGEST_CTX ctx)
 {
     uint32 * in = (uint32 *) ctx->in;
     uint32 A, B, C, D, E;     /* Local vars */
+#ifdef	SHA_DEBUG
+int i;
+#define	DPRINTF(_a)	fprintf _a
+#else
+#define	DPRINTF(_a)
+#endif
 
     /* Set up first buffer and local data buffer */
     A = ctx->digest[0];
@@ -146,89 +156,175 @@ SHA1Transform(DIGEST_CTX ctx)
     D = ctx->digest[3];
     E = ctx->digest[4];
 
+#ifdef	SHA_DEBUG
+for (i = 0; i < 16; i++)
+DPRINTF((stderr, "W[%2d]: %08X\n", i, in[i]));
+#endif
     /* Heavy mangling, in 4 sub-rounds of 20 interations each. */
+#ifdef	SHA_DEBUG
+i = 0;
+#endif
     subRound( A, B, C, D, E, f1, K1, in[ 0] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f1, K1, in[ 1] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f1, K1, in[ 2] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f1, K1, in[ 3] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f1, K1, in[ 4] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f1, K1, in[ 5] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f1, K1, in[ 6] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f1, K1, in[ 7] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f1, K1, in[ 8] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f1, K1, in[ 9] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f1, K1, in[10] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f1, K1, in[11] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f1, K1, in[12] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f1, K1, in[13] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f1, K1, in[14] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f1, K1, in[15] );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f1, K1, expand( in, 16 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f1, K1, expand( in, 17 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f1, K1, expand( in, 18 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f1, K1, expand( in, 19 ) );
 
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f2, K2, expand( in, 20 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f2, K2, expand( in, 21 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f2, K2, expand( in, 22 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f2, K2, expand( in, 23 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f2, K2, expand( in, 24 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f2, K2, expand( in, 25 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f2, K2, expand( in, 26 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f2, K2, expand( in, 27 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f2, K2, expand( in, 28 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f2, K2, expand( in, 29 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f2, K2, expand( in, 30 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f2, K2, expand( in, 31 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f2, K2, expand( in, 32 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f2, K2, expand( in, 33 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f2, K2, expand( in, 34 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f2, K2, expand( in, 35 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f2, K2, expand( in, 36 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f2, K2, expand( in, 37 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f2, K2, expand( in, 38 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f2, K2, expand( in, 39 ) );
 
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f3, K3, expand( in, 40 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f3, K3, expand( in, 41 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f3, K3, expand( in, 42 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f3, K3, expand( in, 43 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f3, K3, expand( in, 44 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f3, K3, expand( in, 45 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f3, K3, expand( in, 46 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f3, K3, expand( in, 47 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f3, K3, expand( in, 48 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f3, K3, expand( in, 49 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f3, K3, expand( in, 50 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f3, K3, expand( in, 51 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f3, K3, expand( in, 52 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f3, K3, expand( in, 53 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f3, K3, expand( in, 54 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f3, K3, expand( in, 55 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f3, K3, expand( in, 56 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f3, K3, expand( in, 57 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f3, K3, expand( in, 58 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f3, K3, expand( in, 59 ) );
 
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f4, K4, expand( in, 60 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f4, K4, expand( in, 61 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f4, K4, expand( in, 62 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f4, K4, expand( in, 63 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f4, K4, expand( in, 64 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f4, K4, expand( in, 65 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f4, K4, expand( in, 66 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f4, K4, expand( in, 67 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f4, K4, expand( in, 68 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f4, K4, expand( in, 69 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f4, K4, expand( in, 70 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f4, K4, expand( in, 71 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f4, K4, expand( in, 72 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f4, K4, expand( in, 73 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f4, K4, expand( in, 74 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, A,B,C,D,E ));
     subRound( A, B, C, D, E, f4, K4, expand( in, 75 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, E,A,B,C,D ));
     subRound( E, A, B, C, D, f4, K4, expand( in, 76 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, D,E,A,B,C ));
     subRound( D, E, A, B, C, f4, K4, expand( in, 77 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, C,D,E,A,B ));
     subRound( C, D, E, A, B, f4, K4, expand( in, 78 ) );
+DPRINTF((stderr, "%5d: %08X %08X %08X %08X %08X\n", i++, B,C,D,E,A ));
     subRound( B, C, D, E, A, f4, K4, expand( in, 79 ) );
 
     /* Build message digest */
@@ -356,7 +452,12 @@ MD5Transform(DIGEST_CTX ctx)
 }
 
 static int _ie = 0x44332211;
-static union _mendian { int i; char b[4]; } *_endian = (union _mendian *)&_ie;
+/*@-redef@*/
+static union _mendian {
+/*@unused@*/ int i;
+    char b[4];
+} *_endian = (union _mendian *)&_ie;
+/*@=redef@*/
 #define	IS_BIG_ENDIAN()		(_endian->b[0] == '\x44')
 #define	IS_LITTLE_ENDIAN()	(_endian->b[0] == '\x11')
 
@@ -365,24 +466,40 @@ static union _mendian { int i; char b[4]; } *_endian = (union _mendian *)&_ie;
  * @param buf		data buffer (uint32 aligned address)
  * @param nbytes	no. bytes of data (multiple of sizeof(uint32))
  */
+/*@-shadow@*/
 static void
 byteReverse(byte *buf, unsigned nbytes)
+	/*@modifies *buf @*/
 {
     unsigned nlongs = nbytes / sizeof(uint32);
     uint32 t;
-    do {
-	t = (uint32) ((unsigned) buf[3] << 8 | buf[2]) << 16 |
-	    ((unsigned) buf[1] << 8 | buf[0]);
-	*(uint32 *) buf = t;
-	buf += 4;
-    } while (--nlongs);
+    if (IS_BIG_ENDIAN()) {
+	do {
+	    t = (uint32) ((unsigned) buf[3] << 8 | buf[2]) << 16 |
+		((unsigned) buf[1] << 8 | buf[0]);
+	    *(uint32 *) buf = t;
+	    buf += 4;
+	} while (--nlongs);
+    } else {
+	do {
+	    t = (uint32) ((unsigned) buf[0] << 8 | buf[1]) << 16 |
+		((unsigned) buf[2] << 8 | buf[3]);
+	    *(uint32 *) buf = t;
+	    buf += 4;
+	} while (--nlongs);
+    }
 }
+/*@=shadow@*/
 
 DIGEST_CTX
 rpmDigestInit(rpmDigestFlags flags)
 {
     DIGEST_CTX ctx = xcalloc(1, sizeof(*ctx));
 
+#ifdef	SHA_DEBUG
+int i;
+DPRINTF((stderr, "*** Init(%x)\n", flags));
+#endif
     ctx->flags = flags;
 
     if (flags & RPMDIGEST_MD5) {
@@ -393,6 +510,8 @@ rpmDigestInit(rpmDigestFlags flags)
 	ctx->digest[1] = 0xefcdab89;
 	ctx->digest[2] = 0x98badcfe;
 	ctx->digest[3] = 0x10325476;
+	/* md5 sums are little endian (no swap) so big endian needs the swap. */
+	ctx->doByteReverse = (IS_BIG_ENDIAN()) ? 1 : 0;
     }
 
     if (flags & RPMDIGEST_SHA1) {
@@ -404,12 +523,16 @@ rpmDigestInit(rpmDigestFlags flags)
 	ctx->digest[ 2 ] = 0x98badcfe;
 	ctx->digest[ 3 ] = 0x10325476;
 	ctx->digest[ 4 ] = 0xc3d2e1f0;
+	/* md5 sums are little endian (no swap) so big endian needs the swap. */
+	ctx->doByteReverse = (IS_BIG_ENDIAN()) ? 0 : 1;
+#ifdef	SHA_DEBUG
+for (i =0; i < 5; i++)
+DPRINTF((stderr, "H[%2d]: %X\n", i, ctx->digest[i]));
+#endif
     }
 
-    /* md5 sums are little endian (no swap) so big endian needs the swap. */
-    ctx->doByteReverse = (IS_BIG_ENDIAN()) ? 1 : 0;
-    if (flags & RPMDIGEST_NATIVE)
-	ctx->doByteReverse = 0;
+    if (flags & RPMDIGEST_REVERSE)
+	ctx->doByteReverse ^= 1;
 
     ctx->bits[0] = 0;
     ctx->bits[1] = 0;
@@ -444,21 +567,25 @@ rpmDigestUpdate(DIGEST_CTX ctx, const void * data, size_t len)
 	memcpy(p, buf, t);
 	if (ctx->doByteReverse)
 	    byteReverse(ctx->in, ctx->datalen);
+	/*@-moduncon@*/
 	ctx->transform(ctx);
+	/*@=moduncon@*/
 	buf += t;
 	len -= t;
     }
 
     /* Process data in ctx->datalen chunks */
     for (; len >= ctx->datalen; buf += ctx->datalen, len -= ctx->datalen) {
-	memcpy(ctx->in, buf, ctx->datalen);
+	memmove(ctx->in, buf, ctx->datalen);
 	if (ctx->doByteReverse)
 	    byteReverse(ctx->in, ctx->datalen);
+	/*@-moduncon@*/
 	ctx->transform(ctx);
+	/*@=moduncon@*/
     }
 
     /* Handle any remaining bytes of data. */
-    memcpy(ctx->in, buf, len);
+    memmove(ctx->in, buf, len);
 }
 
 void
@@ -480,7 +607,9 @@ rpmDigestFinal(/*@only@*/ DIGEST_CTX ctx, /*@out@*/ void ** datap,
 	memset(p, 0, count);
 	if (ctx->doByteReverse)
 	    byteReverse(ctx->in, ctx->datalen);
+	/*@-moduncon@*/
 	ctx->transform(ctx);
+	/*@=moduncon@*/
 	p = ctx->in;
 	count = ctx->datalen;
     }
@@ -489,14 +618,22 @@ rpmDigestFinal(/*@only@*/ DIGEST_CTX ctx, /*@out@*/ void ** datap,
     memset(p, 0, count - sizeof(ctx->bits));
     if (ctx->doByteReverse)
 	byteReverse(ctx->in, ctx->datalen - sizeof(ctx->bits));
-    ((uint32 *) ctx->in)[14] = ctx->bits[0];
-    ((uint32 *) ctx->in)[15] = ctx->bits[1];
-    ctx->transform(ctx);
 
-    /* Return final digest. */
+    if (ctx->flags & (RPMDIGEST_MD5|RPMDIGEST_BCSWAP)) {
+	((uint32 *) ctx->in)[14] = ctx->bits[0];
+	((uint32 *) ctx->in)[15] = ctx->bits[1];
+    } else {
+	((uint32 *) ctx->in)[14] = ctx->bits[1];
+	((uint32 *) ctx->in)[15] = ctx->bits[0];
+    }
+    /*@-moduncon@*/
+    ctx->transform(ctx);
+    /*@=moduncon@*/
+
     if (ctx->doByteReverse)
 	byteReverse((byte *) ctx->digest, ctx->digestlen);
 
+    /* Return final digest. */
     if (!asAscii) {
 	if (lenp) *lenp = ctx->digestlen;
 	if (datap) {
