@@ -11,10 +11,14 @@
 /*@access FD_t@*/
 /*@access urlinfo@*/
 
+/*@-redecl@*/
 extern int _rpmio_debug;
+/*@=redecl@*/
 
 /* =============================================================== */
-static int ftpMkdir(const char * path, /*@unused@*/ mode_t mode) {
+static int ftpMkdir(const char * path, /*@unused@*/ mode_t mode)
+	/*@modifies fileSystem @*/
+{
     int rc;
     if ((rc = ftpCmd("MKD", path, NULL)) != 0)
 	return rc;
@@ -27,28 +31,37 @@ static int ftpMkdir(const char * path, /*@unused@*/ mode_t mode) {
     return rc;
 }
 
-static int ftpChdir(const char * path) {
+static int ftpChdir(const char * path)
+	/*@modifies fileSystem @*/
+{
     return ftpCmd("CWD", path, NULL);
 }
 
-static int ftpRmdir(const char * path) {
+static int ftpRmdir(const char * path)
+	/*@modifies fileSystem @*/
+{
     return ftpCmd("RMD", path, NULL);
 }
 
-static int ftpRename(const char * oldpath, const char * newpath) {
+static int ftpRename(const char * oldpath, const char * newpath)
+	/*@modifies fileSystem @*/
+{
     int rc;
     if ((rc = ftpCmd("RNFR", oldpath, NULL)) != 0)
 	return rc;
     return ftpCmd("RNTO", newpath, NULL);
 }
 
-static int ftpUnlink(const char * path) {
+static int ftpUnlink(const char * path)
+	/*@modifies fileSystem @*/
+{
     return ftpCmd("DELE", path, NULL);
 }
 
 /* =============================================================== */
 /* XXX rebuilddb.c: analogues to mkdir(2)/rmdir(2). */
-int Mkdir (const char *path, mode_t mode) {
+int Mkdir (const char * path, mode_t mode)
+{
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
@@ -70,7 +83,8 @@ int Mkdir (const char *path, mode_t mode) {
     return mkdir(path, mode);
 }
 
-int Chdir (const char *path) {
+int Chdir (const char * path)
+{
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
@@ -92,7 +106,8 @@ int Chdir (const char *path) {
     return chdir(path);
 }
 
-int Rmdir (const char *path) {
+int Rmdir (const char * path)
+{
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
@@ -116,7 +131,8 @@ int Rmdir (const char *path) {
 
 /* XXX rpmdb.c: analogue to rename(2). */
 
-int Rename (const char *oldpath, const char * newpath) {
+int Rename (const char * oldpath, const char * newpath)
+{
     const char *oe = NULL;
     const char *ne = NULL;
     int oldut, newut;
@@ -162,7 +178,8 @@ fprintf(stderr, "*** rename old %*s new %*s\n", (int)(oe - oldpath), oldpath, (i
     return rename(oldpath, newpath);
 }
 
-int Link (const char *oldpath, const char * newpath) {
+int Link (const char * oldpath, const char * newpath)
+{
     const char *oe = NULL;
     const char *ne = NULL;
     int oldut, newut;
@@ -247,6 +264,7 @@ static int   column_ptr [MAXCOLS]; /* Index from 0 to the starting positions of
 
 static int
 vfs_split_text (char *p)
+	/*@modifies *p, columns, column_ptr @*/
 {
     char *original = p;
     int  numcols;
@@ -267,6 +285,7 @@ vfs_split_text (char *p)
 
 static int
 is_num (int idx)
+	/*@*/
 {
     if (!columns [idx] || columns [idx][0] < '0' || columns [idx][0] > '9')
 	return 0;
@@ -274,82 +293,92 @@ is_num (int idx)
 }
 
 static int
-is_dos_date(char *str)
+is_dos_date(/*@null@*/ const char *str)
+	/*@*/
 {
-    if (strlen(str) == 8 && str[2] == str[5] && strchr("\\-/", (int)str[2]) != NULL)
-	return (1);
-
-    return (0);
+    if (str != NULL && strlen(str) == 8 &&
+		str[2] == str[5] && strchr("\\-/", (int)str[2]) != NULL)
+	return 1;
+    return 0;
 }
 
 static int
-is_week (char *str, struct tm *tim)
+is_week (/*@null@*/ const char * str, /*@out@*/ struct tm * tim)
+	/*@modifies *tim @*/
 {
-    static char *week = "SunMonTueWedThuFriSat";
-    char *pos;
+/*@observer@*/ static const char * week = "SunMonTueWedThuFriSat";
+    const char * pos;
 
-    if((pos=strstr(week, str)) != NULL){
-        if(tim != NULL)
+    /*@-observertrans -mayaliasunique@*/
+    if (str != NULL && (pos=strstr(week, str)) != NULL) {
+    /*@=observertrans =mayaliasunique@*/
+        if (tim != NULL)
 	    tim->tm_wday = (pos - week)/3;
-	return (1);
+	return 1;
     }
-    return (0);    
+    return 0;    
 }
 
 static int
-is_month (char *str, struct tm *tim)
+is_month (/*@null@*/ const char * str, /*@out@*/ struct tm * tim)
+	/*@modifies *tim @*/
 {
-    static char *month = "JanFebMarAprMayJunJulAugSepOctNovDec";
-    char *pos;
+/*@observer@*/ static const char * month = "JanFebMarAprMayJunJulAugSepOctNovDec";
+    const char * pos;
     
-    if((pos=strstr(month, str)) != NULL){
-        if(tim != NULL)
+    /*@-observertrans -mayaliasunique@*/
+    if (str != NULL && (pos = strstr(month, str)) != NULL) {
+    /*@=observertrans -mayaliasunique@*/
+        if (tim != NULL)
 	    tim->tm_mon = (pos - month)/3;
-	return (1);
+	return 1;
     }
-    return (0);
+    return 0;
 }
 
 static int
-is_time (char *str, struct tm *tim)
+is_time (/*@null@*/ const char * str, /*@out@*/ struct tm * tim)
+	/*@modifies *tim @*/
 {
-    char *p, *p2;
+    const char * p, * p2;
 
-    if ((p=strchr(str, ':')) && (p2=strrchr(str, ':'))) {
+    if (str != NULL && (p = strchr(str, ':')) && (p2 = strrchr(str, ':'))) {
 	if (p != p2) {
     	    if (sscanf (str, "%2d:%2d:%2d", &tim->tm_hour, &tim->tm_min, &tim->tm_sec) != 3)
-		return (0);
-	}
-	else {
+		return 0;
+	} else {
 	    if (sscanf (str, "%2d:%2d", &tim->tm_hour, &tim->tm_min) != 2)
-		return (0);
+		return 0;
 	}
-    }
-    else 
-        return (0);
+    } else 
+        return 0;
     
-    return (1);
+    return 1;
 }
 
-static int is_year(char *str, struct tm *tim)
+static int is_year(/*@null@*/ const char * str, /*@out@*/ struct tm * tim)
+	/*@modifies *tim @*/
 {
     long year;
 
-    if (strchr(str,':'))
-        return (0);
+    if (str == NULL)
+	return 0;
 
-    if (strlen(str)!=4)
-        return (0);
+    if (strchr(str,':'))
+        return 0;
+
+    if (strlen(str) != 4)
+        return 0;
 
     if (sscanf(str, "%ld", &year) != 1)
-        return (0);
+        return 0;
 
     if (year < 1900 || year > 3000)
-        return (0);
+        return 0;
 
     tim->tm_year = (int) (year - 1900);
 
-    return (1);
+    return 1;
 }
 
 /*
@@ -360,8 +389,9 @@ static int is_year(char *str, struct tm *tim)
 
 static int
 vfs_parse_filetype (char c)
+	/*@*/
 {
-    switch (c){
+    switch (c) {
         case 'd': return S_IFDIR; 
         case 'b': return S_IFBLK;
         case 'c': return S_IFCHR;
@@ -377,37 +407,38 @@ vfs_parse_filetype (char c)
     }
 }
 
-static int vfs_parse_filemode (char *p)
+static int vfs_parse_filemode (const char *p)
+	/*@*/
 {	/* converts rw-rw-rw- into 0666 */
     int res = 0;
-    switch (*(p++)){
+    switch (*(p++)) {
 	case 'r': res |= 0400; break;
 	case '-': break;
 	default: return -1;
     }
-    switch (*(p++)){
+    switch (*(p++)) {
 	case 'w': res |= 0200; break;
 	case '-': break;
 	default: return -1;
     }
-    switch (*(p++)){
+    switch (*(p++)) {
 	case 'x': res |= 0100; break;
 	case 's': res |= 0100 | S_ISUID; break;
 	case 'S': res |= S_ISUID; break;
 	case '-': break;
 	default: return -1;
     }
-    switch (*(p++)){
+    switch (*(p++)) {
 	case 'r': res |= 0040; break;
 	case '-': break;
 	default: return -1;
     }
-    switch (*(p++)){
+    switch (*(p++)) {
 	case 'w': res |= 0020; break;
 	case '-': break;
 	default: return -1;
     }
-    switch (*(p++)){
+    switch (*(p++)) {
 	case 'x': res |= 0010; break;
 	case 's': res |= 0010 | S_ISGID; break;
         case 'l': /* Solaris produces these */
@@ -415,17 +446,17 @@ static int vfs_parse_filemode (char *p)
 	case '-': break;
 	default: return -1;
     }
-    switch (*(p++)){
+    switch (*(p++)) {
 	case 'r': res |= 0004; break;
 	case '-': break;
 	default: return -1;
     }
-    switch (*(p++)){
+    switch (*(p++)) {
 	case 'w': res |= 0002; break;
 	case '-': break;
 	default: return -1;
     }
-    switch (*(p++)){
+    switch (*(p++)) {
 	case 'x': res |= 0001; break;
 	case 't': res |= 0001 | S_ISVTX; break;
 	case 'T': res |= S_ISVTX; break;
@@ -435,7 +466,8 @@ static int vfs_parse_filemode (char *p)
     return res;
 }
 
-static int vfs_parse_filedate(int idx, time_t *t)
+static int vfs_parse_filedate(int idx, /*@out@*/ time_t *t)
+	/*@modifies *t @*/
 {	/* This thing parses from idx in columns[] array */
 
     char *p;
@@ -482,7 +514,8 @@ static int vfs_parse_filedate(int idx, time_t *t)
         if (is_dos_date(p)){
             p[2] = p[5] = '-';
 	    
-	    if(sscanf(p, "%2d-%2d-%2d", &d[0], &d[1], &d[2]) == 3){
+	    memset(d, 0, sizeof(d));
+	    if (sscanf(p, "%2d-%2d-%2d", &d[0], &d[1], &d[2]) == 3){
 	    /*  We expect to get:
 		1. MM-DD-YY
 		2. DD-MM-YY
@@ -540,7 +573,10 @@ static int vfs_parse_filedate(int idx, time_t *t)
 }
 
 static int
-vfs_parse_ls_lga (char *p, struct stat *st, char **filename, char **linkname)
+vfs_parse_ls_lga (char * p, /*@out@*/ struct stat * st,
+		/*@out@*/ const char ** filename,
+		/*@out@*/ const char ** linkname)
+	/*@modifies *st, *filename, *linkname @*/
 {
     int idx, idx2, num_cols;
     int i;
@@ -563,11 +599,13 @@ vfs_parse_ls_lga (char *p, struct stat *st, char **filename, char **linkname)
 	if (strlen (p) <= 8 || p [8] != ']')
 	    goto error;
 	/* Should parse here the Notwell permissions :) */
+	/*@-unrecog@*/
 	if (S_ISDIR (st->st_mode))
 	    st->st_mode |= (S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR | S_IXUSR | S_IXGRP | S_IXOTH);
 	else
 	    st->st_mode |= (S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR);
 	p += 9;
+	/*@=unrecog@*/
     } else {
 	if ((i = vfs_parse_filemode(p)) == -1)
 	    goto error;
@@ -591,7 +629,7 @@ vfs_parse_ls_lga (char *p, struct stat *st, char **filename, char **linkname)
 #ifdef	HACK
 	st->st_uid = finduid (columns [1]);
 #else
-	unameToUid (columns [1], &st->st_uid);
+	(void) unameToUid (columns [1], &st->st_uid);
 #endif
     else
         st->st_uid = (uid_t) atol (columns [1]);
@@ -615,7 +653,7 @@ vfs_parse_ls_lga (char *p, struct stat *st, char **filename, char **linkname)
 #ifdef	HACK
 	    st->st_gid = findgid (columns [2]);
 #else
-	    gnameToGid (columns [1], &st->st_gid);
+	    (void) gnameToGid (columns [1], &st->st_gid);
 #endif
 	idx2 = 3;
     }
@@ -731,7 +769,9 @@ error:
     }
 #endif
 
+    /*@-usereleased@*/
     if (p_copy != p)		/* Carefull! */
+    /*@=usereleased@*/
 	g_free (p_copy);
     return 0;
 }
@@ -749,7 +789,9 @@ static /*@only@*/ char * ftpBuf = NULL;
 #define alloca_strdup(_s)       strcpy(alloca(strlen(_s)+1), (_s))
 
 static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
-	struct stat * st, char * rlbuf, size_t rlbufsiz)
+		/*@out@*/ /*@null@*/ struct stat * st,
+		/*@out@*/ /*@null@*/ char * rlbuf, size_t rlbufsiz)
+	/*@modifies *st, *rlbuf, fileSystem @*/
 {
     FD_t fd;
     const char * path;
@@ -796,6 +838,11 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 
 	/* XXX possibly should do "NLST -lais" to get st_ino/st_blocks also */
 	u->openError = ftpReq(fd, "NLST", "-la");
+
+	if (bn == NULL || nbn <= 0) {
+	    rc = -2;
+	    goto exit;
+	}
 	break;
     }
 
@@ -839,7 +886,8 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 
 	    while (*se && *se != '\n') se++;
 	    if (se > s && se[-1] == '\r') se[-1] = '\0';
-	    if (*se == '\0') break;
+	    if (*se == '\0') 
+		/*@innerbreak@*/ break;
 	    *se++ = '\0';
 
 	    if (!strncmp(s, "total ", sizeof("total ")-1)) continue;
@@ -852,25 +900,28 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
 		    break;
 		case ' ':
 		    if (o || !(n[-3] == ' ' && n[-2] == '-' && n[-1] == '>')) {
-			while (*(++n) == ' ');
+			while (*(++n) == ' ')
+			    {};
 			bingo++;
 			break;
 		    }
-		    for (o = n + 1; *o == ' '; o++);
+		    for (o = n + 1; *o == ' '; o++)
+			{};
 		    n -= 3;
 		    ne = n;
 		    break;
 		default:
 		    break;
 		}
-		if (bingo) break;
+		if (bingo)
+		    /*@innerbreak@*/ break;
 	    }
 
 	    if (nbn != (ne - n))	continue;	/* Same name length? */
 	    if (strncmp(n, bn, nbn))	continue;	/* Same name? */
 
 	    moretodo = 0;
-	    break;
+	    /*@innerbreak@*/ break;
 	}
 
         if (moretodo && se > s) {
@@ -916,16 +967,19 @@ static int ftpNLST(const char * url, ftpSysCall_t ftpSysCall,
     }
 
 exit:
-    ufdClose(fd);
+    (void) ufdClose(fd);
     return rc;
 }
 
-static int ftpStat(const char * path, struct stat *st)
+static int ftpStat(const char * path, /*@out@*/ struct stat *st)
+	/*@modifies *st @*/
 {
     return ftpNLST(path, DO_FTP_STAT, st, NULL, 0);
 }
 
-static int ftpLstat(const char * path, struct stat *st) {
+static int ftpLstat(const char * path, /*@out@*/ struct stat *st)
+	/*@modifies *st @*/
+{
     int rc;
     rc = ftpNLST(path, DO_FTP_LSTAT, st, NULL, 0);
 if (_rpmio_debug)
@@ -933,20 +987,26 @@ fprintf(stderr, "*** ftpLstat(%s) rc %d\n", path, rc);
     return rc;
 }
 
-static int ftpReadlink(const char * path, char * buf, size_t bufsiz) {
+static int ftpReadlink(const char * path, /*@out@*/ char * buf, size_t bufsiz)
+	/*@modifies *buf @*/
+{
     return ftpNLST(path, DO_FTP_READLINK, NULL, buf, bufsiz);
 }
 
 static int ftpGlob(const char * path, int flags,
-		int errfunc(const char * epath, int eerno), glob_t * pglob)
+		int errfunc(const char * epath, int eerno),
+		/*@out@*/ glob_t * pglob)
+	/*@modifies *pglob, fileSystem @*/
 {
     int rc;
 
     if (pglob == NULL)
 	return -2;
     rc = ftpNLST(path, DO_FTP_GLOB, NULL, NULL, 0);
+/*@-castfcnptr@*/
 if (_rpmio_debug)
-fprintf(stderr, "*** ftpGlob(%s,0x%x,%p,%p) ftpNLST rc %d\n", path, (unsigned)flags, errfunc, pglob, rc);
+fprintf(stderr, "*** ftpGlob(%s,0x%x,%p,%p) ftpNLST rc %d\n", path, (unsigned)flags, (void *)errfunc, pglob, rc);
+/*@=castfcnptr@*/
     if (rc)
 	return rc;
     rc = poptParseArgvString(ftpBuf, &pglob->gl_pathc, (const char ***)&pglob->gl_pathv);
@@ -954,14 +1014,19 @@ fprintf(stderr, "*** ftpGlob(%s,0x%x,%p,%p) ftpNLST rc %d\n", path, (unsigned)fl
     return rc;
 }
 
-static void ftpGlobfree(glob_t * pglob) {
+static void ftpGlobfree(glob_t * pglob)
+	/*@modifies *pglob @*/
+{
 if (_rpmio_debug)
 fprintf(stderr, "*** ftpGlobfree(%p)\n", pglob);
-    if (pglob->gl_offs == -1)	/* XXX HACK HACK HACK */
+    if (pglob->gl_offs == -1) {	/* XXX HACK HACK HACK */
 	free((void *)pglob->gl_pathv);
+	pglob->gl_pathv = NULL;
+    }
 }
 
-int Stat(const char * path, struct stat * st) {
+int Stat(const char * path, struct stat * st)
+{
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
@@ -985,7 +1050,8 @@ fprintf(stderr, "*** Stat(%s,%p)\n", path, st);
     return stat(path, st);
 }
 
-int Lstat(const char * path, struct stat * st) {
+int Lstat(const char * path, struct stat * st)
+{
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
@@ -1009,7 +1075,8 @@ fprintf(stderr, "*** Lstat(%s,%p)\n", path, st);
     return lstat(path, st);
 }
 
-int Readlink(const char * path, char * buf, size_t bufsiz) {
+int Readlink(const char * path, char * buf, size_t bufsiz)
+{
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
@@ -1031,7 +1098,8 @@ int Readlink(const char * path, char * buf, size_t bufsiz) {
     return readlink(path, buf, bufsiz);
 }
 
-int Access(const char * path, int amode) {
+int Access(const char * path, int amode)
+{
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
@@ -1059,8 +1127,10 @@ int Glob(const char *path, int flags,
     const char * lpath;
     int ut = urlPath(path, &lpath);
 
+/*@-castfcnptr@*/
 if (_rpmio_debug)
-fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", path, (unsigned)flags, errfunc, pglob);
+fprintf(stderr, "*** Glob(%s,0x%x,%p,%p)\n", path, (unsigned)flags, (void *)errfunc, pglob);
+/*@=castfcnptr@*/
     switch (ut) {
     case URL_IS_FTP:		/* XXX WRONG WRONG WRONG */
 	return ftpGlob(path, flags, errfunc, pglob);
@@ -1112,16 +1182,18 @@ fprintf(stderr, "*** Opendir(%s)\n", path);
     return opendir(path);
 }
 
+/*@+voidabstract@*/
 struct dirent * Readdir(DIR * dir)
 {
 if (_rpmio_debug)
-fprintf(stderr, "*** Readdir(%p)\n", dir);
+fprintf(stderr, "*** Readdir(%p)\n", (void *)dir);
     return readdir(dir);
 }
 
 int Closedir(DIR * dir)
 {
 if (_rpmio_debug)
-fprintf(stderr, "*** Closedir(%p)\n", dir);
+fprintf(stderr, "*** Closedir(%p)\n", (void *)dir);
     return closedir(dir);
 }
+/*@=voidabstract@*/
